@@ -2,17 +2,18 @@
  * \file CassiniSoldner.hpp
  * \brief Header for GeographicLib::CassiniSoldner class
  *
- * Copyright (c) Charles Karney (2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_CASSINISOLDNER_HPP)
-#define GEOGRAPHICLIB_CASSINISOLDNER_HPP "$Id: CassiniSoldner.hpp 6867 2010-09-11 13:04:26Z karney $"
+#define GEOGRAPHICLIB_CASSINISOLDNER_HPP \
+  "$Id: d794ea8a1e64fd9cbb8dcee34755b6dc4fee623a $"
 
-#include "GeographicLib/Geodesic.hpp"
-#include "GeographicLib/GeodesicLine.hpp"
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/Geodesic.hpp>
+#include <GeographicLib/GeodesicLine.hpp>
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -54,25 +55,32 @@ namespace GeographicLib {
    * solving the inverse geodesic problem for (\e lat, \e lon) and its twin
    * obtained by reflection in the meridional plane.  The scale is found by
    * determining where two neighboring geodesics intersecting the central
-   * meridan at \e lat1 and \e lat1 + \e dlat1 intersect and taking the ratio
+   * meridian at \e lat1 and \e lat1 + \e dlat1 intersect and taking the ratio
    * of the reduced lengths for the two geodesics between that point and,
    * respectively, (\e lat1, \e lon1) and (\e lat, \e lon).
+   *
+   * Example of use:
+   * \include example-CassiniSoldner.cpp
+   *
+   * <a href="GeodesicProj.1.html">GeodesicProj</a> is a command-line utility
+   * providing access to the functionality of AzimuthalEquidistant, Gnomonic,
+   * and CassiniSoldner.
    **********************************************************************/
 
-  class CassiniSoldner {
+  class GEOGRAPHIC_EXPORT CassiniSoldner {
   private:
     typedef Math::real real;
-    const Geodesic _earth;
+    Geodesic _earth;
     GeodesicLine _meridian;
     real _sbet0, _cbet0;
-    static const real eps1, eps2;
-    static const unsigned maxit =  10;
+    static const real eps1_;
+    static const real tiny_;
+    static const unsigned maxit_ = 10;
 
-    static inline real sq(real x) throw() { return x * x; }
     // The following private helper functions are copied from Geodesic.
     static inline real AngNormalize(real x) throw() {
       // Place angle in [-180, 180).  Assumes x is in [-540, 540).
-      return x >= 180 ? x - 360 : x < -180 ? x + 360 : x;
+      return x >= 180 ? x - 360 : (x < -180 ? x + 360 : x);
     }
     static inline real AngRound(real x) throw() {
       // The makes the smallest gap in x = 1/16 - nextafter(1/16, 0) = 1/2^57
@@ -100,7 +108,7 @@ namespace GeographicLib {
      *   By default this uses the WGS84 ellipsoid.
      *
      * This constructor makes an "uninitialized" object.  Call Reset to set the
-     * central latitude and longuitude, prior to calling Forward and Reverse.
+     * central latitude and longitude, prior to calling Forward and Reverse.
      **********************************************************************/
     explicit CassiniSoldner(const Geodesic& earth = Geodesic::WGS84) throw()
       : _earth(earth) {}
@@ -214,15 +222,22 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _earth.MajorRadius(); }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value inherited from the Geodesic object used in the constructor.  A
-     *   value of 0 is returned for a sphere (infinite inverse flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the value
+     *   inherited from the Geodesic object used in the constructor.
+     **********************************************************************/
+    Math::real Flattening() const throw() { return _earth.Flattening(); }
+    ///@}
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
      **********************************************************************/
     Math::real InverseFlattening() const throw()
     { return _earth.InverseFlattening(); }
-    ///@}
+    /// \endcond
   };
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_CASSINISOLDNER_HPP
