@@ -2,17 +2,24 @@
  * \file DMS.hpp
  * \brief Header for GeographicLib::DMS class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_DMS_HPP)
-#define GEOGRAPHICLIB_DMS_HPP "$Id: DMS.hpp 6954 2011-02-16 14:36:56Z karney $"
+#define GEOGRAPHICLIB_DMS_HPP "$Id: 67770a78c105495a31a9d3755c811e938729c85a $"
 
-#include "GeographicLib/Constants.hpp"
 #include <sstream>
 #include <iomanip>
+#include <GeographicLib/Constants.hpp>
+#include <GeographicLib/Utility.hpp>
+
+#if defined(_MSC_VER)
+// Squelch warnings about dll vs string
+#pragma warning (push)
+#pragma warning (disable: 4251)
+#endif
 
 namespace GeographicLib {
 
@@ -21,23 +28,29 @@ namespace GeographicLib {
    *
    * Parse a string representing degree, minutes, and seconds and return the
    * angle in degrees and format an angle in degrees as degree, minutes, and
-   * seconds.
+   * seconds.  In addition, handle NANs and infinities on input and output.
+   *
+   * Example of use:
+   * \include example-DMS.cpp
    **********************************************************************/
-  class DMS {
+  class GEOGRAPHIC_EXPORT DMS {
   private:
     typedef Math::real real;
-    static int lookup(const std::string& s, char c) throw() {
-      std::string::size_type r = s.find(toupper(c));
-      return r == std::string::npos ? -1 : int(r);
+    // Replace all occurrences of pat by c
+    static void replace(std::string& s, const std::string& pat, char c) {
+      std::string::size_type p = 0;
+      while (true) {
+        p = s.find(pat, p);
+        if (p == std::string::npos)
+          break;
+        s.replace(p, pat.length(), 1, c);
+      }
     }
-    template<typename T> static std::string str(T x) {
-      std::ostringstream s; s << x; return s.str();
-    }
-    static const std::string hemispheres;
-    static const std::string signs;
-    static const std::string digits;
-    static const std::string dmsindicators;
-    static const std::string components[3];
+    static const std::string hemispheres_;
+    static const std::string signs_;
+    static const std::string digits_;
+    static const std::string dmsindicators_;
+    static const std::string components_[3];
     static Math::real NumMatch(const std::string& s);
     DMS();                      // Disable constructor
 
@@ -101,23 +114,65 @@ namespace GeographicLib {
      * Convert a string in DMS to an angle.
      *
      * @param[in] dms string input.
-     * @param[out] ind a DMS::flag value indicating the presence of a
+     * @param[out] ind a DMS::flag value signaling the presence of a
      *   hemisphere indicator.
      * @return angle (degrees).
      *
-     * Degrees, minutes, and seconds are indicated by the letters d, ', &quot;,
-     * and these components may only be given in this order.  Any (but not all)
-     * components may be omitted.  The last component indicator may be omitted
-     * and is assumed to be tbe next smallest unit (thus 33d10 is interpreted
-     * as 33d10').  The final component may be a decimal fraction but the
-     * non-final components must be integers.  The integer parts of the minutes
-     * and seconds components must be less than 60.  A single leading sign is
-     * permitted.  A hemisphere designator (N, E, W, S) may be added to tbe
-     * beginning or end of the string.  The result is multiplied by the implied
-     * signed of the hemisphere designator (negative for S and W).  In addition
-     * \e ind is set to DMS::LATITUDE if N or S is present, to DMS::LONGITUDE
-     * if E or W is present, and to DMS::NONE otherwise.  Throws an error on a
-     * malformed string.  No check is performed on the range of the result.
+     * Degrees, minutes, and seconds are indicated by the characters d, '
+     * (single quote), &quot; (double quote), and these components may only be
+     * given in this order.  Any (but not all) components may be omitted and
+     * other symbols (e.g., the <sup>o</sup> symbol for degrees and the unicode
+     * prime and double prime symbols for minutes and seconds) may be
+     * substituted.  The last component indicator may be omitted and is assumed
+     * to be the next smallest unit (thus 33d10 is interpreted as 33d10').  The
+     * final component may be a decimal fraction but the non-final components
+     * must be integers.  Instead of using d, ', and &quot; to indicate
+     * degrees, minutes, and seconds, : (colon) may be used to <i>separate</i>
+     * these components (numbers must appear before and after each colon); thus
+     * 50d30'10.3&quot; may be written as 50:30:10.3, 5.5' may be written
+     * 0:5.5, and so on.  The integer parts of the minutes and seconds
+     * components must be less than 60.  A single leading sign is permitted.  A
+     * hemisphere designator (N, E, W, S) may be added to the beginning or end
+     * of the string.  The result is multiplied by the implied sign of the
+     * hemisphere designator (negative for S and W).  In addition \e ind is set
+     * to DMS::LATITUDE if N or S is present, to DMS::LONGITUDE if E or W is
+     * present, and to DMS::NONE otherwise.  Throws an error on a malformed
+     * string.  No check is performed on the range of the result.  Examples of
+     * legal and illegal strings are
+     * - <i>LEGAL</i> (all the entries on each line are equivalent)
+     *   - -20.51125, 20d30'40.5&quot;S, -20d30'40.5, -20d30.675,
+     *     N-20d30'40.5&quot;, -20:30:40.5
+     *   - 4d0'9, 4d9&quot;, 4d9'', 4:0:9, 004:00:09, 4.0025, 4.0025d, 4d0.15,
+     *     04:.15
+     * - <i>ILLEGAL</i> (the exception thrown explains the problem)
+     *   - 4d5&quot;4', 4::5, 4:5:, :4:5, 4d4.5'4&quot;, -N20.5, 1.8e2d, 4:60,
+     *     4d-5'
+     *
+     * <b>NOTE:</b> At present, all the string handling in the C++
+     * implementation %GeographicLib is with 8-bit characters.  The support for
+     * unicode symbols for degrees, minutes, and seconds is therefore via the
+     * <a href="http://en.wikipedia.org/wiki/UTF-8">UTF-8</a> encoding.  (The
+     * Javascript implementation of this class uses unicode natively, of
+     * course.)
+     *
+     * Here is the list of Unicode symbols supported for degrees, minutes,
+     * seconds:
+     * - degrees:
+     *   - d, D lower and upper case letters
+     *   - U+00b0 degree symbol
+     *   - U+00ba masculine ordinal indicator
+     *   - U+2070 superscript zero
+     * - minutes:
+     *   - ' apostrophe
+     *   - U+2032 prime
+     *   - U+00b4 acute accent
+     * - seconds:
+     *   - &quot; quotation mark
+     *   - U+2033 double prime
+     *   - '&nbsp;' any two consecutive symbols for minutes
+     * .
+     * The codes with a leading zero byte, e.g., U+00b0, are accepted in their
+     * UTF-8 coded form 0xc2 0xb0 and as a single byte 0xb0.
      **********************************************************************/
     static Math::real Decode(const std::string& dms, flag& ind);
 
@@ -136,21 +191,38 @@ namespace GeographicLib {
     static Math::real Decode(real d, real m = 0, real s = 0) throw()
     { return d + (m + s/real(60))/real(60); }
 
+    /// \cond SKIP
     /**
+     * <b>DEPRECATED</b> (use Utility::num, instead).
      * Convert a string to a real number.
      *
      * @param[in] str string input.
      * @return decoded number.
      **********************************************************************/
-    static Math::real Decode(const std::string& str);
+    static Math::real Decode(const std::string& str)
+    { return Utility::num<real>(str); }
 
     /**
-     * Convert a pait of strings to latitude and longitude.
+     * <b>DEPRECATED</b> (use Utility::fract, instead).
+     * Convert a string to a real number treating the case where the string is
+     * a simple fraction.
+     *
+     * @param[in] str string input.
+     * @return decoded number.
+     **********************************************************************/
+    static Math::real DecodeFraction(const std::string& str)
+    { return Utility::fract<real>(str); }
+    /// \endcond
+
+    /**
+     * Convert a pair of strings to latitude and longitude.
      *
      * @param[in] dmsa first string.
      * @param[in] dmsb second string.
      * @param[out] lat latitude.
      * @param[out] lon longitude.
+     * @param[in] swaplatlong if true assume longitude is given before latitude
+     *   in the absence of hemisphere designators (default false).
      *
      * By default, the \e lat (resp., \e lon) is assigned to the results of
      * decoding \e dmsa (resp., \e dmsb).  However this is overridden if either
@@ -162,7 +234,7 @@ namespace GeographicLib {
      * range [-180<sup>o</sup>, 180<sup>o</sup>).
      **********************************************************************/
     static void DecodeLatLon(const std::string& dmsa, const std::string& dmsb,
-                             real& lat, real& lon);
+                             real& lat, real& lon, bool swaplatlong = false);
 
     /**
      * Convert a string to an angle in degrees.
@@ -189,7 +261,7 @@ namespace GeographicLib {
     static Math::real DecodeAzimuth(const std::string& azistr);
 
     /**
-     * Convert angle (in degrees) into a DMS string.
+     * Convert angle (in degrees) into a DMS string (using d, ', and &quot;).
      *
      * @param[in] angle input angle (degrees)
      * @param[in] trailing DMS::component value indicating the trailing units
@@ -197,6 +269,8 @@ namespace GeographicLib {
      * @param[in] prec the number of digits after the decimal point for the
      *   trailing component.
      * @param[in] ind DMS::flag value indicated additional formatting.
+     * @param[in] dmssep if non-null, use as the DMS separator character
+     *   (instead of d, ', &quot; delimiters).
      * @return formatted string
      *
      * The interpretation of \e ind is as follows:
@@ -212,41 +286,60 @@ namespace GeographicLib {
      * The integer parts of the minutes and seconds components are always given
      * with 2 digits.
      **********************************************************************/
+    static std::string Encode(real angle, component trailing, unsigned prec,
+                              flag ind, char dmssep);
+
+    /**
+     * Convert angle (in degrees) into a DMS string (using d, ', and &quot;).
+     *
+     * @param[in] angle input angle (degrees)
+     * @param[in] trailing DMS::component value indicating the trailing units
+     *   on the string and this is given as a decimal number if necessary.
+     * @param[in] prec the number of digits after the decimal point for the
+     *   trailing component.
+     * @param[in] ind DMS::flag value indicated additional formatting.
+     * @return formatted string
+     *
+     * <b>COMPATIBILITY NOTE:</b> This function calls
+     * Encode(real, component, unsigned, flag, char) with a 5th
+     * argument of char(0).  At some point,
+     * Encode(real, component, unsigned, flag) and will be withdrawn
+     * and the interface to
+     * Encode(real, component, unsigned, flag, char) changed so that
+     * its 4th and 5th arguments have default values.  This will
+     * preserve source-level compatibility.
+     **********************************************************************/
     static std::string Encode(real angle, component trailing, unsigned prec,
                               flag ind = NONE);
 
     /**
-     * Convert angle into a DMS string selecting the trailing component
-     * based on the precision.
+     * Convert angle into a DMS string (using d, ', and &quot;) selecting the
+     * trailing component based on the precision.
      *
      * @param[in] angle input angle (degrees)
      * @param[in] prec the precision relative to 1 degree.
      * @param[in] ind DMS::flag value indicated additional formatting.
+     * @param[in] dmssep if non-null, use as the DMS separator character
+     *   (instead of d, ', &quot; delimiters).
      * @return formatted string
      *
      * \e prec indicates the precision relative to 1 degree, e.g., \e prec = 3
      * gives a result accurate to 0.1' and \e prec = 4 gives a result accurate
      * to 1&quot;.  \e ind is interpreted as in DMS::Encode with the additional
-     * facility at DMS::NUMBER treats \e angle a number in fixed format with
-     * precision \e prec.
+     * facility that DMS::NUMBER represents \e angle as a number in fixed
+     * format with precision \e prec.
      **********************************************************************/
-    static std::string Encode(real angle, unsigned prec, flag ind = NONE) {
-      if (ind == NUMBER) {
-        if (!Math::isfinite(angle))
-          return angle < 0 ? std::string("-inf") :
-            (angle > 0 ? std::string("inf") : std::string("nan"));
-        std::ostringstream s;
-        s << std::fixed << std::setprecision(prec) << angle;
-        return s.str();
-      } else
-        return Encode(angle,
-                      prec < 2 ? DEGREE : (prec < 4 ? MINUTE : SECOND),
-                      prec < 2 ? prec : (prec < 4 ? prec - 2 : prec - 4),
-                      ind);
+    static std::string Encode(real angle, unsigned prec, flag ind = NONE,
+			      char dmssep = char(0)) {
+      return ind == NUMBER ? Utility::str<real>(angle, int(prec)) :
+        Encode(angle,
+               prec < 2 ? DEGREE : (prec < 4 ? MINUTE : SECOND),
+               prec < 2 ? prec : (prec < 4 ? prec - 2 : prec - 4),
+               ind, dmssep);
     }
 
     /**
-     * Split angle into degrees and mintues
+     * Split angle into degrees and minutes
      *
      * @param[in] ang angle (degrees)
      * @param[out] d degrees (an integer returned as a real)
@@ -257,7 +350,7 @@ namespace GeographicLib {
     }
 
     /**
-     * Split angle into degrees and mintues and seconds.
+     * Split angle into degrees and minutes and seconds.
      *
      * @param[in] ang angle (degrees)
      * @param[out] d degrees (an integer returned as a real)
@@ -273,4 +366,8 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
+#if defined(_MSC_VER)
+#pragma warning (pop)
 #endif
+
+#endif  // GEOGRAPHICLIB_DMS_HPP
