@@ -2,15 +2,16 @@
  * \file EllipticFunction.hpp
  * \brief Header for GeographicLib::EllipticFunction class
  *
- * Copyright (c) Charles Karney (2008, 2009) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_ELLIPTICFUNCTION_HPP)
-#define GEOGRAPHICLIB_ELLIPTICFUNCTION_HPP "$Id: EllipticFunction.hpp 6866 2010-09-11 02:15:29Z karney $"
+#define GEOGRAPHICLIB_ELLIPTICFUNCTION_HPP \
+  "$Id: 30ac447643e48afcaf5ab4671fbf2b235008dabe $"
 
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -18,10 +19,10 @@ namespace GeographicLib {
    * \brief Elliptic functions needed for TransverseMercatorExact
    *
    * This provides the subset of elliptic functions needed for
-   * TransverseMercatorExact.  For a given ellipsoid, only parameters \e
-   * e<sup>2</sup> and 1 - \e e<sup>2</sup> are needed.  This class taken the
-   * parameter as a constructor parameters and caches the values of the
-   * required complete integrals.  A method is provided for Jacobi elliptic
+   * TransverseMercatorExact.  For a given ellipsoid, only parameters
+   * <i>e</i><sup>2</sup> and 1 - <i>e</i><sup>2</sup> are needed.  This class
+   * taken the parameter as a constructor parameters and caches the values of
+   * the required complete integrals.  A method is provided for Jacobi elliptic
    * functions and for the incomplete elliptic integral of the second kind in
    * terms of the amplitude.
    *
@@ -38,16 +39,24 @@ namespace GeographicLib {
    *   78&ndash;90 (1965).
    * .
    * The notation follows Abramowitz and Stegun, Chapters 16 and 17.
+   *
+   * Example of use:
+   * \include example-EllipticFunction.cpp
    **********************************************************************/
-  class EllipticFunction {
+  class GEOGRAPHIC_EXPORT EllipticFunction {
   private:
     typedef Math::real real;
-    static const real tol, tolRF, tolRD, tolRG0, tolJAC, tolJAC1;
-    enum { num = 10 }; // Max depth required for sncndn.  Probably 5 is enough.
+    static const real tol_;
+    static const real tolRF_;
+    static const real tolRD_;
+    static const real tolRG0_;
+    static const real tolJAC_;
+    static const real tolJAC1_;
+    enum { num_ = 10 }; // Max depth required for sncndn.  Probably 5 is enough.
     static real RF(real x, real y, real z) throw();
     static real RD(real x, real y, real z) throw();
     static real RG0(real x, real y) throw();
-    const real _m, _m1;
+    real _m, _m1;
     mutable bool _init;
     mutable real _kc, _ec, _kec;
     bool Init() const throw();
@@ -114,7 +123,7 @@ namespace GeographicLib {
      * @param[in] dn
      * @return int dn(\e w)<sup>2</sup> \e dw (A+S 17.2.10).
      *
-     * Instead of specifying the ampltiude \e phi, we provide \e sn = sin(\e
+     * Instead of specifying the amplitude \e phi, we provide \e sn = sin(\e
      * phi), \e cn = cos(\e phi), \e dn = sqrt(1 - \e m sin<sup>2</sup>(\e
      * phi)).
      **********************************************************************/
@@ -124,4 +133,4 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_ELLIPTICFUNCTION_HPP
