@@ -2,16 +2,17 @@
  * \file GeoCoords.hpp
  * \brief Header for GeographicLib::GeoCoords class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #ifndef GEOGRAPHICLIB_GEOCOORDS_HPP
-#define GEOGRAPHICLIB_GEOCOORDS_HPP "$Id: GeoCoords.hpp 6876 2010-10-18 13:47:55Z karney $"
+#define GEOGRAPHICLIB_GEOCOORDS_HPP \
+  "$Id: e706d3a35c3be0e2beaf39041cac29beb468a5aa $"
 
-#include "GeographicLib/UTMUPS.hpp"
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/UTMUPS.hpp>
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -22,7 +23,7 @@ namespace GeographicLib {
    * constructors or Reset via
    * - latitude and longitude
    * - UTM or UPS coordinates
-   * - a string represention of these or an MGRS coordinate string
+   * - a string representation of these or an MGRS coordinate string
    *
    * The state consists of the latitude and longitude and the supplied UTM or
    * UPS coordinates (possibly derived from the MGRS coordinates).  If latitude
@@ -39,8 +40,14 @@ namespace GeographicLib {
    *
    * Once the input string has been parsed, you can print the result out in any
    * of the formats, decimal degrees, degrees minutes seconds, MGRS, UTM/UPS.
+   *
+   * Example of use:
+   * \include example-GeoCoords.cpp
+   *
+   * <a href="GeoConvert.1.html">GeoConvert</a> is a command-line utility
+   * providing access to the functionality of GeoCoords.
    **********************************************************************/
-  class GeoCoords {
+  class GEOGRAPHIC_EXPORT GeoCoords {
   private:
     typedef Math::real real;
     real _lat, _long, _easting, _northing, _gamma, _k;
@@ -65,8 +72,8 @@ namespace GeographicLib {
      **********************************************************************/
     ///@{
     /**
-     * The default contructor is equivalent to \e latitude = 90<sup>o</sup>, \e
-     * longitude = 0<sup>o</sup>.
+     * The default constructor is equivalent to \e latitude = 90<sup>o</sup>,
+     * \e longitude = 0<sup>o</sup>.
      **********************************************************************/
     GeoCoords() throw()
       // This is the N pole
@@ -85,6 +92,8 @@ namespace GeographicLib {
      *   the position.
      * @param[in] centerp governs the interpretation of MGRS coordinates (see
      *   below).
+     * @param[in] swaplatlong governs the interpretation of geographic
+     *   coordinates (see below).
      *
      * Parse as a string and interpret it as a geographic position.  The input
      * string is broken into space (or comma) separated pieces and Basic
@@ -97,7 +106,8 @@ namespace GeographicLib {
      * - Latitude and Longitude
      *   -  33.44      43.27
      *   -  N33d26.4'  E43d16.2'
-     *   -  43d16'12"E 33d26'24"N
+     *   -  43d16'12&quot;E 33d26'24&quot;N
+     *   -  43:16:12E  33:26:24
      * - MGRS
      *   -  38SLC301
      *   -  38SLC391014
@@ -108,21 +118,27 @@ namespace GeographicLib {
      *   -  897039 3708229 37N
      *
      * Latitude and Longitude parsing.  Latitude precedes longitude, unless a
-     * N, S, E, W hemisphere designator is used on one or both coordinates.
-     * Thus
+     * N, S, E, W hemisphere designator is used on one or both coordinates.  If
+     * \e swaplatlong = true (default is false), then longitude precedes
+     * latitude in the absence of a hemisphere designator.  Thus (with \e
+     * swaplatlong = false)
      * - 40 -75
      * - N40 W75
      * - -75 N40
      * - 75W 40N
      * - E-75 -40S
      * .
-     * are all the same position.  The coodinates may be given in decimal
-     * degrees, degrees and decimal minutes, degrees, minutes, seconds, etc.
-     * Use d, ', and " to make off the degrees, minutes and seconds.  Thus
-     * - 40d30'30"
+     * are all the same position.  The coordinates may be given in
+     * decimal degrees, degrees and decimal minutes, degrees, minutes,
+     * seconds, etc.  Use d, ', and &quot; to mark off the degrees,
+     * minutes and seconds.  Alternatively, use : to separate these
+     * components.  Thus
+     * - 40d30'30&quot;
      * - 40d30'30
      * - 40d30.5'
      * - 40d30.5
+     * - 40:30:30
+     * - 40:30.5
      * - 40.508333333
      * .
      * all specify the same angle.  The leading sign applies to all components
@@ -150,8 +166,9 @@ namespace GeographicLib {
      * - 38SMB4484       = 38N 444000 3684000
      * - 38SMB44148470   = 38N 444140 3684700
      **********************************************************************/
-    explicit GeoCoords(const std::string& s, bool centerp = true)
-    { Reset(s, centerp); }
+    explicit GeoCoords(const std::string& s,
+                       bool centerp = true, bool swaplatlong = false)
+    { Reset(s, centerp, swaplatlong); }
 
     /**
      * Construct from geographic coordinates.
@@ -179,9 +196,10 @@ namespace GeographicLib {
 
     /**
      * Reset the location from a string.  See
-     * GeoCoords(const std::string& s, bool centerp).
+     * GeoCoords(const std::string& s, bool centerp, bool swaplatlong).
      **********************************************************************/
-    void Reset(const std::string& s, bool centerp = true);
+    void Reset(const std::string& s,
+               bool centerp = true, bool swaplatlong = false);
 
     /**
      * Reset the location in terms of geographic coordinates.  See
@@ -309,12 +327,12 @@ namespace GeographicLib {
     Math::real AltNorthing() const throw() { return _alt_northing; }
 
     /**
-     * @return meridian convergence (degrees) for altermate zone.
+     * @return meridian convergence (degrees) for alternate zone.
      **********************************************************************/
     Math::real AltConvergence() const throw() { return _alt_gamma; }
 
     /**
-     * @return scale for altermate zone.
+     * @return scale for alternate zone.
      **********************************************************************/
     Math::real AltScale() const throw() { return _alt_k; }
     ///@}
@@ -327,6 +345,7 @@ namespace GeographicLib {
      * degrees.
      *
      * @param[in] prec precision (relative to about 1m).
+     * @param[in] swaplatlong if true give longitude first (default = false)
      * @return decimal latitude/longitude string representation.
      *
      * Precision specifies accuracy of representation as follows:
@@ -335,13 +354,16 @@ namespace GeographicLib {
      * - prec = 3, 10<sup>-8</sup>d
      * - prec = 9 (max), 10<sup>-14</sup>d
      **********************************************************************/
-    std::string GeoRepresentation(int prec = 0) const;
+    std::string GeoRepresentation(int prec = 0, bool swaplatlong = false) const;
 
     /**
      * String representation with latitude and longitude as degrees, minutes,
      * seconds, and hemisphere.
      *
      * @param[in] prec precision (relative to about 1m)
+     * @param[in] swaplatlong if true give longitude first (default = false)
+     * @param[in] dmssep if non-null, use as the DMS separator character
+     *   (instead of d, ', &quot; delimiters).
      * @return DMS latitude/longitude string representation.
      *
      * Precision specifies accuracy of representation as follows:
@@ -349,12 +371,31 @@ namespace GeographicLib {
      * - prec = -4, 0.1d
      * - prec = -3, 1'
      * - prec = -2, 0.1'
-     * - prec = -1, 1"
-     * - prec = 0, 0.1" (about 3m)
-     * - prec = 1, 0.01"
-     * - prec = 10 (max), 10<sup>-11</sup>"
+     * - prec = -1, 1&quot;
+     * - prec = 0, 0.1&quot; (about 3m)
+     * - prec = 1, 0.01&quot;
+     * - prec = 10 (max), 10<sup>-11</sup>&quot;
      **********************************************************************/
-    std::string DMSRepresentation(int prec = 0) const;
+    std::string DMSRepresentation(int prec, bool swaplatlong, char dmssep)
+      const;
+
+    /**
+     * String representation with latitude and longitude as degrees, minutes,
+     * seconds, and hemisphere.
+     *
+     * @param[in] prec precision (relative to about 1m)
+     * @param[in] swaplatlong if true give longitude first (default = false)
+     * @return DMS latitude/longitude string representation.
+     *
+     * <b>COMPATIBILITY NOTE:</b> This function calls
+     * DMSRepresentation(int, bool, char) const with a 3rd argument of
+     * char(0).  At some point, DMSRepresentation(int, bool) const and
+     * will be withdrawn and the interface to
+     * DMSRepresentation(int, bool, char) const changed so that its
+     * arguments have default values.  This will preserve source-level
+     * compatibility.
+     **********************************************************************/
+    std::string DMSRepresentation(int prec = 0, bool swaplatlong = false) const;
 
     /**
      * MGRS string.
@@ -365,7 +406,7 @@ namespace GeographicLib {
      * This gives the coordinates of the enclosing grid square with size given
      * by the precision.  Thus 38N 444180 3684790 converted to a MGRS
      * coordinate at precision -2 (100m) is 38SMB441847 and not 38SMB442848.
-     * \e prec specifies the precision of the MSGRS string as follows:
+     * \e prec specifies the precision of the MGRS string as follows:
      * - prec = -5 (min), 100km
      * - prec = -4, 10km
      * - prec = -3, 1km
@@ -418,15 +459,24 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return UTMUPS::MajorRadius(); }
 
     /**
-     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     * @return \e f the flattening of the WGS84 ellipsoid.
      *
      * (The WGS84 value is returned because the UTM and UPS projections are
      * based on this ellipsoid.)
      **********************************************************************/
+    Math::real Flattening() const throw() { return UTMUPS::Flattening(); }
+    ///@}
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
     Math::real InverseFlattening() const throw()
     { return UTMUPS::InverseFlattening(); }
-    ///@}
+    /// \endcond
   };
 
 } // namespace GeographicLib
+
 #endif  // GEOGRAPHICLIB_GEOCOORDS_HPP
