@@ -2,21 +2,23 @@
  * \file Geodesic.hpp
  * \brief Header for GeographicLib::Geodesic class
  *
- * Copyright (c) Charles Karney (2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_GEODESIC_HPP)
-#define GEOGRAPHICLIB_GEODESIC_HPP "$Id: Geodesic.hpp 6950 2011-02-11 04:09:24Z karney $"
+#define GEOGRAPHICLIB_GEODESIC_HPP \
+  "$Id: c1b085aadd7b8eabe0f9518b29531a38c152d495 $"
 
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/Constants.hpp>
 
 #if !defined(GEOD_ORD)
 /**
  * The order of the expansions used by Geodesic.
  **********************************************************************/
-#define GEOD_ORD (GEOGRAPHICLIB_PREC == 1 ? 6 : GEOGRAPHICLIB_PREC == 0 ? 3 : 7)
+#define GEOD_ORD \
+  (GEOGRAPHICLIB_PREC == 1 ? 6 : (GEOGRAPHICLIB_PREC == 0 ? 3 : 7))
 #endif
 
 namespace GeographicLib {
@@ -44,7 +46,7 @@ namespace GeographicLib {
    * Given \e lat1, \e lon1, \e lat2, and \e lon2, we can determine \e azi1, \e
    * azi2, and \e s12.  This is the \e inverse geodesic problem, whose solution
    * is given by Geodesic::Inverse.  Usually, the solution to the inverse
-   * problem is unique.  In cases where there are muliple solutions (all with
+   * problem is unique.  In cases where there are multiple solutions (all with
    * the same \e s12, of course), all the solutions can be easily generated
    * once a particular solution is provided.
    *
@@ -64,9 +66,10 @@ namespace GeographicLib {
    *   by \e dazi1 (radians), the the second point is displaced \e m12 \e dazi1
    *   in the direction \e azi2 + 90<sup>o</sup>.  The quantity \e m12 is
    *   called the "reduced length" and is symmetric under interchange of the
-   *   two points.  On a flat surface, we have \e m12 = \e s12.  The ratio \e
-   *   s12/\e m12 gives the azimuthal scale for an azimuthal equidistant
-   *   projection.
+   *   two points.  On a curved surface the reduced length obeys a symmetry
+   *   relation, \e m12 + \e m21 = 0.  On a flat surface, we have \e m12 = \e
+   *   s12.  The ratio <i>s12</i>/\e m12 gives the azimuthal scale for an
+   *   azimuthal equidistant projection.
    * - <i>geodesic scale</i>.  Consider a reference geodesic and a second
    *   geodesic parallel to this one at point 1 and separated by a small
    *   distance \e dt.  The separation of the two geodesics at point 2 is \e
@@ -85,66 +88,84 @@ namespace GeographicLib {
    * Overloaded versions of Geodesic::Direct, Geodesic::ArcDirect, and
    * Geodesic::Inverse allow these quantities to be returned.  In addition
    * there are general functions Geodesic::GenDirect, and Geodesic::GenInverse
-   * which allow an arbitrary set of results to be computed.
+   * which allow an arbitrary set of results to be computed.  The quantities \e
+   * m12, \e M12, \e M21 which all specify the behavior of nearby geodesics
+   * obey addition rules.  Let points 1, 2, and 3 all lie on a single geodesic,
+   * then
+   * - \e m13 = \e m12 \e M23 + \e m23 \e M21
+   * - \e M13 = \e M12 \e M23 - (1 - \e M12 \e M21) \e m23 / \e m12
+   * - \e M31 = \e M32 \e M21 - (1 - \e M23 \e M32) \e m12 / \e m23
    *
-   * Additional functionality if provided by the GeodesicLine class, which
+   * Additional functionality is provided by the GeodesicLine class, which
    * allows a sequence of points along a geodesic to be computed.
    *
-   * The calculations are accurate to better than 15 nm.  See Sec. 9 of
-   * <a href="http://arxiv.org/abs/1102.1215">arXiv:1102.1215</a> for details.
+   * The calculations are accurate to better than 15 nm (15 nanometers).  See
+   * Sec. 9 of
+   * <a href="http://arxiv.org/abs/1102.1215v1">arXiv:1102.1215v1</a>
+   * for details.
    *
    * The algorithms are described in
    * - C. F. F. Karney,
-   *   <a href="http://arxiv.org/abs/1102.1215">Geodesics
+   *   <a href="http://arxiv.org/abs/1102.1215v1">Geodesics
    *   on an ellipsoid of revolution</a>,
    *   Feb. 2011;
    *   preprint
-   *   <a href="http://arxiv.org/abs/1102.1215">arXiv:1102.1215</a>.
+   *   <a href="http://arxiv.org/abs/1102.1215v1">arXiv:1102.1215v1</a>.
+   * - C. F. F. Karney,
+   *   <a href="http://arxiv.org/abs/1109.4448">Algorithms for geodesics</a>,
+   *   Sept. 2011;
+   *   preprint
+   *   <a href="http://arxiv.org/abs/1109.4448">arxiv:1109.4448</a>.
    * .
    * For more information on geodesics see \ref geodesic.
+   *
+   * Example of use:
+   * \include example-Geodesic.cpp
+   *
+   * <a href="Geod.1.html">Geod</a> is a command-line utility providing access
+   * to the functionality of Geodesic and GeodesicLine.
    **********************************************************************/
 
-  class Geodesic {
+  class GEOGRAPHIC_EXPORT Geodesic {
   private:
     typedef Math::real real;
     friend class GeodesicLine;
-    static const int nA1 = GEOD_ORD, nC1 = GEOD_ORD, nC1p = GEOD_ORD,
-      nA2 = GEOD_ORD, nC2 = GEOD_ORD,
-      nA3 = GEOD_ORD, nA3x = nA3,
-      nC3 = GEOD_ORD, nC3x = (nC3 * (nC3 - 1)) / 2,
-      nC4 = GEOD_ORD, nC4x = (nC4 * (nC4 + 1)) / 2;
-    static const unsigned maxit = 50;
+    static const int nA1_ = GEOD_ORD;
+    static const int nC1_ = GEOD_ORD;
+    static const int nC1p_ = GEOD_ORD;
+    static const int nA2_ = GEOD_ORD;
+    static const int nC2_ = GEOD_ORD;
+    static const int nA3_ = GEOD_ORD;
+    static const int nA3x_ = nA3_;
+    static const int nC3_ = GEOD_ORD;
+    static const int nC3x_ = (nC3_ * (nC3_ - 1)) / 2;
+    static const int nC4_ = GEOD_ORD;
+    static const int nC4x_ = (nC4_ * (nC4_ + 1)) / 2;
+    static const unsigned maxit_ = 50;
 
-    static inline real sq(real x) throw() { return x * x; }
-    void Lengths(real eps, real sig12,
-                 real ssig1, real csig1, real ssig2, real csig2,
-                 real cbet1, real cbet2,
-                 real& s12s, real& m12a, real& m0,
-                 bool scalep, real& M12, real& M21,
-                 real tc[], real zc[]) const throw();
-    static real Astroid(real R, real z) throw();
-    real InverseStart(real sbet1, real cbet1, real sbet2, real cbet2,
-                      real lam12,
-                      real& salp1, real& calp1,
-                      real& salp2, real& calp2,
-                      real C1a[], real C2a[]) const throw();
-    real Lambda12(real sbet1, real cbet1, real sbet2, real cbet2,
-                  real salp1, real calp1,
-                  real& salp2, real& calp2, real& sig12,
-                  real& ssig1, real& csig1, real& ssig2, real& csig2,
-                  real& eps, real& domg12, bool diffp, real& dlam12,
-                  real C1a[], real C2a[], real C3a[])
-      const throw();
+    static const real tiny_;
+    static const real tol0_;
+    static const real tol1_;
+    static const real tol2_;
+    static const real xthresh_;
+
+    enum captype {
+      CAP_NONE = 0U,
+      CAP_C1   = 1U<<0,
+      CAP_C1p  = 1U<<1,
+      CAP_C2   = 1U<<2,
+      CAP_C3   = 1U<<3,
+      CAP_C4   = 1U<<4,
+      CAP_ALL  = 0x1FU,
+      OUT_ALL  = 0x7F80U,
+    };
 
-    static const real eps2, tol0, tol1, tol2, xthresh;
-    const real _a, _r, _f, _f1, _e2, _ep2, _n, _b, _c2, _etol2;
-    real _A3x[nA3x], _C3x[nC3x], _C4x[nC4x];
     static real SinCosSeries(bool sinp,
                              real sinx, real cosx, const real c[], int n)
       throw();
     static inline real AngNormalize(real x) throw() {
       // Place angle in [-180, 180).  Assumes x is in [-540, 540).
-      return x >= 180 ? x - 360 : x < -180 ? x + 360 : x;
+      return x >= 180 ? x - 360 : (x < -180 ? x + 360 : x);
     }
     static inline real AngRound(real x) throw() {
       // The makes the smallest gap in x = 1/16 - nextafter(1/16, 0) = 1/2^57
@@ -163,6 +184,30 @@ namespace GeographicLib {
       sinx /= r;
       cosx /= r;
     }
+    static real Astroid(real x, real y) throw();
+
+    real _a, _f, _f1, _e2, _ep2, _n, _b, _c2, _etol2;
+    real _A3x[nA3x_], _C3x[nC3x_], _C4x[nC4x_];
+
+    void Lengths(real eps, real sig12,
+                 real ssig1, real csig1, real ssig2, real csig2,
+                 real cbet1, real cbet2,
+                 real& s12s, real& m12a, real& m0,
+                 bool scalep, real& M12, real& M21,
+                 real C1a[], real C2a[]) const throw();
+    real InverseStart(real sbet1, real cbet1, real sbet2, real cbet2,
+                      real lam12,
+                      real& salp1, real& calp1,
+                      real& salp2, real& calp2,
+                      real C1a[], real C2a[]) const throw();
+    real Lambda12(real sbet1, real cbet1, real sbet2, real cbet2,
+                  real salp1, real calp1,
+                  real& salp2, real& calp2, real& sig12,
+                  real& ssig1, real& csig1, real& ssig2, real& csig2,
+                  real& eps, real& domg12, bool diffp, real& dlam12,
+                  real C1a[], real C2a[], real C3a[])
+      const throw();
+
     // These are Maxima generated functions to provide series approximations to
     // the integrals for the ellipsoidal geodesic.
     static real A1m1f(real eps) throw();
@@ -170,6 +215,7 @@ namespace GeographicLib {
     static void C1pf(real eps, real c[]) throw();
     static real A2m1f(real eps) throw();
     static void C2f(real eps, real c[]) throw();
+
     void A3coeff() throw();
     real A3f(real eps) const throw();
     void C3coeff() throw();
@@ -177,16 +223,6 @@ namespace GeographicLib {
     void C4coeff() throw();
     void C4f(real k2, real c[]) const throw();
 
-    enum captype {
-      CAP_NONE = 0U,
-      CAP_C1   = 1U<<0,
-      CAP_C1p  = 1U<<1,
-      CAP_C2   = 1U<<2,
-      CAP_C3   = 1U<<3,
-      CAP_C4   = 1U<<4,
-      CAP_ALL  = 0x1FU,
-      OUT_ALL  = 0x7F80U,
-    };
   public:
 
     /**
@@ -260,15 +296,15 @@ namespace GeographicLib {
     /**
      * Constructor for a ellipsoid with
      *
-     * @param[in] a equatorial radius (meters)
-     * @param[in] r reciprocal flattening.  Setting \e r = 0 implies \e r = inf
-     *   or flattening = 0 (i.e., a sphere).  Negative \e r indicates a prolate
-     *   ellipsoid.
+     * @param[in] a equatorial radius (meters).
+     * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
+     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      *
      * An exception is thrown if either of the axes of the ellipsoid is
      * non-positive.
      **********************************************************************/
-    Geodesic(real a, real r);
+    Geodesic(real a, real f);
     ///@}
 
     /** \name Direct geodesic problem specified in terms of distance.
@@ -294,6 +330,10 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
      * @return \e a12 arc length of between point 1 and point 2 (degrees).
      *
+     * \e lat1 should be in the range [-90, 90]; \e lon1 and \e azi1 should be
+     * in the range [-180, 360].  The values of \e lon2 and \e azi2 returned
+     * are in the range [-180, 180).
+     *
      * If either point is at a pole, the azimuth is defined by keeping the
      * longitude fixed and writing \e lat = 90 - \e eps or -90 + \e eps and
      * taking the limit \e eps -> 0 from above.  An arc length greater that 180
@@ -403,6 +443,10 @@ namespace GeographicLib {
      *   (dimensionless).
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
      *
+     * \e lat1 should be in the range [-90, 90]; \e lon1 and \e azi1 should be
+     * in the range [-180, 360].  The values of \e lon2 and \e azi2 returned
+     * are in the range [-180, 180).
+     *
      * If either point is at a pole, the azimuth is defined by keeping the
      * longitude fixed and writing \e lat = 90 - \e eps or -90 + \e eps and
      * taking the limit \e eps -> 0 from above.  An arc length greater that 180
@@ -563,7 +607,7 @@ namespace GeographicLib {
      * @param[in] lon2 longitude of point 2 (degrees).
      * @param[out] s12 distance between point 1 and point 2 (meters).
      * @param[out] azi1 azimuth at point 1 (degrees).
-     * @param[out] azi2 (forward) azimuth at point 1 (degrees).
+     * @param[out] azi2 (forward) azimuth at point 2 (degrees).
      * @param[out] m12 reduced length of geodesic (meters).
      * @param[out] M12 geodesic scale of point 2 relative to point 1
      *   (dimensionless).
@@ -572,12 +616,17 @@ namespace GeographicLib {
      * @param[out] S12 area under the geodesic (meters<sup>2</sup>).
      * @return \e a12 arc length of between point 1 and point 2 (degrees).
      *
+     * \e lat1 and \e lat2 should be in the range [-90, 90]; \e lon1 and \e
+     * lon2 should be in the range [-180, 360].  The values of \e azi1 and \e
+     * azi2 returned are in the range [-180, 180).
+     *
      * If either point is at a pole, the azimuth is defined by keeping the
      * longitude fixed and writing \e lat = 90 - \e eps or -90 + \e eps and
      * taking the limit \e eps -> 0 from above.  If the routine fails to
-     * converge, then all the requested outputs are set to Math::NaN().  This
-     * is not expected to happen with ellipsoidal models of the earth; please
-     * report all cases where this occurs.
+     * converge, then all the requested outputs are set to Math::NaN().  (Test
+     * for such results with Math::isnan.)  This is not expected to happen with
+     * ellipsoidal models of the earth; please report all cases where this
+     * occurs.
      *
      * The following functions are overloaded versions of Geodesic::Inverse
      * which omit some of the output parameters.  Note, however, that the arc
@@ -679,7 +728,7 @@ namespace GeographicLib {
      *   specifying which of the following parameters should be set.
      * @param[out] s12 distance between point 1 and point 2 (meters).
      * @param[out] azi1 azimuth at point 1 (degrees).
-     * @param[out] azi2 (forward) azimuth at point 1 (degrees).
+     * @param[out] azi2 (forward) azimuth at point 2 (degrees).
      * @param[out] m12 reduced length of geodesic (meters).
      * @param[out] M12 geodesic scale of point 2 relative to point 1
      *   (dimensionless).
@@ -700,9 +749,9 @@ namespace GeographicLib {
      * The arc length is always computed and returned as the function value.
      **********************************************************************/
     Math::real GenInverse(real lat1, real lon1, real lat2, real lon2,
-                       unsigned outmask,
-                       real& s12, real& azi1, real& azi2,
-                       real& m12, real& M12, real& M21, real& S12)
+                          unsigned outmask,
+                          real& s12, real& azi1, real& azi2,
+                          real& m12, real& M12, real& M21, real& S12)
       const throw();
     ///@}
 
@@ -711,7 +760,7 @@ namespace GeographicLib {
     ///@{
 
     /**
-     * Set up to do a series of ranges.
+     * Set up to compute several points on a singe geodesic.
      *
      * @param[in] lat1 latitude of point 1 (degrees).
      * @param[in] lon1 longitude of point 1 (degrees).
@@ -721,6 +770,9 @@ namespace GeographicLib {
      *   i.e., which quantities can be returned in calls to
      *   GeodesicLib::Position.
      *
+     * \e lat1 should be in the range [-90, 90]; \e lon1 and \e azi1 should be
+     * in the range [-180, 360].
+     *
      * The Geodesic::mask values are
      * - \e caps |= Geodesic::LATITUDE for the latitude \e lat2; this is
      *   added automatically
@@ -759,11 +811,18 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _a; }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value used in the constructor.  A value of 0 is returned for a sphere
-     *   (infinite inverse flattening).
+     * @return \e f the  flattening of the ellipsoid.  This is the
+     *   value used in the constructor.
      **********************************************************************/
-    Math::real InverseFlattening() const throw() { return _r; }
+    Math::real Flattening() const throw() { return _f; }
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw() { return 1/_f; }
+    /// \endcond
 
     /**
      * @return total area of ellipsoid in meters<sup>2</sup>.  The area of a
@@ -781,41 +840,8 @@ namespace GeographicLib {
      **********************************************************************/
     static const Geodesic WGS84;
 
-
-    /** \name Deprecated function.
-     **********************************************************************/
-    ///@{
-    /**
-     * <b>DEPRECATED</b> Perform the direct geodesic calculation.  Given a
-     * latitude, \e lat1, longitude, \e lon1, and azimuth \e azi1 (degrees) for
-     * point 1 and a range, \e s12 (meters) from point 1 to point 2, return the
-     * latitude, \e lat2, longitude, \e lon2, and forward azimuth, \e azi2
-     * (degrees) for point 2 and the reduced length \e m12 (meters).  If either
-     * point is at a pole, the azimuth is defined by keeping the longitude
-     * fixed and writing \e lat = 90 - \e eps or -90 + \e eps and taking the
-     * limit \e eps -> 0 from above.  If \e arcmode (default false) is set to
-     * true, \e s12 is interpreted as the arc length \e a12 (degrees) on the
-     * auxiliary sphere.  An arc length greater that 180 degrees results in a
-     * geodesic which is not a shortest path.  For a prolate ellipsoid, an
-     * additional condition is necessary for a shortest path: the longitudinal
-     * extent must not exceed of 180 degrees.  Returned value is the arc length
-     * \e a12 (degrees) if \e arcmode is false, otherwise it is the distance \e
-     * s12 (meters).
-     **********************************************************************/
-    Math::real Direct(real lat1, real lon1, real azi1, real s12_a12,
-                      real& lat2, real& lon2, real& azi2, real& m12,
-                      bool arcmode) const throw() {
-      if (arcmode) {
-        real a12 = s12_a12, s12;
-        ArcDirect(lat1, lon1, azi1, a12, lat2, lon2, azi2, s12, m12);
-        return s12;
-      } else {
-        real s12 = s12_a12;
-        return Direct(lat1, lon1, azi1, s12, lat2, lon2, azi2, m12);
-      }
-    }
-    ///@}
   };
 
 } // namespace GeographicLib
-#endif
+
+#endif  // GEOGRAPHICLIB_GEODESIC_HPP
