@@ -2,16 +2,17 @@
  * \file GeodesicLine.hpp
  * \brief Header for GeographicLib::GeodesicLine class
  *
- * Copyright (c) Charles Karney (2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_GEODESICLINE_HPP)
-#define GEOGRAPHICLIB_GEODESICLINE_HPP "$Id: GeodesicLine.hpp 6950 2011-02-11 04:09:24Z karney $"
+#define GEOGRAPHICLIB_GEODESICLINE_HPP \
+  "$Id: 4bbc611bc3837d78456bc227e17bea39cb443745 $"
 
-#include "GeographicLib/Constants.hpp"
-#include "GeographicLib/Geodesic.hpp"
+#include <GeographicLib/Constants.hpp>
+#include <GeographicLib/Geodesic.hpp>
 
 namespace GeographicLib {
 
@@ -23,76 +24,57 @@ namespace GeographicLib {
    * azi1 are specified in the constructor.  GeodesicLine.Position returns the
    * location of point 2 a distance \e s12 along the geodesic.  Alternatively
    * GeodesicLine.ArcPosition gives the position of point 2 an arc length \e
-   * a12 along the geodesic.  An example of use of this class is:
-   \code
-   #include <iostream>
-   #include <iomanip>
-   #include <cmath>
-   #include "GeographicLib/Geodesic.hpp"
-   #include "GeographicLib/GeodesicLine.hpp"
-
-   int main() {
-     // Print waypoints between JFK and SIN at
-     // approximately 100km intervals.
-     double
-       lat1 = 40.640, lon1 = -73.779, // JFK
-       lat2 = 1.359, lon2 = 177.486;  // SIN
-     const GeographicLib::Geodesic&
-       g = GeographicLib::Geodesic::WGS84;
-     double azi1, azi2,
-       a12 = g.Inverse(lat1, lon1, lat2, lon2, azi1, azi2);
-     double ds = 100e3;  // Nominal distance between points = 100 km
-     int num = std::ceil(a12/ (90 * ds / 10e6)); // 90 deg = 10e6 m
-     double da = a12/num;              // Arc length between points
-     const GeographicLib::GeodesicLine l(g, lat1, lon1, azi1);
-     std::cout << std::fixed << std::setprecision(3);
-     for (int i = 0; i <= num; ++i) {
-       double lat, lon;
-       l.ArcPosition(i * da, lat, lon);
-       std::cout << lat << " " << lon << "\n";
-     }
-   }
-   \endcode
-   * The default copy constructor and assignment operators work with this
-   * class, so that, for example, the previous example could start
-   \code
-       GeographicLib::GeodesicLine l;
-       l = g.Line(lat1, lon1, azi1);
-   \endcode
-   * Similarly, a vector can be used to hold GeodesicLine objects.
+   * a12 along the geodesic.
    *
-   * The calculations are accurate to better than 15 nm.  See Sec. 9 of
-   * <a href="http://arxiv.org/abs/1102.1215">arXiv:1102.1215</a> for details.
+   * The default copy constructor and assignment operators work with this
+   * class.  Similarly, a vector can be used to hold GeodesicLine objects.
+   *
+   * The calculations are accurate to better than 15 nm (15 nanometers).  See
+   * Sec. 9 of
+   * <a href="http://arxiv.org/abs/1102.1215v1">arXiv:1102.1215v1</a> for
+   * details.
    *
    * The algorithms are described in
    * - C. F. F. Karney,
-   *   <a href="http://arxiv.org/abs/1102.1215">Geodesics
+   *   <a href="http://arxiv.org/abs/1102.1215v1">Geodesics
    *   on an ellipsoid of revolution</a>,
    *   Feb. 2011;
    *   preprint
-   *   <a href="http://arxiv.org/abs/1102.1215">arXiv:1102.1215</a>.
+   *   <a href="http://arxiv.org/abs/1102.1215v1">arXiv:1102.1215v1</a>.
+   * - C. F. F. Karney,
+   *   <a href="http://arxiv.org/abs/1109.4448">Algorithms for geodesics</a>,
+   *   Sept. 2011;
+   *   preprint
+   *   <a href="http://arxiv.org/abs/1109.4448">arxiv:1109.4448</a>.
    * .
    * For more information on geodesics see \ref geodesic.
+   *
+   * Example of use:
+   * \include example-GeodesicLine.cpp
+   *
+   * <a href="Geod.1.html">Geod</a> is a command-line utility providing access
+   * to the functionality of Geodesic and GeodesicLine.
    **********************************************************************/
 
-  class GeodesicLine {
+  class GEOGRAPHIC_EXPORT GeodesicLine {
   private:
     typedef Math::real real;
     friend class Geodesic;
-    static const int nC1 = Geodesic::nC1, nC1p = Geodesic::nC1p,
-      nC2 = Geodesic::nC2, nC3 = Geodesic::nC3, nC4 = Geodesic::nC4;
+    static const int nC1_ = Geodesic::nC1_;
+    static const int nC1p_ = Geodesic::nC1p_;
+    static const int nC2_ = Geodesic::nC2_;
+    static const int nC3_ = Geodesic::nC3_;
+    static const int nC4_ = Geodesic::nC4_;
 
     real _lat1, _lon1, _azi1;
-    real _a, _r, _b, _c2, _f1, _salp0, _calp0, _k2,
+    real _a, _f, _b, _c2, _f1, _salp0, _calp0, _k2,
       _salp1, _calp1, _ssig1, _csig1, _stau1, _ctau1, _somg1, _comg1,
       _A1m1, _A2m1, _A3c, _B11, _B21, _B31, _A4, _B41;
     // index zero elements of _C1a, _C1pa, _C2a, _C3a are unused
-    real _C1a[nC1 + 1], _C1pa[nC1p + 1], _C2a[nC2 + 1], _C3a[nC3],
-      _C4a[nC4];    // all the elements of _C4a are used
-    bool _areap;
+    real _C1a[nC1_ + 1], _C1pa[nC1p_ + 1], _C2a[nC2_ + 1], _C3a[nC3_],
+      _C4a[nC4_];    // all the elements of _C4a are used
     unsigned _caps;
 
-    static inline real sq(real x) throw() { return x * x; }
     enum captype {
       CAP_NONE = Geodesic::CAP_NONE,
       CAP_C1   = Geodesic::CAP_C1,
@@ -106,11 +88,9 @@ namespace GeographicLib {
   public:
 
     /**
-     * Bit masks for what calculations to do.  These masks do double duty.
-     * They signify to the GeodesicLine::GeodesicLine constructor and to
-     * Geodesic::Line what capabilities should be included in the GeodesicLine
-     * object.  They also specify which results to return in the general
-     * routines Geodesic::GenDirect and Geodesic::GenInverse routines.  This is
+     * Bit masks for what calculations to do.  They signify to the
+     * GeodesicLine::GeodesicLine constructor and to Geodesic::Line what
+     * capabilities should be included in the GeodesicLine object.  This is
      * merely a duplication of Geodesic::mask.
      **********************************************************************/
     enum mask {
@@ -176,7 +156,7 @@ namespace GeographicLib {
 
     /**
      * Constructor for a geodesic line staring at latitude \e lat1, longitude
-     * \e lon1, and aziumuth \e azi1 (all in degrees).
+     * \e lon1, and azimuth \e azi1 (all in degrees).
      *
      * @param[in] g A Geodesic object used to compute the necessary information
      *   about the GeodesicLine.
@@ -189,6 +169,9 @@ namespace GeographicLib {
      *   i.e., which quantities can be returned in calls to
      *   GeodesicLib::Position.
      *
+     * \e lat1 should be in the range [-90, 90]; \e lon1 and \e azi1 should be
+     * in the range [-180, 360].
+     *
      * The GeodesicLine::mask values are
      * - \e caps |= GeodesicLine::LATITUDE for the latitude \e lat2; this is
      *   added automatically
@@ -221,7 +204,7 @@ namespace GeographicLib {
      * calculations).  The object can be set with a call to Geodesic::Line.
      * Use Init() to test whether object is still in this uninitialized state.
      **********************************************************************/
-    GeodesicLine() throw() : _caps(0U) {};
+    GeodesicLine() throw() : _caps(0U) {}
     ///@}
 
     /** \name Position in terms of distance
@@ -253,6 +236,8 @@ namespace GeographicLib {
      *   GeodesicLine::AREA.
      * @return \e a12 arc length of between point 1 and point 2 (degrees).
      *
+     * The values of \e lon2 and \e azi2 returned are in the range [-180, 180).
+     *
      * The GeodesicLine object \e must have been constructed with \e caps |=
      * GeodesicLine::DISTANCE_IN; otherwise Math::NaN() is returned and no
      * parameters are set.  Requesting a value which the GeodesicLine object is
@@ -368,6 +353,8 @@ namespace GeographicLib {
      *   that the GeodesicLine object was constructed with \e caps |=
      *   GeodesicLine::AREA.
      *
+     * The values of \e lon2 and \e azi2 returned are in the range [-180, 180).
+     *
      * Requesting a value which the GeodesicLine object is not capable of
      * computing is not an error; the corresponding argument will not be
      * altered.
@@ -530,24 +517,28 @@ namespace GeographicLib {
     /**
      * @return \e lat1 the latitude of point 1 (degrees).
      **********************************************************************/
-    Math::real Latitude() const throw() { return Init() ? _lat1 : 0; }
+    Math::real Latitude() const throw()
+    { return Init() ? _lat1 : Math::NaN<real>(); }
 
     /**
      * @return \e lon1 the longitude of point 1 (degrees).
      **********************************************************************/
-    Math::real Longitude() const throw() { return Init() ? _lon1 : 0; }
+    Math::real Longitude() const throw()
+    { return Init() ? _lon1 : Math::NaN<real>(); }
 
     /**
      * @return \e azi1 the azimuth (degrees) of the geodesic line at point 1.
      **********************************************************************/
-    Math::real Azimuth() const throw() { return Init() ? _azi1 : 0; }
+    Math::real Azimuth() const throw()
+    { return Init() ? _azi1 : Math::NaN<real>(); }
 
     /**
      * @return \e azi0 the azimuth (degrees) of the geodesic line as it crosses
      * the equator in a northward direction.
      **********************************************************************/
     Math::real EquatorialAzimuth() const throw() {
-      return Init() ? atan2(_salp0, _calp0) / Math::degree<real>() : 0;
+      return Init() ?
+        atan2(_salp0, _calp0) / Math::degree<real>() : Math::NaN<real>();
     }
 
     /**
@@ -555,21 +546,32 @@ namespace GeographicLib {
      * crossing and point 1.
      **********************************************************************/
     Math::real EquatorialArc() const throw() {
-      return Init() ? atan2(_ssig1, _csig1) / Math::degree<real>() : 0;
+      return Init() ?
+        atan2(_ssig1, _csig1) / Math::degree<real>() : Math::NaN<real>();
     }
 
     /**
      * @return \e a the equatorial radius of the ellipsoid (meters).  This is
      *   the value inherited from the Geodesic object used in the constructor.
      **********************************************************************/
-    Math::real MajorRadius() const throw() { return Init() ? _a : 0; }
+    Math::real MajorRadius() const throw()
+    { return Init() ? _a : Math::NaN<real>(); }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value inherited from the Geodesic object used in the constructor.  A
-     *   value of 0 is returned for a sphere (infinite inverse flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the value
+     *   inherited from the Geodesic object used in the constructor.
      **********************************************************************/
-    Math::real InverseFlattening() const throw() { return Init() ? _r : 0; }
+    Math::real Flattening() const throw()
+    { return Init() ? _f : Math::NaN<real>(); }
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw()
+    { return Init() ? 1/_f : Math::NaN<real>(); }
+    /// \endcond
 
     /**
      * @return \e caps the computational capabilities that this object was
@@ -587,47 +589,8 @@ namespace GeographicLib {
     }
     ///@}
 
-    /** \name Deprecated Functions
-     **********************************************************************/
-    ///@{
-
-    /**
-     * <b>DEPRECATED</b>.  Return the latitude, \e lat2, longitude, \e lon2,
-     * and forward azimuth, \e azi2 (degrees) of the point 2 which is a
-     * distance, \e s12 (in meters), from point 1.  Also return the reduced
-     * length \e m12 (meters).  \e s12 can be signed.  If \e arcmode (default
-     * false) is set to true, \e s12 is interpreted as the arc length \e a12
-     * (in degrees) on the auxiliary sphere.  Returned value is the arc length
-     * \e a12 (degrees) if \e arcmode is false, otherwise it is the distance \e
-     * s12 (meters).
-     **********************************************************************/
-    Math::real Position(real s12, real& lat2, real& lon2,
-                        real& azi2, real &m12, bool arcmode)
-      const throw() {
-      if (arcmode) {
-        real s12x;
-        ArcPosition(s12, lat2, lon2, azi2, s12x, m12);
-        return s12x;
-      } else
-        return Position(s12, lat2, lon2, azi2, m12);
-    }
-
-    /**
-     * <b>DEPRECATED</b>.  Return the scale of the geodesic line extending an
-     * arc length \e a12 (degrees) from point 1 to point 2.  \e M12 (a number)
-     * measures the convergence of initially parallel geodesics.  It is defined
-     * by the following construction: starting at point 1 proceed at azimuth \e
-     * azi1 + 90<sup>o</sup> a small distance \e dt; turn -90<sup>o</sup> and
-     * proceed a distance \e s12 (\e not the arc length \e a12); the distance
-     * to point 2 is given by \e M12 \e dt.  \e M21 is defined analogously.
-     **********************************************************************/
-    void Scale(real a12, real& M12, real& M21) const throw() {
-      real lat2, lon2, azi2, s12;
-      ArcPosition(a12, lat2, lon2, azi2, s12, M12, M21);
-    }
-    ///@}
-
   };
 
 } // namespace GeographicLib
-#endif
+
+#endif  // GEOGRAPHICLIB_GEODESICLINE_HPP
