@@ -2,18 +2,36 @@
  * \file Geoid.hpp
  * \brief Header for GeographicLib::Geoid class
  *
- * Copyright (c) Charles Karney (2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2012) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_GEOID_HPP)
-#define GEOGRAPHICLIB_GEOID_HPP "$Id: Geoid.hpp 6937 2011-02-01 20:17:13Z karney $"
+#define GEOGRAPHICLIB_GEOID_HPP \
+  "$Id: 4e4eb5941d16ad00416798703d246a6f7ef5fe46 $"
 
-#include "GeographicLib/Constants.hpp"
 #include <string>
 #include <vector>
 #include <fstream>
+#include <GeographicLib/Constants.hpp>
+
+#if defined(_MSC_VER)
+// Squelch warnings about dll vs vector
+#pragma warning (push)
+#pragma warning (disable: 4251)
+#endif
+
+#if !defined(PGM_PIXEL_WIDTH)
+/**
+ * The size of the pixel data in the pgm data files for the geoids.  2
+ * is the standard size corresponding to a maxval 2^16-1.  Setting it
+ * to 4 uses a maxval of 2^32-1 and changes the extension for the data
+ * files from .pgm to .pgm4.  Note that the format of these pgm4 files
+ * is a non-standard extension of the pgm format.
+ **********************************************************************/
+#define PGM_PIXEL_WIDTH 2
+#endif
 
 namespace GeographicLib {
 
@@ -32,7 +50,7 @@ namespace GeographicLib {
    *
    * The geoids are defined in terms of spherical harmonics.  However in order
    * to provide a quick and flexible method of evaluating the geoid heights,
-   * this class evaluates the height by interpolation inot a grid of
+   * this class evaluates the height by interpolation into a grid of
    * precomputed values.
    *
    * See \ref geoid for details of how to install the data sets, the data
@@ -42,28 +60,48 @@ namespace GeographicLib {
    * be calculated.  The gradient is defined as the rate of change of the geoid
    * as a function of position on the ellipsoid.  This uses the parameters for
    * the WGS84 ellipsoid.  The gradient defined in terms of the interpolated
-   * heights.
+   * heights.  As a result of the way that the geoid data is stored, the
+   * calculation of gradients can result in large quantization errors.  This is
+   * particularly acute for fine grids, at high latitudes, and for the easterly
+   * gradient.
    *
    * This class is typically \e not thread safe in that a single instantiation
    * cannot be safely used by multiple threads because of the way the object
    * reads the data set and because it maintains a single-cell cache.  If
    * multiple threads need to calculate geoid heights they should all construct
    * thread-local instantiations.  Alternatively, set the optional \e
-   * threadsafe parameter to true in the constuctor.  This causes the
+   * threadsafe parameter to true in the constructor.  This causes the
    * constructor to read all the data into memory and to turn off the
    * single-cell caching which results in a Geoid object which \e is thread
    * safe.
+   *
+   * Example of use:
+   * \include example-Geoid.cpp
+   *
+   * <a href="GeoidEval.1.html">GeoidEval</a> is a command-line utility
+   * providing access to the functionality of Geoid.
    **********************************************************************/
 
-  class Geoid {
+  class GEOGRAPHIC_EXPORT Geoid {
   private:
     typedef Math::real real;
-    static const unsigned stencilsize = 12;
-    static const unsigned nterms = ((3 + 1) * (3 + 2))/2; // for a cubic fit
-    static const real c0, c0n, c0s;
-    static const real c3[stencilsize * nterms];
-    static const real c3n[stencilsize * nterms];
-    static const real c3s[stencilsize * nterms];
+#if PGM_PIXEL_WIDTH != 4
+    typedef unsigned short pixel_t;
+    static const unsigned pixel_size_ = 2;
+    static const unsigned pixel_max_ = 0xffffu;
+#else
+    typedef unsigned pixel_t;
+    static const unsigned pixel_size_ = 4;
+    static const unsigned pixel_max_ = 0xffffffffu;
+#endif
+    static const unsigned stencilsize_ = 12;
+    static const unsigned nterms_ = ((3 + 1) * (3 + 2))/2; // for a cubic fit
+    static const real c0_;
+    static const real c0n_;
+    static const real c0s_;
+    static const real c3_[stencilsize_ * nterms_];
+    static const real c3n_[stencilsize_ * nterms_];
+    static const real c3s_[stencilsize_ * nterms_];
 
     std::string _name, _dir, _filename;
     const bool _cubic;
@@ -76,21 +114,22 @@ namespace GeographicLib {
     unsigned long long _datastart, _swidth;
     bool _threadsafe;
     // Area cache
-    mutable std::vector< std::vector<unsigned short> > _data;
+    mutable std::vector< std::vector<pixel_t> > _data;
     mutable bool _cache;
     // NE corner and extent of cache
     mutable int _xoffset, _yoffset, _xsize, _ysize;
     // Cell cache
     mutable int _ix, _iy;
     mutable real _v00, _v01, _v10, _v11;
-    mutable real _t[nterms];
+    mutable real _t[nterms_];
     void filepos(int ix, int iy) const {
       _file.seekg(
 #if !(defined(__GNUC__) && __GNUC__ < 4)
                   // g++ 3.x doesn't know about the cast to streamoff.
                   std::ios::streamoff
 #endif
-                  (_datastart + 2ULL * (unsigned(iy)*_swidth + unsigned(ix))));
+                  (_datastart +
+                   pixel_size_ * (unsigned(iy)*_swidth + unsigned(ix))));
     }
     real rawval(int ix, int iy) const {
       if (ix < 0)
@@ -112,7 +151,13 @@ namespace GeographicLib {
           char a, b;
           _file.get(a);
           _file.get(b);
-          return real((unsigned char)(a) * 256u + (unsigned char)(b));
+          unsigned r = ((unsigned char)(a) << 8) | (unsigned char)(b);
+          if (pixel_size_ == 4) {
+            _file.get(a);
+            _file.get(b);
+            r = (r << 16) | ((unsigned char)(a) << 8) | (unsigned char)(b);
+          }
+          return real(r);
         }
         catch (const std::exception& e) {
           // throw GeographicErr("Error reading " + _filename + ": "
@@ -158,7 +203,7 @@ namespace GeographicLib {
      **********************************************************************/
     ///@{
     /**
-     * Construct a Geoid.
+     * Construct a geoid.
      *
      * @param[in] name the name of the geoid.
      * @param[in] path (optional) directory for data file.
@@ -169,15 +214,12 @@ namespace GeographicLib {
      *
      * The data file is formed by appending ".pgm" to the name.  If \e path is
      * specified (and is non-empty), then the file is loaded from directory, \e
-     * path.  Otherwise the path is given by the GEOID_PATH environment
-     * variable.  If that is undefined, a compile-time default path is used
-     * (/usr/local/share/GeographicLib/geoids on non-Windows systems and
-     * C:/cygwin/usr/local/share/GeographicLib/geoids on Windows systems).
-     * This may throw an exception because the file does not exist, is
-     * unreadable, or is corrupt.  If the \e threadsafe parameter is true, the
-     * data set is read into memory (which this may also cause an exception to
-     * be thrown), the data file is closed, and single-cell caching is turned
-     * off; this results in a Geoid object which \e is thread safe.
+     * path.  Otherwise the path is given by DefaultGeoidPath().  This may
+     * throw an exception because the file does not exist, is unreadable, or is
+     * corrupt.  If the \e threadsafe parameter is true, the data set is read
+     * into memory (which this may also cause an exception to be thrown), the
+     * data file is closed, and single-cell caching is turned off; this results
+     * in a Geoid object which \e is thread safe.
      **********************************************************************/
     explicit Geoid(const std::string& name, const std::string& path = "",
                    bool cubic = true, bool threadsafe = false);
@@ -190,10 +232,11 @@ namespace GeographicLib {
      * @param[in] north latitude (degrees) of the north edge of the cached area.
      * @param[in] east longitude (degrees) of the east edge of the cached area.
      *
-     * Cache the data for the specified rectangular area .  \e east is always
-     * interpreted as being east of \e west, if necessary by adding
-     * 360<sup>o</sup> to its value.  This may throw an error because of
-     * insufficent memory or because of an error reading the data from the
+     * Cache the data for the specified "rectangular" area bounded by the
+     * parallels \e south and \e north and the meridians \e west and \e east.
+     * \e east is always interpreted as being east of \e west, if necessary by
+     * adding 360<sup>o</sup> to its value.  This may throw an error because of
+     * insufficient memory or because of an error reading the data from the
      * file.  In this case, you can catch the error and either do nothing (you
      * will have no cache in this case) or try again with a smaller area.  \e
      * south and \e north should be in the range [-90, 90]; \e west and \e east
@@ -206,7 +249,7 @@ namespace GeographicLib {
      * Cache all the data.  On most computers, this is fast for data sets with
      * grid resolution of 5' or coarser.  For a 1' grid, the required RAM is
      * 450MB; a 2.5' grid needs 72MB; and a 5' grid needs 18MB.  This may throw
-     * an error because of insufficent memory or because of an error reading
+     * an error because of insufficient memory or because of an error reading
      * the data from the file.  In this case, you can catch the error and
      * either do nothing (you will have no cache in this case) or try using
      * Geoid::CacheArea on a specific area.  An exception is thrown if this
@@ -233,7 +276,7 @@ namespace GeographicLib {
      * @param[in] lon longitude of the point (degrees).
      * @return geoid height (meters).
      *
-     * The latitude should be in [-90, 90] and longitude should bein
+     * The latitude should be in [-90, 90] and longitude should be in
      * [-180,360].  This may throw an error because of an error reading data
      * from disk.  However, it will not throw if (\e lat, \e lon) is within a
      * successfully cached area.
@@ -255,7 +298,12 @@ namespace GeographicLib {
      * The latitude should be in [-90, 90] and longitude should be in [-180,
      * 360].  This may throw an error because of an error reading data from
      * disk.  However, it will not throw if (\e lat, \e lon) is within a
-     * successfully cached area.
+     * successfully cached area.  As a result of the way that the geoid data is
+     * stored, the calculation of gradients can result in large quantization
+     * errors.  This is particularly acute for fine grids, at high latitudes,
+     * and for the easterly gradient.  If you need to compute the direction of
+     * the acceleration due to gravity accurately, you should use
+     * GravityModel::Gravity.
      **********************************************************************/
     Math::real operator()(real lat, real lon, real& gradn, real& grade) const {
       return height(lat, lon, true, gradn, grade);
@@ -406,20 +454,32 @@ namespace GeographicLib {
     { return Constants::WGS84_a<real>(); }
 
     /**
-     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     * @return \e f the flattening of the WGS84 ellipsoid.
      *
      * (The WGS84 value is returned because the supported geoid models are all
      * based on this ellipsoid.)
      **********************************************************************/
-    Math::real InverseFlattening() const throw()
-    { return Constants::WGS84_r<real>(); }
+    Math::real Flattening() const throw() { return Constants::WGS84_f<real>(); }
     ///@}
 
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw()
+    { return 1/Constants::WGS84_f<real>(); }
+    /// \endcond
+
     /**
      * @return the default path for geoid data files.
      *
-     * This is the value of the environment variable GEOID_PATH, if set,
-     * otherwise, it is a compile-time default.
+     * This is the value of the environment variable GEOID_PATH, if set;
+     * otherwise, it is $GEOGRAPHICLIB_DATA/geoids if the environment variable
+     * GEOGRAPHICLIB_DATA is set; otherwise, it is a compile-time default
+     * (/usr/local/share/GeographicLib/geoids on non-Windows systems and
+     * C:/Documents and Settings/All Users/Application
+     * Data/GeographicLib/geoids on Windows systems).
      **********************************************************************/
     static std::string DefaultGeoidPath();
 
@@ -433,18 +493,12 @@ namespace GeographicLib {
      **********************************************************************/
     static std::string DefaultGeoidName();
 
-    /**
-     * <b>DEPRECATED</b> Return the compile-time default path for the geoid
-     * data files.
-     **********************************************************************/
-    static std::string DefaultPath();
-
-    /**
-     * <b>DEPRECATED</b> Return the value of the environment variable
-     * GEOID_PATH.
-     **********************************************************************/
-    static std::string GeoidPath();
   };
 
 } // namespace GeographicLib
+
+#if defined(_MSC_VER)
+#pragma warning (pop)
 #endif
+
+#endif  // GEOGRAPHICLIB_GEOID_HPP
