@@ -3,15 +3,16 @@
  * \brief Header for GeographicLib::LambertConformalConic class
  *
  * Copyright (c) Charles Karney (2010, 2011) <charles@karney.com> and licensed
- * under the LGPL.  For more information, see
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_LAMBERTCONFORMALCONIC_HPP)
-#define GEOGRAPHICLIB_LAMBERTCONFORMALCONIC_HPP "$Id: LambertConformalConic.hpp 6937 2011-02-01 20:17:13Z karney $"
+#define GEOGRAPHICLIB_LAMBERTCONFORMALCONIC_HPP \
+  "$Id: 9aef04f77098543818681966f13ef97ea47dedb4 $"
 
-#include "GeographicLib/Constants.hpp"
 #include <algorithm>
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -28,9 +29,9 @@ namespace GeographicLib {
    * differences have been used to transform the expressions into ones which
    * may be evaluated accurately and that Newton's method is used to invert the
    * projection.  In this implementation, the projection correctly becomes the
-   * Mercator projection or the polar sterographic projection when the standard
-   * latitude is the equator or a pole.  The accuracy of the projections is
-   * about 10 nm.
+   * Mercator projection or the polar stereographic projection when the
+   * standard latitude is the equator or a pole.  The accuracy of the
+   * projections is about 10 nm (10 nanometers).
    *
    * The ellipsoid parameters, the standard parallels, and the scale on the
    * standard parallels are set in the constructor.  Internally, the case with
@@ -49,44 +50,23 @@ namespace GeographicLib {
    * calling function.  For example the Pennsylvania South state coordinate
    * system (<a href="http://www.spatialreference.org/ref/epsg/3364/">
    * EPSG:3364</a>) is obtained by:
-   \code
-   const double
-     a = GeographicLib::Constants::WGS84_a<double>(),
-     r = 298.257222101,                        // GRS80
-     lat1 = 39 + 56/60.0, lat1 = 40 + 58/60.0, // standard parallels
-     k1 = 1,                                   // scale
-     lat0 = 39 + 20/60.0, lon0 = 75 + 45/60.0, // origin
-     fe = 600000, fn = 0;                      // false easting and northing
-   // Set up basic projection
-   const GeographicLib::LambertConformalConic PASouth(a, r, lat1, lat2, k1);
-   double x0, y0;
-   {
-     // Transform origin point
-     PASouth.Forward(lon0, lat0, lon0, x0, y0);
-     x0 -= fe; y0 -= fn;         // Combine result with false origin
-   }
-   double lat, lon, x, y;
-   // Sample conversion from geodetic to PASouth grid
-   std::cin >> lat >> lon;
-   PASouth.Forward(lon0, lat, lon, x, y);
-   x -= x0; y -= y0;
-   std::cout << x << " " << y << "\n";
-   // Sample conversion from PASouth grid to geodetic
-   std::cin >> x >> y;
-   x += x0; y += y0;
-   PASouth.Reverse(lon0, x, y, lat, lon);
-   std::cout << lat << " " << lon << "\n";
-   \endcode
+   * \include example-LambertConformalConic.cpp
+   *
+   * <a href="ConicProj.1.html">ConicProj</a> is a command-line utility
+   * providing access to the functionality of LambertConformalConic and
+   * AlbersEqualArea.
    **********************************************************************/
-  class LambertConformalConic {
+  class GEOGRAPHIC_EXPORT LambertConformalConic {
   private:
     typedef Math::real real;
-    const real _a, _r, _f, _fm, _e2, _e, _e2m;
+    real _a, _f, _fm, _e2, _e, _e2m;
     real _sign, _n, _nc, _t0nm1, _scale, _lat0, _k0;
     real _scbet0, _tchi0, _scchi0, _psi0, _nrho0;
-    static const real eps, epsx, tol, ahypover;
-    static const int numit = 5;
-    static inline real sq(real x) throw() { return x * x; }
+    static const real eps_;
+    static const real epsx_;
+    static const real tol_;
+    static const real ahypover_;
+    static const int numit_ = 5;
     static inline real hyp(real x) throw() { return Math::hypot(real(1), x); }
     // e * atanh(e * x) = log( ((1 + e*x)/(1 - e*x))^(e/2) ) if f >= 0
     // - sqrt(-e2) * atan( sqrt(-e2) * x)                    if f < 0
@@ -98,7 +78,7 @@ namespace GeographicLib {
     // See: W. M. Kahan and R. J. Fateman,
     // Symbolic computation of divided differences,
     // SIGSAM Bull. 33(3), 7-28 (1999)
-    // http://doi.acm.org/10.1145/334714.334716
+    // http://dx.doi.org/10.1145/334714.334716
     // http://www.cs.berkeley.edu/~fateman/papers/divdiff.pdf
     //
     // General rules
@@ -116,7 +96,7 @@ namespace GeographicLib {
     static inline real Dsn(real x, real y, real sx, real sy) throw() {
       // sx = x/hyp(x)
       real t = x * y;
-      return t > 0 ? (x + y) * sq( (sx * sy)/t ) / (sx + sy) :
+      return t > 0 ? (x + y) * Math::sq( (sx * sy)/t ) / (sx + sy) :
         (x - y != 0 ? (sx - sy) / (x - y) : 1);
     }
     // Dlog1p(x,y) = log1p((x-y)/(1+y)/(x-y)
@@ -136,9 +116,9 @@ namespace GeographicLib {
     static inline real Dsinh(real x, real y, real sx, real sy, real cx, real cy)
       // sx = sinh(x), cx = cosh(x)
       throw() {
-      // real t = (x  - y)/2, c = sqrt((1 + cx) * (1 + cy));
+      // real t = (x - y)/2, c = sqrt((1 + cx) * (1 + cy));
       // return (t != 0 ? sinh(t)/t : real(1)) * (c + sx * sy / c) /2;
-      real t = (x  - y)/2;
+      real t = (x - y)/2;
       return (t != 0 ? sinh(t)/t : real(1)) * sqrt((sx * sy + cx * cy + 1) /2);
     }
     // Dasinh(x,y) = asinh((x-y)*(x+y)/(x*sqrt(1+y^2)+y*sqrt(1+x^2)))/(x-y)
@@ -161,25 +141,25 @@ namespace GeographicLib {
     /**
      * Constructor with a single standard parallel.
      *
-     * @param[in] a equatorial radius of ellipsoid (meters)
-     * @param[in] r reciprocal flattening of ellipsoid.  Setting \e r = 0
-     *   implies \e r = inf or flattening = 0 (i.e., a sphere).  Negative \e r
-     *   indicates a prolate ellipsoid.
+     * @param[in] a equatorial radius of ellipsoid (meters).
+     * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
+     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      * @param[in] stdlat standard parallel (degrees), the circle of tangency.
      * @param[in] k0 scale on the standard parallel.
      *
      * An exception is thrown if \e a or \e k0 is not positive or if \e stdlat
      * is not in the range [-90, 90].
      **********************************************************************/
-    LambertConformalConic(real a, real r, real stdlat, real k0);
+    LambertConformalConic(real a, real f, real stdlat, real k0);
 
     /**
      * Constructor with two standard parallels.
      *
-     * @param[in] a equatorial radius of ellipsoid (meters)
-     * @param[in] r reciprocal flattening of ellipsoid.  Setting \e r = 0
-     *   implies \e r = inf or flattening = 0 (i.e., a sphere).  Negative \e r
-     *   indicates a prolate ellipsoid.
+     * @param[in] a equatorial radius of ellipsoid (meters).
+     * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
+     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      * @param[in] stdlat1 first standard parallel (degrees).
      * @param[in] stdlat2 second standard parallel (degrees).
      * @param[in] k1 scale on the standard parallels.
@@ -189,15 +169,15 @@ namespace GeographicLib {
      * stdlat1 or \e stdlat2 is a pole, then an exception is thrown if \e
      * stdlat1 is not equal \e stdlat2.
      **********************************************************************/
-    LambertConformalConic(real a, real r, real stdlat1, real stdlat2, real k1);
+    LambertConformalConic(real a, real f, real stdlat1, real stdlat2, real k1);
 
     /**
      * Constructor with two standard parallels specified by sines and cosines.
      *
-     * @param[in] a equatorial radius of ellipsoid (meters)
-     * @param[in] r reciprocal flattening of ellipsoid.  Setting \e r = 0
-     *   implies \e r = inf or flattening = 0 (i.e., a sphere).  Negative \e r
-     *   indicates a prolate ellipsoid.
+     * @param[in] a equatorial radius of ellipsoid (meters).
+     * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
+     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      * @param[in] sinlat1 sine of first standard parallel.
      * @param[in] coslat1 cosine of first standard parallel.
      * @param[in] sinlat2 sine of second standard parallel.
@@ -209,12 +189,12 @@ namespace GeographicLib {
      * latitude.  In the case where \e lat1 and \e lat2 are different, the
      * errors in this routines are as follows: if \e dlat = abs(\e lat2 - \e
      * lat1) <= 160<sup>o</sup> and max(abs(\e lat1), abs(\e lat2)) <= 90 -
-     * min(0.0002, 2.2e-6(180 - \e dlat), 6e-8\e dlat<sup>2</sup>) (in
+     * min(0.0002, 2.2e-6(180 - \e dlat), 6e-8 <i>dlat</i><sup>2</sup>) (in
      * degrees), then the error in the latitude of origin is less than
      * 4.5e-14<sup>o</sup> and the relative error in the scale is less than
      * 7e-15.
      **********************************************************************/
-    LambertConformalConic(real a, real r,
+    LambertConformalConic(real a, real f,
                           real sinlat1, real coslat1,
                           real sinlat2, real coslat2,
                           real k1);
@@ -245,10 +225,11 @@ namespace GeographicLib {
      * The latitude origin is given by LambertConformalConic::LatitudeOrigin().
      * No false easting or northing is added and \e lat should be in the range
      * [-90, 90]; \e lon and \e lon0 should be in the range [-180, 360].  The
-     * error in the projection is less than about 10 nm (true distance) and the
-     * errors in the meridian convergence and scale are consistent with this.
-     * The values of \e x and \e y returned for points which project to
-     * infinity (i.e., one or both of the poles) will be large but finite.
+     * error in the projection is less than about 10 nm (10 nanometers), true
+     * distance, and the errors in the meridian convergence and scale are
+     * consistent with this.  The values of \e x and \e y returned for points
+     * which project to infinity (i.e., one or both of the poles) will be large
+     * but finite.
      **********************************************************************/
     void Forward(real lon0, real lat, real lon,
                  real& x, real& y, real& gamma, real& k) const throw();
@@ -267,9 +248,9 @@ namespace GeographicLib {
      * The latitude origin is given by LambertConformalConic::LatitudeOrigin().
      * No false easting or northing is added.  \e lon0 should be in the range
      * [-180, 360].  The value of \e lon returned is in the range [-180, 180).
-     * The error in the projection is less than about 10 nm (true distance) and
-     * the errors in the meridian convergence and scale are consistent with
-     * this.
+     * The error in the projection is less than about 10 nm (10 nanometers),
+     * true distance, and the errors in the meridian convergence and scale are
+     * consistent with this.
      **********************************************************************/
     void Reverse(real lon0, real x, real y,
                  real& lat, real& lon, real& gamma, real& k) const throw();
@@ -304,11 +285,18 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _a; }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value used in the constructor.  A value of 0 is returned for a sphere
-     *   (infinite inverse flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the
+     *   value used in the constructor.
      **********************************************************************/
-    Math::real InverseFlattening() const throw() { return _r; }
+    Math::real Flattening() const throw() { return _f; }
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw() { return 1/_f; }
+    /// \endcond
 
     /**
      * @return latitude of the origin for the projection (degrees).
@@ -336,4 +324,4 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_LAMBERTCONFORMALCONIC_HPP
