@@ -2,16 +2,17 @@
  * \file LocalCartesian.hpp
  * \brief Header for GeographicLib::LocalCartesian class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_LOCALCARTESIAN_HPP)
-#define GEOGRAPHICLIB_LOCALCARTESIAN_HPP "$Id: LocalCartesian.hpp 6952 2011-02-14 20:26:44Z karney $"
+#define GEOGRAPHICLIB_LOCALCARTESIAN_HPP \
+  "$Id: 133014a30695adf3bdea9f4b52613fab458c505a $"
 
-#include "GeographicLib/Geocentric.hpp"
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/Geocentric.hpp>
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -27,20 +28,27 @@ namespace GeographicLib {
    *
    * The conversions all take place via geocentric coordinates using a
    * Geocentric object (by default Geocentric::WGS84).
+   *
+   * Example of use:
+   * \include example-LocalCartesian.cpp
+   *
+   * <a href="CartConvert.1.html">CartConvert</a> is a command-line utility
+   * providing access to the functionality of Geocentric and LocalCartesian.
    **********************************************************************/
 
-  class LocalCartesian {
+  class GEOGRAPHIC_EXPORT LocalCartesian {
   private:
     typedef Math::real real;
-    static const size_t dim = 3, dim2 = dim * dim;
-    const Geocentric _earth;
+    static const size_t dim_ = 3;
+    static const size_t dim2_ = dim_ * dim_;
+    Geocentric _earth;
     real _lat0, _lon0, _h0;
-    real _x0, _y0, _z0, _r[dim2];
+    real _x0, _y0, _z0, _r[dim2_];
     void IntForward(real lat, real lon, real h, real& x, real& y, real& z,
-                    real M[dim2]) const throw();
+                    real M[dim2_]) const throw();
     void IntReverse(real x, real y, real z, real& lat, real& lon, real& h,
-                    real M[dim2]) const throw();
-    void MatrixMultiply(real M[dim2]) const throw();
+                    real M[dim2_]) const throw();
+    void MatrixMultiply(real M[dim2_]) const throw();
   public:
 
     /**
@@ -111,17 +119,26 @@ namespace GeographicLib {
      * @param[out] M if the length of the vector is 9, fill with the rotation
      *   matrix in row-major order.
      *
-     * Pre-multiplying a unit vector in local cartesian coordinates at (lat,
-     * lon, h) by \e M transforms the vector to local cartesian coordinates at
-     * (lat0, lon0, h0).
+     * Let \e v be a unit vector located at (\e lat, \e lon, \e h).  We can
+     * express \e v as \e column vectors in one of two ways
+     * - in east, north, up coordinates (where the components are relative to a
+     *   local coordinate system at (\e lat, \e lon, \e h)); call this
+     *   representation \e v1.
+     * - in \e x, \e y, \e z coordinates (where the components are relative to
+     *   the local coordinate system at (\e lat0, \e lon0, \e h0)); call this
+     *   representation \e v0.
+     * .
+     * Then we have \e v0 = \e M . \e v1.
      **********************************************************************/
     void Forward(real lat, real lon, real h, real& x, real& y, real& z,
                  std::vector<real>& M)
       const throw()  {
-      real t[dim2];
-      IntForward(lat, lon, h, x, y, z, t);
-      if (M.end() == M.begin() + dim2)
-        copy(t, t + dim2, M.begin());
+      if (M.end() == M.begin() + dim2_) {
+        real t[dim2_];
+        IntForward(lat, lon, h, x, y, z, t);
+        copy(t, t + dim2_, M.begin());
+      } else
+        IntForward(lat, lon, h, x, y, z, NULL);
     }
 
     /**
@@ -154,17 +171,27 @@ namespace GeographicLib {
      * @param[out] M if the length of the vector is 9, fill with the rotation
      *   matrix in row-major order.
      *
-     * Pre-multiplying a unit vector in local cartesian coordinates at (lat0,
-     * lon0, h0) by the transpose of \e M transforms the vector to local
-     * cartesian coordinates at (lat, lon, h).
+     * Let \e v be a unit vector located at (\e lat, \e lon, \e h).  We can
+     * express \e v as \e column vectors in one of two ways
+     * - in east, north, up coordinates (where the components are relative to a
+     *   local coordinate system at (\e lat, \e lon, \e h)); call this
+     *   representation \e v1.
+     * - in \e x, \e y, \e z coordinates (where the components are relative to
+     *   the local coordinate system at (\e lat0, \e lon0, \e h0)); call this
+     *   representation \e v0.
+     * .
+     * Then we have \e v1 = \e M^T . \e v0, where \e M^T is the transpose of \e
+     * M.
      **********************************************************************/
     void Reverse(real x, real y, real z, real& lat, real& lon, real& h,
                  std::vector<real>& M)
       const throw() {
-      real t[dim2];
-      IntReverse(x, y, z, lat, lon, h, t);
-      if (M.end() == M.begin() + dim2)
-        copy(t, t + dim2, M.begin());
+      if (M.end() == M.begin() + dim2_) {
+        real t[dim2_];
+        IntReverse(x, y, z, lat, lon, h, t);
+        copy(t, t + dim2_, M.begin());
+      } else
+        IntReverse(x, y, z, lat, lon, h, NULL);
     }
 
     /** \name Inspector functions
@@ -193,16 +220,22 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _earth.MajorRadius(); }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value of \e r inherited from the Geocentric object used in the
-     *   constructor.  A value of 0 is returned for a sphere (infinite inverse
-     *   flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the value
+     *   inherited from the Geocentric object used in the constructor.
+     **********************************************************************/
+    Math::real Flattening() const throw() { return _earth.Flattening(); }
+    ///@}
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
      **********************************************************************/
     Math::real InverseFlattening() const throw()
     { return _earth.InverseFlattening(); }
-    ///@}
+    /// \endcond
   };
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_LOCALCARTESIAN_HPP
