@@ -2,17 +2,23 @@
  * \file MGRS.hpp
  * \brief Header for GeographicLib::MGRS class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_MGRS_HPP)
-#define GEOGRAPHICLIB_MGRS_HPP "$Id: MGRS.hpp 6911 2010-12-09 23:13:55Z karney $"
+#define GEOGRAPHICLIB_MGRS_HPP "$Id: 80e08da6eca9d9cf92c5adad148c64302df2573d $"
 
-#include "GeographicLib/Constants.hpp"
-#include "GeographicLib/UTMUPS.hpp"
 #include <sstream>
+#include <GeographicLib/Constants.hpp>
+#include <GeographicLib/UTMUPS.hpp>
+
+#if defined(_MSC_VER)
+// Squelch warnings about dll vs string
+#pragma warning (push)
+#pragma warning (disable: 4251)
+#endif
 
 namespace GeographicLib {
 
@@ -34,8 +40,8 @@ namespace GeographicLib {
    *   identity.  (This is affected in predictable ways by errors in
    *   determining the latitude band and by loss of precision in the MGRS
    *   coordinates.)
-   * - All MGRS coordinates truncate to legal 100km blocks.  All MGRS
-   *   coordinates with a legal 100km block prefix are legal (even though the
+   * - All MGRS coordinates truncate to legal 100 km blocks.  All MGRS
+   *   coordinates with a legal 100 km block prefix are legal (even though the
    *   latitude band letter may now belong to a neighboring band).
    * - The range of UTM/UPS coordinates allowed for conversion to MGRS
    *   coordinates is the maximum consistent with staying within the letter
@@ -50,46 +56,42 @@ namespace GeographicLib {
    * - Inconsistent rules are used to determine the whether a particular MGRS
    *   coordinate is legal.  A more systematic approach is taken here.
    * - The underlying projections are not very accurately implemented.
+   *
+   * Example of use:
+   * \include example-MGRS.cpp
    **********************************************************************/
-  class MGRS {
+  class GEOGRAPHIC_EXPORT MGRS {
   private:
     typedef Math::real real;
-    // The smallest length s.t., 1.0e7 - eps < 1.0e7 (approx 1.9 nm)
-    static const real eps;
-    // The smallest angle s.t., 90 - eps < 90 (approx 50e-12 arcsec)
-    static const real angeps;
-    static const std::string hemispheres;
-    static const std::string utmcols[3];
-    static const std::string utmrow;
-    static const std::string upscols[4];
-    static const std::string upsrows[2];
-    static const std::string latband;
-    static const std::string upsband;
-    static const std::string digits;
+    // The smallest length s.t., 1.0e7 - eps_ < 1.0e7 (approx 1.9 nm)
+    static const real eps_;
+    // The smallest angle s.t., 90 - eps_ < 90 (approx 50e-12 arcsec)
+    static const real angeps_;
+    static const std::string hemispheres_;
+    static const std::string utmcols_[3];
+    static const std::string utmrow_;
+    static const std::string upscols_[4];
+    static const std::string upsrows_[2];
+    static const std::string latband_;
+    static const std::string upsband_;
+    static const std::string digits_;
 
-    static const int mineasting[4];
-    static const int maxeasting[4];
-    static const int minnorthing[4];
-    static const int maxnorthing[4];
+    static const int mineasting_[4];
+    static const int maxeasting_[4];
+    static const int minnorthing_[4];
+    static const int maxnorthing_[4];
     enum {
-      base = 10,
-      // Top-level tiles are 10^5 m = 100km on a side
-      tilelevel = 5,
+      base_ = 10,
+      // Top-level tiles are 10^5 m = 100 km on a side
+      tilelevel_ = 5,
       // Period of UTM row letters
-      utmrowperiod = 20,
+      utmrowperiod_ = 20,
       // Row letters are shifted by 5 for even zones
-      utmevenrowshift = 5,
+      utmevenrowshift_ = 5,
       // Maximum precision is um
-      maxprec = 5 + 6,
+      maxprec_ = 5 + 6,
     };
     static void CheckCoords(bool utmp, bool& northp, real& x, real& y);
-    static int lookup(const std::string& s, char c) throw() {
-      std::string::size_type r = s.find(toupper(c));
-      return r == std::string::npos ? -1 : int(r);
-    }
-    template<typename T> static std::string str(T x) {
-      std::ostringstream s; s << x; return s.str();
-    }
     static int UTMRow(int iband, int icol, int irow) throw();
 
     friend class UTMUPS;        // UTMUPS::StandardZone calls LatitudeBand
@@ -101,21 +103,21 @@ namespace GeographicLib {
     }
     // UTMUPS access these enums
     enum {
-      tile = 100000,            // Size MGRS blocks
-      minutmcol = 1,
-      maxutmcol = 9,
-      minutmSrow = 10,
-      maxutmSrow = 100,         // Also used for UTM S false northing
-      minutmNrow = 0,           // Also used for UTM N false northing
-      maxutmNrow = 95,
-      minupsSind = 8,           // These 4 ind's apply to easting and northing
-      maxupsSind = 32,
-      minupsNind = 13,
-      maxupsNind = 27,
-      upseasting = 20,          // Also used for UPS false northing
-      utmeasting = 5,           // UTM false easting
+      tile_ = 100000,            // Size MGRS blocks
+      minutmcol_ = 1,
+      maxutmcol_ = 9,
+      minutmSrow_ = 10,
+      maxutmSrow_ = 100,         // Also used for UTM S false northing
+      minutmNrow_ = 0,           // Also used for UTM N false northing
+      maxutmNrow_ = 95,
+      minupsSind_ = 8,           // These 4 ind's apply to easting and northing
+      maxupsSind_ = 32,
+      minupsNind_ = 13,
+      maxupsNind_ = 27,
+      upseasting_ = 20,          // Also used for UPS false northing
+      utmeasting_ = 5,           // UTM false easting
       // Difference between S hemisphere northing and N hemisphere northing
-      utmNshift = (maxutmSrow - minutmNrow) * tile
+      utmNshift_ = (maxutmSrow_ - minutmNrow_) * tile_
     };
     MGRS();                     // Disable constructor
 
@@ -126,20 +128,20 @@ namespace GeographicLib {
      *
      * @param[in] zone UTM zone (zero means UPS).
      * @param[in] northp hemisphere (true means north, false means south).
-     * @param[in] x (meters).
-     * @param[in] y (meters).
+     * @param[in] x easting of point (meters).
+     * @param[in] y northing of point (meters).
      * @param[in] prec precision relative to 100 km.
      * @param[out] mgrs MGRS string.
      *
-     * \e prec specifies the precision of the MSGRS string as follows:
-     * - prec = 0 (min), 100km
-     * - prec = 1, 10km
-     * - prec = 2, 1km
-     * - prec = 3, 100m
-     * - prec = 4, 10m
-     * - prec = 5, 1m
-     * - prec = 6, 0.1m
-     * - prec = 11 (max), 1um
+     * \e prec specifies the precision of the MGRS string as follows:
+     * - prec = 0 (min), 100 km
+     * - prec = 1, 10 km
+     * - prec = 2, 1 km
+     * - prec = 3, 100 m
+     * - prec = 4, 10 m
+     * - prec = 5, 1 m
+     * - prec = 6, 0.1 m
+     * - prec = 11 (max), 1 um
      *
      * UTM eastings are allowed to be in the range [100 km, 900 km], northings
      * are allowed to be in in [0 km, 9500 km] for the northern hemisphere and
@@ -159,17 +161,18 @@ namespace GeographicLib {
      * and of 1000 km as the minimum northing for southern hemisphere provide
      * at least 0.5 degree extension into standard UPS zones.  The upper ends
      * of the ranges for the UPS coordinates is dictated by requiring symmetry
-     * about the meridans 0E and 90E.
+     * about the meridians 0E and 90E.
      *
      * All allowed UTM and UPS coordinates may now be converted to legal MGRS
      * coordinates with the proviso that eastings and northings on the upper
-     * boundaries are silently reduced by about 4nm to place them \e within the
-     * allowed range.  (This includes reducing a southern hemisphere northing
-     * of 10000km by 4nm so that it is placed in latitude band M.)  The UTM or
-     * UPS coordinates are truncated to requested precision to determine the
-     * MGRS coordinate.  Thus in UTM zone 38N, the square area with easting in
-     * [444 km, 445 km) and northing in [3688 km, 3689 km) maps to MGRS
-     * coordinate 38SMB4488 (at \e prec = 2, 1km), Khulani Sq., Baghdad.
+     * boundaries are silently reduced by about 4 nm (4 nanometers) to place
+     * them \e within the allowed range.  (This includes reducing a southern
+     * hemisphere northing of 10000 km by 4 nm so that it is placed in latitude
+     * band M.)  The UTM or UPS coordinates are truncated to requested
+     * precision to determine the MGRS coordinate.  Thus in UTM zone 38N, the
+     * square area with easting in [444 km, 445 km) and northing in [3688 km,
+     * 3689 km) maps to MGRS coordinate 38SMB4488 (at \e prec = 2, 1 km),
+     * Khulani Sq., Baghdad.
      *
      * The UTM/UPS selection and the UTM zone is preserved in the conversion to
      * MGRS coordinate.  Thus for \e zone > 0, the MGRS coordinate begins with
@@ -199,8 +202,8 @@ namespace GeographicLib {
      *
      * @param[in] zone UTM zone (zero means UPS).
      * @param[in] northp hemisphere (true means north, false means south).
-     * @param[in] x (meters).
-     * @param[in] y (meters).
+     * @param[in] x easting of point (meters).
+     * @param[in] y northing of point (meters).
      * @param[in] lat latitude (degrees).
      * @param[in] prec precision relative to 100 km.
      * @param[out] mgrs MGRS string.
@@ -218,8 +221,8 @@ namespace GeographicLib {
      * @param[in] mgrs MGRS string.
      * @param[out] zone UTM zone (zero means UPS).
      * @param[out] northp hemisphere (true means north, false means south).
-     * @param[out] x (meters).
-     * @param[out] y (meters).
+     * @param[out] x easting of point (meters).
+     * @param[out] y northing of point (meters).
      * @param[out] prec precision relative to 100 km.
      * @param[in] centerp if true (default), return center of the MGRS square,
      *   else return SW (lower left) corner.
@@ -229,7 +232,7 @@ namespace GeographicLib {
      * (The leading 0 may be dropped from an input MGRS coordinate for UTM
      * zones 1&ndash;9.)  In addition, MGRS coordinates with a neighboring
      * latitude band letter are permitted provided that some portion of the
-     * 100km block is within the given latitude band.  Thus
+     * 100 km block is within the given latitude band.  Thus
      *   - 38VLS and 38WLS are allowed (latitude 64N intersects the square
      *     38[VW]LS); but 38VMS is not permitted (all of 38VMS is north of 64N)
      *   - 38MPE and 38NPF are permitted (they straddle the equator); but 38NPE
@@ -262,15 +265,28 @@ namespace GeographicLib {
     static Math::real MajorRadius() throw() { return UTMUPS::MajorRadius(); }
 
     /**
-     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     * @return \e f the flattening of the WGS84 ellipsoid.
      *
      * (The WGS84 value is returned because the UTM and UPS projections are
      * based on this ellipsoid.)
      **********************************************************************/
+    static Math::real Flattening() throw() { return UTMUPS::Flattening(); }
+    ///@}
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     **********************************************************************/
     static Math::real InverseFlattening() throw()
     { return UTMUPS::InverseFlattening(); }
-    ///@}
+    /// \endcond
   };
 
 } // namespace GeographicLib
+
+#if defined(_MSC_VER)
+#pragma warning (pop)
 #endif
+
+#endif  // GEOGRAPHICLIB_MGRS_HPP
