@@ -2,15 +2,16 @@
  * \file PolarStereographic.hpp
  * \brief Header for GeographicLib::PolarStereographic class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_POLARSTEREOGRAPHIC_HPP)
-#define GEOGRAPHICLIB_POLARSTEREOGRAPHIC_HPP "$Id: PolarStereographic.hpp 6906 2010-12-02 22:10:56Z karney $"
+#define GEOGRAPHICLIB_POLARSTEREOGRAPHIC_HPP \
+  "$Id: 07add8492c46e42012007a8738060abc902a5504 $"
 
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -25,16 +26,25 @@ namespace GeographicLib {
    *
    * This is a straightforward implementation of the equations in Snyder except
    * that Newton's method is used to invert the projection.
+   *
+   * Example of use:
+   * \include example-PolarStereographic.cpp
    **********************************************************************/
-  class PolarStereographic {
+  class GEOGRAPHIC_EXPORT PolarStereographic {
   private:
     typedef Math::real real;
     // _Cx used to be _C but g++ 3.4 has a macro of that name
-    const real _a, _r, _f, _e2, _e, _e2m, _Cx, _c;
+    real _a, _f, _e2, _e, _e2m, _Cx, _c;
     real _k0;
-    static const real tol, overflow;
-    static const int numit = 5;
-    static inline real sq(real x) throw() { return x * x; }
+    static const real tol_;
+    static const real overflow_;
+    static const int numit_ = 5;
+    // tan(x) for x in [-pi/2, pi/2] ensuring that the sign is right
+    static inline real tanx(real x) throw() {
+      real t = std::tan(x);
+      // Write the tests this way to ensure that tanx(NaN()) is NaN()
+      return x >= 0 ? (!(t < 0) ? t : overflow_) : (!(t >= 0) ? t : -overflow_);
+    }
     // Return e * atanh(e * x) for f >= 0, else return
     // - sqrt(-e2) * atan( sqrt(-e2) * x) for f < 0
     inline real eatanhe(real x) const throw() {
@@ -45,16 +55,16 @@ namespace GeographicLib {
     /**
      * Constructor for a ellipsoid with
      *
-     * @param[in] a equatorial radius (meters)
-     * @param[in] r reciprocal flattening.  Setting \e r = 0 implies \e r = inf
-     *   or flattening = 0 (i.e., a sphere).  Negative \e r indicates a prolate
-     *   ellipsoid.
+     * @param[in] a equatorial radius (meters).
+     * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
+     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      * @param[in] k0 central scale factor.
      *
      * An exception is thrown if either of the axes of the ellipsoid is
      * not positive \e a or if \e k0 is not positive.
      **********************************************************************/
-    PolarStereographic(real a, real r, real k0);
+    PolarStereographic(real a, real f, real k0);
 
     /**
      * Set the scale for the projection.
@@ -133,11 +143,18 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _a; }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value used in the constructor.  A value of 0 is returned for a sphere
-     *   (infinite inverse flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the value used in
+     *   the constructor.
      **********************************************************************/
-    Math::real InverseFlattening() const throw() { return _r; }
+    Math::real Flattening() const throw() { return _f; }
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw() { return 1/_f; }
+    /// \endcond
 
     /**
      * The central scale for the projection.  This is the value of \e k0 used
@@ -157,4 +174,4 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_POLARSTEREOGRAPHIC_HPP
