@@ -2,15 +2,16 @@
  * \file TransverseMercator.hpp
  * \brief Header for GeographicLib::TransverseMercator class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_TRANSVERSEMERCATOR_HPP)
-#define GEOGRAPHICLIB_TRANSVERSEMERCATOR_HPP "$Id: TransverseMercator.hpp 6950 2011-02-11 04:09:24Z karney $"
+#define GEOGRAPHICLIB_TRANSVERSEMERCATOR_HPP \
+  "$Id: 94bb078aa13d2d7392cee5498aae7df6e9914e4a $"
 
-#include "GeographicLib/Constants.hpp"
+#include <GeographicLib/Constants.hpp>
 
 #if !defined(TM_TX_MAXPOW)
 /**
@@ -18,7 +19,7 @@
  * TM_TX_MAXPOW can be set to any integer in [4, 8].
  **********************************************************************/
 #define TM_TX_MAXPOW \
-(GEOGRAPHICLIB_PREC == 1 ? 6 : GEOGRAPHICLIB_PREC == 0 ? 4 : 8)
+  (GEOGRAPHICLIB_PREC == 1 ? 6 : (GEOGRAPHICLIB_PREC == 0 ? 4 : 8))
 #endif
 
 namespace GeographicLib {
@@ -36,14 +37,14 @@ namespace GeographicLib {
    *  - C. F. F. Karney,
    *    <a href="http://dx.doi.org/10.1007/s00190-011-0445-3">
    *    Transverse Mercator with an accuracy of a few nanometers,</a>
-   *    J. Geodesy (2011);
+   *    J. Geodesy 85(8), 475-485 (Aug. 2011);
    *    preprint
    *    <a href="http://arxiv.org/abs/1002.1417">arXiv:1002.1417</a>.
    *
    * Kr&uuml;ger's method has been extended from 4th to 6th order.  The maximum
-   * errors is 5 nm (ground distance) for all positions within 35 degrees of
-   * the central meridian.  The error in the convergence is 2e-15&quot; and the
-   * relative error in the scale is 6e-12%%.  See Sec. 4 of
+   * error is 5 nm (5 nanometers), ground distance, for all positions within 35
+   * degrees of the central meridian.  The error in the convergence is
+   * 2e-15&quot; and the relative error in the scale is 6e-12%%.  See Sec. 4 of
    * <a href="http://arxiv.org/abs/1002.1417">arXiv:1002.1417</a> for details.
    * The speed penalty in going to 6th order is only about 1%.
    * TransverseMercatorExact is an alternative implementation of the projection
@@ -56,7 +57,7 @@ namespace GeographicLib {
    * TransverseMercator::Reverse functions.  The latitude of origin is taken to
    * be the equator.  There is no provision in this class for specifying a
    * false easting or false northing or a different latitude of origin.
-   * However these are can be simply included by the calling funtcion.  For
+   * However these are can be simply included by the calling function.  For
    * example, the UTMUPS class applies the false easting and false northing for
    * the UTM projections.  A more complicated example is the British National
    * Grid (<a href="http://www.spatialreference.org/ref/epsg/7405/">
@@ -66,23 +67,30 @@ namespace GeographicLib {
    * See TransverseMercator.cpp for more information on the implementation.
    *
    * See \ref transversemercator for a discussion of this projection.
+   *
+   * Example of use:
+   * \include example-TransverseMercator.cpp
+   *
+   * <a href="TransverseMercatorProj.1.html">TransverseMercatorProj</a> is a
+   * command-line utility providing access to the functionality of
+   * TransverseMercator and TransverseMercatorExact.
    **********************************************************************/
 
-  class TransverseMercator {
+  class GEOGRAPHIC_EXPORT TransverseMercator {
   private:
     typedef Math::real real;
-    static const int maxpow = TM_TX_MAXPOW;
-    static const real tol, overflow;
-    static const int numit = 5;
-    const real _a, _r, _f, _k0, _e2, _e, _e2m,  _c, _n;
+    static const int maxpow_ = TM_TX_MAXPOW;
+    static const real tol_;
+    static const real overflow_;
+    static const int numit_ = 5;
+    real _a, _f, _k0, _e2, _e, _e2m,  _c, _n;
     // _alp[0] and _bet[0] unused
-    real _a1, _b1, _alp[maxpow + 1], _bet[maxpow + 1];
-    static inline real sq(real x) throw() { return x * x; }
+    real _a1, _b1, _alp[maxpow_ + 1], _bet[maxpow_ + 1];
     // tan(x) for x in [-pi/2, pi/2] ensuring that the sign is right
     static inline real tanx(real x) throw() {
       real t = std::tan(x);
       // Write the tests this way to ensure that tanx(NaN()) is NaN()
-      return x >= 0 ? (!(t < 0) ? t : overflow) : (!(t >= 0) ? t : -overflow);
+      return x >= 0 ? (!(t < 0) ? t : overflow_) : (!(t >= 0) ? t : -overflow_);
     }
     // Return e * atanh(e * x) for f >= 0, else return
     // - sqrt(-e2) * atan( sqrt(-e2) * x) for f < 0
@@ -94,16 +102,16 @@ namespace GeographicLib {
     /**
      * Constructor for a ellipsoid with
      *
-     * @param[in] a equatorial radius (meters)
-     * @param[in] r reciprocal flattening.  Setting \e r = 0 implies \e r = inf
-     *   or flattening = 0 (i.e., a sphere).  Negative \e r indicates a prolate
-     *   ellipsoid.
+     * @param[in] a equatorial radius (meters).
+     * @param[in] f flattening of ellipsoid.  Setting \e f = 0 gives a sphere.
+     *   Negative \e f gives a prolate ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      * @param[in] k0 central scale factor.
      *
      * An exception is thrown if either of the axes of the ellipsoid or \e k0
      * is not positive.
      **********************************************************************/
-    TransverseMercator(real a, real r, real k0);
+    TransverseMercator(real a, real f, real k0);
 
     /**
      * Forward projection, from geographic to transverse Mercator.
@@ -167,11 +175,18 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _a; }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value used in the constructor.  A value of 0 is returned for a sphere
-     *   (infinite inverse flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the value used in
+     *   the constructor.
      **********************************************************************/
-    Math::real InverseFlattening() const throw() { return _r; }
+    Math::real Flattening() const throw() { return _f; }
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw() { return 1/_f; }
+    /// \endcond
 
     /**
      * @return \e k0 central scale for the projection.  This is the value of \e
@@ -190,4 +205,4 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_TRANSVERSEMERCATOR_HPP
