@@ -2,16 +2,17 @@
  * \file TransverseMercatorExact.hpp
  * \brief Header for GeographicLib::TransverseMercatorExact class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_HPP)
-#define GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_HPP "$Id: TransverseMercatorExact.hpp 6950 2011-02-11 04:09:24Z karney $"
+#define GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_HPP \
+  "$Id: bd96340b9dc3e7bfd09d4374296a75f4c6e00fc3 $"
 
-#include "GeographicLib/Constants.hpp"
-#include "GeographicLib/EllipticFunction.hpp"
+#include <GeographicLib/Constants.hpp>
+#include <GeographicLib/EllipticFunction.hpp>
 
 namespace GeographicLib {
 
@@ -30,16 +31,16 @@ namespace GeographicLib {
    *  - C. F. F. Karney,
    *    <a href="http://dx.doi.org/10.1007/s00190-011-0445-3">
    *    Transverse Mercator with an accuracy of a few nanometers,</a>
-   *    J. Geodesy (2011);
+   *    J. Geodesy 85(8), 475-485 (Aug. 2011);
    *    preprint
    *    <a href="http://arxiv.org/abs/1002.1417">arXiv:1002.1417</a>.
    *
-   * Lee's gives the correct results for forward and reverse
-   * transformations subject to the branch cut rules (see the description of
-   * the \e extendp argument to the constructor).  The maximum error is about 8
-   * nm (ground distance) for the forward and reverse transformations.  The
-   * error in the convergence is 2e-15&quot;, the relative error in the scale
-   * is 7e-12%%.  See Sec. 3 of
+   * Lee gives the correct results for forward and reverse transformations
+   * subject to the branch cut rules (see the description of the \e extendp
+   * argument to the constructor).  The maximum error is about 8 nm (8
+   * nanometers), ground distance, for the forward and reverse transformations.
+   * The error in the convergence is 2e-15&quot;, the relative error in the
+   * scale is 7e-12%%.  See Sec. 3 of
    * <a href="http://arxiv.org/abs/1002.1417">arXiv:1002.1417</a> for details.
    * The method is "exact" in the sense that the errors are close to the
    * round-off limit and that no changes are needed in the algorithms for them
@@ -58,26 +59,40 @@ namespace GeographicLib {
    * taken to be the equator.  See the documentation on TransverseMercator for
    * how to include a false easting, false northing, or a latitude of origin.
    *
+   * See <a href="http://geographiclib.sourceforge.net/tm-grid.kmz"
+   * type="application/vnd.google-earth.kmz"> tm-grid.kmz</a>, for an
+   * illustration of the transverse Mercator grid in Google Earth.
+   *
    * See TransverseMercatorExact.cpp for more information on the
    * implementation.
    *
    * See \ref transversemercator for a discussion of this projection.
+   *
+   * Example of use:
+   * \include example-TransverseMercatorExact.cpp
+   *
+   * <a href="TransverseMercatorProj.1.html">TransverseMercatorProj</a> is a
+   * command-line utility providing access to the functionality of
+   * TransverseMercator and TransverseMercatorExact.
    **********************************************************************/
 
-  class TransverseMercatorExact {
+  class GEOGRAPHIC_EXPORT TransverseMercatorExact {
   private:
     typedef Math::real real;
-    static const real tol, tol1, tol2, taytol, overflow;
-    static const int numit = 10;
-    const real _a, _r, _f, _k0, _mu, _mv, _e, _ep2;
-    const bool _extendp;
-    const EllipticFunction _Eu, _Ev;
-    static inline real sq(real x) throw() { return x * x; }
+    static const real tol_;
+    static const real tol1_;
+    static const real tol2_;
+    static const real taytol_;
+    static const real overflow_;
+    static const int numit_ = 10;
+    real _a, _f, _k0, _mu, _mv, _e, _ep2;
+    bool _extendp;
+    EllipticFunction _Eu, _Ev;
     // tan(x) for x in [-pi/2, pi/2] ensuring that the sign is right
     static inline real tanx(real x) throw() {
       real t = std::tan(x);
       // Write the tests this way to ensure that tanx(NaN()) is NaN()
-      return x >= 0 ? (!(t < 0) ? t : overflow) : (!(t >= 0) ? t : -overflow);
+      return x >= 0 ? (!(t < 0) ? t : overflow_) : (!(t >= 0) ? t : -overflow_);
     }
 
     real taup(real tau) const throw();
@@ -115,8 +130,9 @@ namespace GeographicLib {
     /**
      * Constructor for a ellipsoid with
      *
-     * @param[in] a equatorial radius (meters)
-     * @param[in] r reciprocal flattening.
+     * @param[in] a equatorial radius (meters).
+     * @param[in] f flattening of ellipsoid.  If \e f > 1, set flattening
+     *   to 1/\e f.
      * @param[in] k0 central scale factor.
      * @param[in] extendp use extended domain.
      *
@@ -141,24 +157,24 @@ namespace GeographicLib {
      *   - \e lat in [0, 90] and \e lon - \e lon0 in [0, 90]
      *   - \e lat in (-90, 0] and \e lon - \e lon0 in [90 (1 - \e e), 90]
      * - the union of
-     *   - \e x/(\e k0 \e a) in [0, inf) and
-     *     \e y/(\e k0 \e a) in [0, E(\e e^2)]
-     *   - \e x/(\e k0 \e a) in [K(1 - \e e^2) - E(1 - \e e^2), inf) and
-     *     \e y/(\e k0 \e a) in (-inf, 0]
+     *   - <i>x</i>/(\e k0 \e a) in [0, inf) and
+     *     <i>y</i>/(\e k0 \e a) in [0, E(<i>e</i><sup>2</sup>)]
+     *   - <i>x</i>/(\e k0 \e a) in [K(1 - <i>e</i><sup>2</sup>) - E(1 -
+     *     <i>e</i><sup>2</sup>), inf) and <i>y</i>/(\e k0 \e a) in (-inf, 0]
      * .
      * See Sec. 5 of
      * <a href="http://arxiv.org/abs/1002.1417">arXiv:1002.1417</a> for a full
      * discussion of the treatment of the branch cut.
      *
-     * The method will work for all ellipsoids used in terrestial geodesy.  The
-     * method cannot be applied directly to the case of a sphere (\e r = inf)
+     * The method will work for all ellipsoids used in terrestrial geodesy.
+     * The method cannot be applied directly to the case of a sphere (\e f = 0)
      * because some the constants characterizing this method diverge in that
-     * limit, and in practise, \e r should be smaller than about
-     * 1/numeric_limits< real >::%epsilon().  However, TransverseMercator
-     * treats the sphere exactly.  An exception is thrown if either axis of the
-     * ellipsoid or \e k0 is not positive or if \e r < 1.
+     * limit, and in practice, \e f should be larger than about numeric_limits<
+     * real >::%epsilon().  However, TransverseMercator treats the sphere
+     * exactly.  An exception is thrown if either axis of the ellipsoid or \e
+     * k0 is not positive or if \e f <= 0.
      **********************************************************************/
-    TransverseMercatorExact(real a, real r, real k0, bool extendp = false);
+    TransverseMercatorExact(real a, real f, real k0, bool extendp = false);
 
     /**
      * Forward projection, from geographic to transverse Mercator.
@@ -224,11 +240,18 @@ namespace GeographicLib {
     Math::real MajorRadius() const throw() { return _a; }
 
     /**
-     * @return \e r the inverse flattening of the ellipsoid.  This is the
-     *   value used in the constructor.  A value of 0 is returned for a sphere
-     *   (infinite inverse flattening).
+     * @return \e f the flattening of the ellipsoid.  This is the value used in
+     *   the constructor.
      **********************************************************************/
-    Math::real InverseFlattening() const throw() { return _r; }
+    Math::real Flattening() const throw() { return _f; }
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the ellipsoid.
+     **********************************************************************/
+    Math::real InverseFlattening() const throw() { return 1/_f; }
+    /// \endcond
 
     /**
      * @return \e k0 central scale for the projection.  This is the value of \e
@@ -247,4 +270,4 @@ namespace GeographicLib {
 
 } // namespace GeographicLib
 
-#endif
+#endif  // GEOGRAPHICLIB_TRANSVERSEMERCATOREXACT_HPP
