@@ -2,16 +2,17 @@
  * \file UTMUPS.hpp
  * \brief Header for GeographicLib::UTMUPS class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
 #if !defined(GEOGRAPHICLIB_UTMUPS_HPP)
-#define GEOGRAPHICLIB_UTMUPS_HPP "$Id: UTMUPS.hpp 6937 2011-02-01 20:17:13Z karney $"
+#define GEOGRAPHICLIB_UTMUPS_HPP \
+  "$Id: a529ed8aeaeffb02994254bbc7eb1209aa41b9ca $"
 
-#include "GeographicLib/Constants.hpp"
 #include <sstream>
+#include <GeographicLib/Constants.hpp>
 
 namespace GeographicLib {
 
@@ -55,21 +56,21 @@ namespace GeographicLib {
    * - Inconsistent rules are used to determine the whether a particular UTM or
    *   UPS coordinate is legal.  A more systematic approach is taken here.
    * - The underlying projections are not very accurately implemented.
+   *
+   * Example of use:
+   * \include example-UTMUPS.cpp
    **********************************************************************/
-  class UTMUPS {
+  class GEOGRAPHIC_EXPORT UTMUPS {
   private:
     typedef Math::real real;
-    static const real falseeasting[4];
-    static const real falsenorthing[4];
-    static const real mineasting[4];
-    static const real maxeasting[4];
-    static const real minnorthing[4];
-    static const real maxnorthing[4];
+    static const real falseeasting_[4];
+    static const real falsenorthing_[4];
+    static const real mineasting_[4];
+    static const real maxeasting_[4];
+    static const real minnorthing_[4];
+    static const real maxnorthing_[4];
     static real CentralMeridian(int zone) throw()
     { return real(6 * zone - 183); }
-    template<typename T> static std::string str(T x) {
-      std::ostringstream s; s << x; return s.str();
-    }
     static void CheckLatLon(real lat, real lon);
     // Throw an error if easting or northing are outside standard ranges.  If
     // throwp = false, return bool instead.
@@ -148,7 +149,7 @@ namespace GeographicLib {
      *
      * @param[in] lat latitude (degrees).
      * @param[in] lon longitude (degrees).
-     * @param[in] setzone zone override (optional)
+     * @param[in] setzone zone override (optional).
      *
      * This is exact.  If the optional argument \e setzone is given then use
      * that zone if it is non-negative, otherwise apply the rules given in
@@ -163,17 +164,16 @@ namespace GeographicLib {
      * @param[in] lat latitude of point (degrees).
      * @param[in] lon longitude of point (degrees).
      * @param[out] zone the UTM zone (zero means UPS).
-     * @param[out] northp hemisphere of location (true means northern, false
-     *   means southern).
+     * @param[out] northp hemisphere (true means north, false means south).
      * @param[out] x easting of point (meters).
      * @param[out] y northing of point (meters).
      * @param[out] gamma meridian convergence at point (degrees).
      * @param[out] k scale of projection at point.
      * @param[in] setzone zone override.
-     * @param[in] mgrslimits if true enforce the stricted MGRS limits on the
+     * @param[in] mgrslimits if true enforce the stricter MGRS limits on the
      *   coordinates (default = false).
      *
-     * The prefered zone for the result can be specified with \e setzone, see
+     * The preferred zone for the result can be specified with \e setzone, see
      * UTMUPS::StandardZone.  Throw error if the resulting easting or northing
      * is outside the allowed range (see Reverse), in which case the arguments
      * are unchanged.  This also returns meridian convergence \e gamma
@@ -188,15 +188,14 @@ namespace GeographicLib {
      * Reverse projection, from  UTM/UPS to geographic.
      *
      * @param[in] zone the UTM zone (zero means UPS).
-     * @param[in] northp hemisphere of location (true means northern, false
-     *   means southern).
+     * @param[in] northp hemisphere (true means north, false means south).
      * @param[in] x easting of point (meters).
      * @param[in] y northing of point (meters).
      * @param[out] lat latitude of point (degrees).
      * @param[out] lon longitude of point (degrees).
      * @param[out] gamma meridian convergence at point (degrees).
      * @param[out] k scale of projection at point.
-     * @param[in] mgrslimits if true enforce the stricted MGRS limits on the
+     * @param[in] mgrslimits if true enforce the stricter MGRS limits on the
      *   coordinates (default = false).
      *
      * Throw error if easting or northing is outside the allowed range (see
@@ -248,10 +247,9 @@ namespace GeographicLib {
     /**
      * Decode a UTM/UPS zone string.
      *
-     * @param[in] zonestr string represention of zone and hemisphere.
+     * @param[in] zonestr string representation of zone and hemisphere.
      * @param[out] zone the UTM zone (zero means UPS).
-     * @param[out] northp the hemisphere (true means northern, false
-     *   means southern).
+     * @param[out] northp hemisphere (true means north, false means south).
      *
      * For UTM, \e zonestr has the form of a zone number in the range
      * [UTMUPS::MINUTMZONE, UTMUPS::MAXUTMZONE] = [1, 60] followed by a
@@ -268,9 +266,8 @@ namespace GeographicLib {
      * Encode a UTM/UPS zone string.
      *
      * @param[out] zone the UTM zone (zero means UPS).
-     * @param[out] northp the hemisphere (true means northern, false
-     *   means southern).
-     * @return string represention of zone and hemisphere.
+     * @param[out] northp hemisphere (true means north, false means south).
+     * @return string representation of zone and hemisphere.
      *
      * \e zone must be in the range [UTMUPS::MINZONE, UTMUPS::MAXZONE] = [0,
      * 60] with \e zone = UTMUPS::UPS, 0, indicating UPS (but the resulting
@@ -299,15 +296,25 @@ namespace GeographicLib {
     { return Constants::WGS84_a<real>(); }
 
     /**
-     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     * @return \e f the flattening of the WGS84 ellipsoid.
      *
      * (The WGS84 value is returned because the UTM and UPS projections are
      * based on this ellipsoid.)
      **********************************************************************/
-    static Math::real InverseFlattening() throw()
-    { return Constants::WGS84_r<real>(); }
+    static Math::real Flattening() throw()
+    { return Constants::WGS84_f<real>(); }
     ///@}
+
+    /// \cond SKIP
+    /**
+     * <b>DEPRECATED</b>
+     * @return \e r the inverse flattening of the WGS84 ellipsoid.
+     **********************************************************************/
+    static Math::real InverseFlattening() throw()
+    { return 1/Constants::WGS84_f<real>(); }
+    /// \endcond
   };
 
 } // namespace GeographicLib
-#endif
+
+#endif  // GEOGRAPHICLIB_UTMUPS_HPP
