@@ -2,14 +2,15 @@
  * \file AzimuthalEquidistant.cpp
  * \brief Implementation for GeographicLib::AzimuthalEquidistant class
  *
- * Copyright (c) Charles Karney (2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/AzimuthalEquidistant.hpp"
+#include <GeographicLib/AzimuthalEquidistant.hpp>
 
-#define GEOGRAPHICLIB_AZIMUTHALEQUIDISTANT_CPP "$Id: AzimuthalEquidistant.cpp 6921 2010-12-31 14:34:50Z karney $"
+#define GEOGRAPHICLIB_AZIMUTHALEQUIDISTANT_CPP \
+  "$Id: 324fc318b35a411f024cfd5046ba58b8ef819df7 $"
 
 RCSID_DECL(GEOGRAPHICLIB_AZIMUTHALEQUIDISTANT_CPP)
 RCSID_DECL(GEOGRAPHICLIB_AZIMUTHALEQUIDISTANT_HPP)
@@ -18,7 +19,7 @@ namespace GeographicLib {
 
   using namespace std;
 
-  const Math::real AzimuthalEquidistant::eps =
+  const Math::real AzimuthalEquidistant::eps_ =
     real(0.01) * sqrt(numeric_limits<real>::min());
 
   void AzimuthalEquidistant::Forward(real lat0, real lon0, real lat, real lon,
@@ -29,7 +30,7 @@ namespace GeographicLib {
     azi0 *= Math::degree<real>();
     x = s * sin(azi0);
     y = s * cos(azi0);
-    rk = !(sig <= eps) ? m / s : 1;
+    rk = !(sig <= eps_) ? m / s : 1;
   }
 
   void AzimuthalEquidistant::Reverse(real lat0, real lon0, real x, real y,
@@ -40,7 +41,7 @@ namespace GeographicLib {
       s = Math::hypot(x, y);
     real sig, m;
     sig = _earth.Direct(lat0, lon0, azi0, s, lat, lon, azi, m);
-    rk = !(sig <= eps) ? m / s : 1;
+    rk = !(sig <= eps_) ? m / s : 1;
   }
 
 } // namespace GeographicLib
