@@ -1,16 +1,16 @@
-
 /**
  * \file CassiniSoldner.cpp
  * \brief Implementation for GeographicLib::CassiniSoldner class
  *
- * Copyright (c) Charles Karney (2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/CassiniSoldner.hpp"
+#include <GeographicLib/CassiniSoldner.hpp>
 
-#define GEOGRAPHICLIB_CASSINISOLDNER_CPP "$Id: CassiniSoldner.cpp 6921 2010-12-31 14:34:50Z karney $"
+#define GEOGRAPHICLIB_CASSINISOLDNER_CPP \
+  "$Id: 2823df38f3e31fd8b882e2f55ad72d6419b03246 $"
 
 RCSID_DECL(GEOGRAPHICLIB_CASSINISOLDNER_CPP)
 RCSID_DECL(GEOGRAPHICLIB_CASSINISOLDNER_HPP)
@@ -19,9 +19,9 @@ namespace GeographicLib {
 
   using namespace std;
 
-  const Math::real CassiniSoldner::eps1 =
+  const Math::real CassiniSoldner::eps1_ =
     real(0.01) * sqrt(numeric_limits<real>::epsilon());
-  const Math::real CassiniSoldner::eps2 = sqrt(numeric_limits<real>::min());
+  const Math::real CassiniSoldner::tiny_ = sqrt(numeric_limits<real>::min());
 
   void CassiniSoldner::Reset(real lat0, real lon0) throw() {
     _meridian = _earth.Line(lat0, lon0, real(0),
@@ -30,7 +30,7 @@ namespace GeographicLib {
                             Geodesic::AZIMUTH);
     real
       phi = LatitudeOrigin() * Math::degree<real>(),
-      f = _earth.InverseFlattening() != 0 ? 1 / _earth.InverseFlattening() : 0;
+      f = _earth.Flattening();
     _sbet0 = (1 - f) * sin(phi);
     _cbet0 = abs(LatitudeOrigin()) == 90 ? 0 : cos(phi);
     SinCosNorm(_sbet0, _cbet0);
@@ -44,7 +44,7 @@ namespace GeographicLib {
     real sig12, s12, azi1, azi2;
     lat = AngRound(lat);
     sig12 = _earth.Inverse(lat, -abs(dlon), lat, abs(dlon), s12, azi1, azi2);
-    if (sig12 < 100 * eps2)
+    if (sig12 < 100 * tiny_)
       sig12 = s12 = 0;
     sig12 *= real(0.5);
     s12 *= real(0.5);
