@@ -2,19 +2,20 @@
  * \file GeoCoords.cpp
  * \brief Implementation for GeographicLib::GeoCoords class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/GeoCoords.hpp"
-#include "GeographicLib/MGRS.hpp"
-#include "GeographicLib/DMS.hpp"
+#include <GeographicLib/GeoCoords.hpp>
 #include <vector>
 #include <sstream>
 #include <iomanip>
+#include <GeographicLib/MGRS.hpp>
+#include <GeographicLib/DMS.hpp>
 
-#define GEOGRAPHICLIB_GEOCOORDS_CPP "$Id: GeoCoords.cpp 6921 2010-12-31 14:34:50Z karney $"
+#define GEOGRAPHICLIB_GEOCOORDS_CPP \
+  "$Id: 084d4ec9163dc9d8989b54b12a04bf0f44a8c23f $"
 
 RCSID_DECL(GEOGRAPHICLIB_GEOCOORDS_CPP)
 RCSID_DECL(GEOGRAPHICLIB_GEOCOORDS_HPP)
@@ -23,12 +24,9 @@ namespace GeographicLib {
 
   using namespace std;
 
-  void GeoCoords::Reset(const std::string& s, bool centerp) {
+  void GeoCoords::Reset(const std::string& s, bool centerp, bool swaplatlong) {
     vector<string> sa;
     const char* spaces = " \t\n\v\f\r,"; // Include comma as a space
-    /*
-    const char* digits = "0123456789.";  // Include period as a digit
-    */
     for (string::size_type pos0 = 0, pos1; pos0 != string::npos;) {
       pos1 = s.find_first_not_of(spaces, pos0);
       if (pos1 == string::npos)
@@ -42,7 +40,7 @@ namespace GeographicLib {
       UTMUPS::Reverse(_zone, _northp, _easting, _northing,
                       _lat, _long, _gamma, _k);
     } else if (sa.size() == 2) {
-      DMS::DecodeLatLon(sa[0], sa[1], _lat, _long);
+      DMS::DecodeLatLon(sa[0], sa[1], _lat, _long, swaplatlong);
       UTMUPS::Forward( _lat, _long,
                        _zone, _northp, _easting, _northing, _gamma, _k);
     } else if (sa.size() == 3) {
@@ -69,28 +67,36 @@ namespace GeographicLib {
   }
 
 
-  string GeoCoords::GeoRepresentation(int prec) const {
+  string GeoCoords::GeoRepresentation(int prec, bool swaplatlong) const {
     prec = max(0, min(9, prec) + 5);
     ostringstream os;
     os << fixed << setprecision(prec);
-    if (_lat == _lat)
-      os << _lat;
+    real a = swaplatlong ? _long : _lat;
+    real b = swaplatlong ? _lat : _long;
+    if (!Math::isnan(a))
+      os << a;
     else
       os << "nan";
     os << " ";
-    if (_long == _long)
-      os << _long;
+    if (!Math::isnan(b))
+      os << b;
     else
       os << "nan";
     return os.str();
   }
 
-  string GeoCoords::DMSRepresentation(int prec) const {
+  string GeoCoords::DMSRepresentation(int prec, bool swaplatlong,
+				      char dmssep) const {
     prec = max(0, min(10, prec) + 5);
-    return DMS::Encode(_lat, unsigned(prec), DMS::LATITUDE) +
-      " " + DMS::Encode(_long, unsigned(prec), DMS::LONGITUDE);
+    return DMS::Encode(swaplatlong ? _long : _lat, unsigned(prec),
+                       swaplatlong ? DMS::LONGITUDE : DMS::LATITUDE, dmssep) +
+      " " + DMS::Encode(swaplatlong ? _lat : _long, unsigned(prec),
+                        swaplatlong ? DMS::LATITUDE : DMS::LONGITUDE, dmssep);
   }
 
+  string GeoCoords::DMSRepresentation(int prec, bool swaplatlong) const
+  { return DMSRepresentation(prec, swaplatlong, char(0)); }
+
   string GeoCoords::MGRSRepresentation(int prec) const {
     // Max precision is um
     prec = max(0, min(6, prec) + 5);
@@ -143,7 +149,7 @@ namespace GeographicLib {
 
   void GeoCoords::FixHemisphere() {
     if (_lat == 0 || (_northp && _lat >= 0) || (!_northp && _lat < 0) ||
-        _lat != _lat)
+        Math::isnan(_lat))
       // Allow either hemisphere for equator
       return;
     if (_zone != UTMUPS::UPS) {
