@@ -2,91 +2,69 @@
  * \file Geocentric.cpp
  * \brief Implementation for GeographicLib::Geocentric class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/Geocentric.hpp"
+#include <GeographicLib/Geocentric.hpp>
 
-#define GEOGRAPHICLIB_GEOCENTRIC_CPP "$Id: Geocentric.cpp 6952 2011-02-14 20:26:44Z karney $"
+#define GEOGRAPHICLIB_GEOCENTRIC_CPP \
+  "$Id: b5135e8042dbbbcddfd5894c66b729bf5c43cab9 $"
 
 RCSID_DECL(GEOGRAPHICLIB_GEOCENTRIC_CPP)
 RCSID_DECL(GEOGRAPHICLIB_GEOCENTRIC_HPP)
 
-#if defined(_MSC_VER)
-// Squelch warnings about unsafe use of copy
-#pragma warning (disable: 4996)
-#endif
-
 namespace GeographicLib {
 
   using namespace std;
 
-  Geocentric::Geocentric(real a, real r)
+  Geocentric::Geocentric(real a, real f)
     : _a(a)
-    , _r(r)
-    , _f(_r != 0 ? 1 / _r : 0)
+    , _f(f <= 1 ? f : 1/f)
     , _e2(_f * (2 - _f))
-    , _e2m(sq(1 - _f))          // 1 - _e2
+    , _e2m(Math::sq(1 - _f))          // 1 - _e2
     , _e2a(abs(_e2))
-    , _e4a(sq(_e2))
+    , _e4a(Math::sq(_e2))
     , _maxrad(2 * _a / numeric_limits<real>::epsilon())
   {
-    if (!(_a > 0))
+    if (!(Math::isfinite(_a) && _a > 0))
       throw GeographicErr("Major radius is not positive");
-    if (!(_f < 1))
+    if (!(Math::isfinite(_f) && _f < 1))
       throw GeographicErr("Minor radius is not positive");
   }
 
   const Geocentric Geocentric::WGS84(Constants::WGS84_a<real>(),
-                                     Constants::WGS84_r<real>());
-  /*
-  void Geocentric::Forward(real lat, real lon, real h,
-                           real& x, real& y, real& z) const throw() {
-    lon = lon >= 180 ? lon - 360 : lon < -180 ? lon + 360 : lon;
-    real
-      phi = lat * Math::degree<real>(),
-      lam = lon * Math::degree<real>(),
-      sphi = sin(phi),
-      cphi = abs(lat) == 90 ? 0 : cos(phi),
-      n = _a/sqrt(1 - _e2 * sq(sphi)),
-      slam = lon == -180 ? 0 : sin(lam),
-      clam = abs(lon) == 90 ? 0 : cos(lam);
-    z = ( _e2m * n + h) * sphi;
-    x = (n + h) * cphi;
-    y = x * slam;
-    x *= clam;
-  }
-*/
+                                     Constants::WGS84_f<real>());
+
   void Geocentric::IntForward(real lat, real lon, real h,
-                              real& x, real& y, real& z,
-                              real M[dim2]) const throw() {
-    lon = lon >= 180 ? lon - 360 : lon < -180 ? lon + 360 : lon;
+                              real& X, real& Y, real& Z,
+                              real M[dim2_]) const throw() {
+    lon = lon >= 180 ? lon - 360 : (lon < -180 ? lon + 360 : lon);
     real
       phi = lat * Math::degree<real>(),
       lam = lon * Math::degree<real>(),
       sphi = sin(phi),
       cphi = abs(lat) == 90 ? 0 : cos(phi),
-      n = _a/sqrt(1 - _e2 * sq(sphi)),
+      n = _a/sqrt(1 - _e2 * Math::sq(sphi)),
       slam = lon == -180 ? 0 : sin(lam),
       clam = abs(lon) == 90 ? 0 : cos(lam);
-    z = ( _e2m * n + h) * sphi;
-    x = (n + h) * cphi;
-    y = x * slam;
-    x *= clam;
+    Z = ( _e2m * n + h) * sphi;
+    X = (n + h) * cphi;
+    Y = X * slam;
+    X *= clam;
     if (M)
       Rotation(sphi, cphi, slam, clam, M);
   }
 
-  void Geocentric::IntReverse(real x, real y, real z,
+  void Geocentric::IntReverse(real X, real Y, real Z,
                               real& lat, real& lon, real& h,
-                              real M[dim2]) const throw() {
+                              real M[dim2_]) const throw() {
     real
-      R = Math::hypot(x, y),
-      slam = R ? y / R : 0,
-      clam = R ? x / R : 1;
-    h = Math::hypot(R, z);      // Distance to center of earth
+      R = Math::hypot(X, Y),
+      slam = R ? Y / R : 0,
+      clam = R ? X / R : 1;
+    h = Math::hypot(R, Z);      // Distance to center of earth
     real sphi, cphi;
     if (h > _maxrad) {
       // We really far away (> 12 million light years); treat the earth as a
@@ -94,27 +72,27 @@ namespace GeographicLib {
       // This avoids overflow, e.g., in the computation of disc below.  It's
       // possible that h has overflowed to inf; but that's OK.
       //
-      // Treat the case x, y finite, but R overflows to +inf by scaling by 2.
-      R = Math::hypot(x/2, y/2);
-      slam = R ? (y/2) / R : 0;
-      clam = R ? (x/2) / R : 1;
-      real H = Math::hypot(z/2, R);
-      sphi = (z/2) / H;
+      // Treat the case X, Y finite, but R overflows to +inf by scaling by 2.
+      R = Math::hypot(X/2, Y/2);
+      slam = R ? (Y/2) / R : 0;
+      clam = R ? (X/2) / R : 1;
+      real H = Math::hypot(Z/2, R);
+      sphi = (Z/2) / H;
       cphi = R / H;
     } else if (_e4a == 0) {
       // Treat the spherical case.  Dealing with underflow in the general case
       // with _e2 = 0 is difficult.  Origin maps to N pole same as with
       // ellipsoid.
-      real H = Math::hypot(h == 0 ? 1 : z, R);
-      sphi = (h == 0 ? 1 : z) / H;
+      real H = Math::hypot(h == 0 ? 1 : Z, R);
+      sphi = (h == 0 ? 1 : Z) / H;
       cphi = R / H;
       h -= _a;
     } else {
-      // Treat prolate spheroids by swapping R and z here and by switching
+      // Treat prolate spheroids by swapping R and Z here and by switching
       // the arguments to phi = atan2(...) at the end.
       real
-        p = sq(R / _a),
-        q = _e2m * sq(z / _a),
+        p = Math::sq(R / _a),
+        q = _e2m * Math::sq(Z / _a),
         r = (p + q - _e4a) / 6;
       if (_f < 0) swap(p, q);
       if ( !(_e4a * q == 0 && r <= 0) ) {
@@ -122,12 +100,12 @@ namespace GeographicLib {
           // Avoid possible division by zero when r = 0 by multiplying
           // equations for s and t by r^3 and r, resp.
           S = _e4a * p * q / 4, // S = r^3 * s
-          r2 = sq(r),
+          r2 = Math::sq(r),
           r3 = r * r2,
-          disc =  S * (2 * r3 + S);
+          disc = S * (2 * r3 + S);
         real u = r;
         if (disc >= 0) {
-          real T3 = r3 + S;
+          real T3 = S + r3;
           // Pick the sign on the sqrt to maximize abs(T3).  This minimizes
           // loss of precision due to cancellation.  The result is unchanged
           // because of the way the T is used in definition of u.
@@ -144,7 +122,7 @@ namespace GeographicLib {
           u += 2 * r * cos(ang / 3);
         }
         real
-          v = sqrt(sq(u) + _e4a * q), // guaranteed positive
+          v = sqrt(Math::sq(u) + _e4a * q), // guaranteed positive
           // Avoid loss of accuracy when u < 0.  Underflow doesn't occur in
           // e4 * q / (v - u) because u ~ e^4 when q is small and u < 0.
           uv = u < 0 ? _e4a * q / (v - u) : u + v, // u+v, guaranteed positive
@@ -152,14 +130,14 @@ namespace GeographicLib {
           w = max(real(0), _e2a * (uv - q) / (2 * v)),
           // Rearrange expression for k to avoid loss of accuracy due to
           // subtraction.  Division by 0 not possible because uv > 0, w >= 0.
-          k = uv / (sqrt(uv + sq(w)) + w),
+          k = uv / (sqrt(uv + Math::sq(w)) + w),
           k1 = _f >= 0 ? k : k - _e2,
           k2 = _f >= 0 ? k + _e2 : k,
           d = k1 * R / k2,
-          H = Math::hypot(z/k1, R/k2);
-        sphi = (z/k1) / H;
+          H = Math::hypot(Z/k1, R/k2);
+        sphi = (Z/k1) / H;
         cphi = (R/k2) / H;
-        h = (1 - _e2m/k1) * Math::hypot(d, z);
+        h = (1 - _e2m/k1) * Math::hypot(d, Z);
       } else {                  // e4 * q == 0 && r <= 0
         // This leads to k = 0 (oblate, equatorial plane) and k + e^2 = 0
         // (prolate, rotation axis) and the generation of 0/0 in the general
@@ -173,7 +151,7 @@ namespace GeographicLib {
           H = Math::hypot(zz, xx);
         sphi = zz / H;
         cphi = xx / H;
-        if (z < 0) sphi = -sphi; // for tiny negative z (not for prolate)
+        if (Z < 0) sphi = -sphi; // for tiny negative Z (not for prolate)
         h = - _a * (_f >= 0 ? _e2m : 1) * H / _e2a;
       }
     }
@@ -185,7 +163,7 @@ namespace GeographicLib {
   }
 
   void Geocentric::Rotation(real sphi, real cphi, real slam, real clam,
-                             real M[dim2]) const throw() {
+                            real M[dim2_]) throw() {
     // This rotation matrix is given by the following quaternion operations
     // qrot(lam, [0,0,1]) * qrot(phi, [0,-1,0]) * [1,1,1,1]/2
     // or
@@ -193,11 +171,11 @@ namespace GeographicLib {
     // where
     // qrot(t,v) = [cos(t/2), sin(t/2)*v[1], sin(t/2)*v[2], sin(t/2)*v[3]]
 
-    // Local x axis (east) in geocentric coords
+    // Local X axis (east) in geocentric coords
     M[0] = -slam;        M[3] =  clam;        M[6] = 0;
-    // Local y axis (north) in geocentric coords
+    // Local Y axis (north) in geocentric coords
     M[1] = -clam * sphi; M[4] = -slam * sphi; M[7] = cphi;
-    // Local z axis (up) in geocentric coords
+    // Local Z axis (up) in geocentric coords
     M[2] =  clam * cphi; M[5] =  slam * cphi; M[8] = sphi;
   }
 
