@@ -2,8 +2,8 @@
  * \file Geodesic.cpp
  * \brief Implementation for GeographicLib::Geodesic class
  *
- * Copyright (c) Charles Karney (2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2012) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  *
  * This is a reformulation of the geodesic problem.  The notation is as
@@ -14,7 +14,7 @@
  *   - omega = longitude on auxiliary sphere
  *   - lambda = longitude
  *   - alpha = azimuth of great circle
- *   - sigma = arc length along greate circle
+ *   - sigma = arc length along great circle
  *   - s = distance
  *   - tau = scaled distance (= sigma at multiples of pi/2)
  * - at northwards equator crossing
@@ -26,10 +26,11 @@
  * - s and c prefixes mean sin and cos
  **********************************************************************/
 
-#include "GeographicLib/Geodesic.hpp"
-#include "GeographicLib/GeodesicLine.hpp"
+#include <GeographicLib/Geodesic.hpp>
+#include <GeographicLib/GeodesicLine.hpp>
 
-#define GEOGRAPHICLIB_GEODESIC_CPP "$Id: Geodesic.cpp 6937 2011-02-01 20:17:13Z karney $"
+#define GEOGRAPHICLIB_GEODESIC_CPP \
+  "$Id: dd137806b8a5ba58211a37eb87e163b8a9bd7aa7 $"
 
 RCSID_DECL(GEOGRAPHICLIB_GEODESIC_CPP)
 RCSID_DECL(GEOGRAPHICLIB_GEODESIC_HPP)
@@ -39,32 +40,35 @@ namespace GeographicLib {
   using namespace std;
 
   // Underflow guard.  We require
-  //   eps2 * epsilon() > 0
-  //   eps2 + epsilon() == epsilon()
-  const Math::real Geodesic::eps2 = sqrt(numeric_limits<real>::min());
-  const Math::real Geodesic::tol0 = numeric_limits<real>::epsilon();
-  const Math::real Geodesic::tol1 = 100 * tol0;
-  const Math::real Geodesic::tol2 = sqrt(numeric_limits<real>::epsilon());
-  const Math::real Geodesic::xthresh = 1000 * tol2;
+  //   tiny_ * epsilon() > 0
+  //   tiny_ + epsilon() == epsilon()
+  const Math::real Geodesic::tiny_ = sqrt(numeric_limits<real>::min());
+  const Math::real Geodesic::tol0_ = numeric_limits<real>::epsilon();
+  // Increase multiplier in defn of tol1_ from 100 to 200 to fix inverse case
+  // 52.784459512564 0 -52.784459512563990912 179.634407464943777557
+  // which otherwise failed for Visual Studio 10 (Release and Debug)
+  const Math::real Geodesic::tol1_ = 200 * tol0_;
+  const Math::real Geodesic::tol2_ = sqrt(numeric_limits<real>::epsilon());
+  const Math::real Geodesic::xthresh_ = 1000 * tol2_;
 
-  Geodesic::Geodesic(real a, real r)
+  Geodesic::Geodesic(real a, real f)
     : _a(a)
-    , _r(r)
-    , _f(_r != 0 ? 1 / _r : 0)
+    , _f(f <= 1 ? f : 1/f)
     , _f1(1 - _f)
     , _e2(_f * (2 - _f))
-    , _ep2(_e2 / sq(_f1))       // e2 / (1 - e2)
+    , _ep2(_e2 / Math::sq(_f1))       // e2 / (1 - e2)
     , _n(_f / ( 2 - _f))
     , _b(_a * _f1)
-    , _c2((sq(_a) + sq(_b) *
+    , _c2((Math::sq(_a) + Math::sq(_b) *
            (_e2 == 0 ? 1 :
             (_e2 > 0 ? Math::atanh(sqrt(_e2)) : atan(sqrt(-_e2))) /
             sqrt(abs(_e2))))/2) // authalic radius squared
-    , _etol2(tol2 / max(real(0.1), sqrt(abs(_e2))))
+      // The sig12 threshold for "really short"
+    , _etol2(10 * tol2_ / max(real(0.1), sqrt(abs(_e2))))
   {
-    if (!(_a > 0))
+    if (!(Math::isfinite(_a) && _a > 0))
       throw GeographicErr("Major radius is not positive");
-    if (!(_f < 1))
+    if (!(Math::isfinite(_b) && _b > 0))
       throw GeographicErr("Minor radius is not positive");
     A3coeff();
     C3coeff();
@@ -72,7 +76,7 @@ namespace GeographicLib {
   }
 
   const Geodesic Geodesic::WGS84(Constants::WGS84_a<real>(),
-                                 Constants::WGS84_r<real>());
+                                 Constants::WGS84_f<real>());
 
   Math::real Geodesic::SinCosSeries(bool sinp,
                                     real sinx, real cosx,
@@ -108,10 +112,9 @@ namespace GeographicLib {
                                  real& lat2, real& lon2, real& azi2,
                                  real& s12, real& m12, real& M12, real& M21,
                                  real& S12) const throw() {
-    return
-      GeodesicLine(*this, lat1, lon1, azi1,
-                   // Automatically supply DISTANCE_IN if necessary
-                   outmask | (arcmode ? NONE : DISTANCE_IN))
+    return GeodesicLine(*this, lat1, lon1, azi1,
+                        // Automatically supply DISTANCE_IN if necessary
+                        outmask | (arcmode ? NONE : DISTANCE_IN))
       .                         // Note the dot!
       GenPosition(arcmode, s12_a12, outmask,
                   lat2, lon2, azi2, s12, m12, M12, M21, S12);
@@ -163,15 +166,31 @@ namespace GeographicLib {
     phi = lat1 * Math::degree<real>();
     // Ensure cbet1 = +epsilon at poles
     sbet1 = _f1 * sin(phi);
-    cbet1 = lat1 == -90 ? eps2 : cos(phi);
+    cbet1 = lat1 == -90 ? tiny_ : cos(phi);
     SinCosNorm(sbet1, cbet1);
 
     phi = lat2 * Math::degree<real>();
     // Ensure cbet2 = +epsilon at poles
     sbet2 = _f1 * sin(phi);
-    cbet2 = abs(lat2) == 90 ? eps2 : cos(phi);
+    cbet2 = abs(lat2) == 90 ? tiny_ : cos(phi);
     SinCosNorm(sbet2, cbet2);
 
+    // If cbet1 < -sbet1, then cbet2 - cbet1 is a sensitive measure of the
+    // |bet1| - |bet2|.  Alternatively (cbet1 >= -sbet1), abs(sbet2) + sbet1 is
+    // a better measure.  This logic is used in assigning calp2 in Lambda12.
+    // Sometimes these quantities vanish and in that case we force bet2 = +/-
+    // bet1 exactly.  An example where is is necessary is the inverse problem
+    // 48.522876735459 0 -48.52287673545898293 179.599720456223079643
+    // which failed with Visual Studio 10 (Release and Debug)
+
+    if (cbet1 < -sbet1) {
+      if (cbet2 == cbet1)
+        sbet2 = sbet2 < 0 ? sbet1 : -sbet1;
+    } else {
+      if (abs(sbet2) == -sbet1)
+        cbet2 = cbet1;
+    }
+
     real
       lam12 = lon12 * Math::degree<real>(),
       slam12 = lon12 == 180 ? 0 : sin(lam12),
@@ -179,7 +198,7 @@ namespace GeographicLib {
 
     real a12, sig12, calp1, salp1, calp2, salp2;
     // index zero elements of these arrays are unused
-    real C1a[nC1 + 1], C2a[nC2 + 1], C3a[nC3];
+    real C1a[nC1_ + 1], C2a[nC2_ + 1], C3a[nC3_];
 
     bool meridian = lat1 == -90 || slam12 == 0;
 
@@ -192,7 +211,7 @@ namespace GeographicLib {
       calp2 = 1; salp2 = 0;           // At the target we're heading north
 
       real
-        // tan(bet) = tan(sig) * cos(alp),
+        // tan(bet) = tan(sig) * cos(alp)
         ssig1 = sbet1, csig1 = calp1 * cbet1,
         ssig2 = sbet2, csig2 = calp2 * cbet2;
 
@@ -224,7 +243,7 @@ namespace GeographicLib {
     real omg12;
     if (!meridian &&
         sbet1 == 0 &&   // and sbet2 == 0
-         // Mimic the way Lambda12 works with calp1 = 0
+        // Mimic the way Lambda12 works with calp1 = 0
         (_f <= 0 || lam12 <= Math::pi<real>() - _f * Math::pi<real>())) {
 
       // Geodesic runs along equator
@@ -234,7 +253,7 @@ namespace GeographicLib {
       if (outmask & GEODESICSCALE)
         M12 = M21 = cos(lam12 / _f1);
       a12 = lon12 / _f1;
-      sig12 = lam12 / _f1;
+      sig12 = omg12 = lam12 / _f1;
 
     } else if (!meridian) {
 
@@ -249,28 +268,27 @@ namespace GeographicLib {
 
       if (sig12 >= 0) {
         // Short lines (InverseStart sets salp2, calp2)
-        real w1 = sqrt(1 - _e2 * sq(cbet1));
-        s12x = sig12 * _a * w1;
-        m12x = sq(w1) * _a / _f1 * sin(sig12 * _f1 / w1);
+        real wm = sqrt(1 - _e2 * Math::sq((cbet1 + cbet2) / 2));
+        s12x = sig12 * _a * wm;
+        m12x = Math::sq(wm) * _a / _f1 * sin(sig12 * _f1 / wm);
         if (outmask & GEODESICSCALE)
-          M12 = M21 = cos(sig12 * _f1 / w1);
+          M12 = M21 = cos(sig12 * _f1 / wm);
         a12 = sig12 / Math::degree<real>();
-        omg12 = lam12 / w1;
+        omg12 = lam12 / wm;
       } else {
 
         // Newton's method
         real ssig1, csig1, ssig2, csig2, eps;
         real ov = 0;
         unsigned numit = 0;
-        for (unsigned trip = 0; numit < maxit; ++numit) {
+        for (unsigned trip = 0; numit < maxit_; ++numit) {
           real dv;
           real v = Lambda12(sbet1, cbet1, sbet2, cbet2, salp1, calp1,
                             salp2, calp2, sig12, ssig1, csig1, ssig2, csig2,
                             eps, omg12, trip < 1, dv, C1a, C2a, C3a) - lam12;
-
-          if (!(abs(v) > eps2) || !(trip < 1)) {
-            if (!(abs(v) <= max(tol1, ov)))
-              numit = maxit;
+          if (!(abs(v) > tiny_) || !(trip < 1)) {
+            if (!(abs(v) <= max(tol1_, ov)))
+              numit = maxit_;
             break;
           }
           real
@@ -287,23 +305,23 @@ namespace GeographicLib {
           // 100 * epsilon.  The second takes credit for an anticipated
           // reduction in abs(v) by v/ov (due to the latest update in alp1) and
           // checks this against epsilon.
-          if (!(abs(v) >= tol1 && sq(v) >= ov * tol0)) ++trip;
+          if (!(abs(v) >= tol1_ && Math::sq(v) >= ov * tol0_)) ++trip;
           ov = abs(v);
         }
 
-        if (numit >= maxit) {
+        if (numit >= maxit_) {
           // Signal failure.
           if (outmask & DISTANCE)
-            s12 = Math::NaN();
+            s12 = Math::NaN<real>();
           if (outmask & AZIMUTH)
-            azi1 = azi2 = Math::NaN();
+            azi1 = azi2 = Math::NaN<real>();
           if (outmask & REDUCEDLENGTH)
-            m12 = Math::NaN();
+            m12 = Math::NaN<real>();
           if (outmask & GEODESICSCALE)
-            M12 = M21 = Math::NaN();
+            M12 = M21 = Math::NaN<real>();
           if (outmask & AREA)
-            S12 = Math::NaN();
-          return Math::NaN();
+            S12 = Math::NaN<real>();
+          return Math::NaN<real>();
         }
 
         {
@@ -320,10 +338,10 @@ namespace GeographicLib {
     }
 
     if (outmask & DISTANCE)
-      s12 = s12x;
+      s12 = 0 + s12x;           // Convert -0 to 0
 
     if (outmask & REDUCEDLENGTH)
-      m12 = m12x;
+      m12 = 0 + m12x;           // Convert -0 to 0
 
     if (outmask & AREA) {
       real
@@ -336,23 +354,24 @@ namespace GeographicLib {
           // From Lambda12: tan(bet) = tan(sig) * cos(alp)
           ssig1 = sbet1, csig1 = calp1 * cbet1,
           ssig2 = sbet2, csig2 = calp2 * cbet2,
-          k2 = sq(calp0) * _ep2,
+          k2 = Math::sq(calp0) * _ep2,
           // Multiplier = a^2 * e^2 * cos(alpha0) * sin(alpha0).
-          A4 = sq(_a) * calp0 * salp0 * _e2;
+          A4 = Math::sq(_a) * calp0 * salp0 * _e2;
         SinCosNorm(ssig1, csig1);
         SinCosNorm(ssig2, csig2);
-        real C4a[nC4];
+        real C4a[nC4_];
         C4f(k2, C4a);
         real
-          B41 = Geodesic::SinCosSeries(false, ssig1, csig1, C4a, nC4),
-          B42 = Geodesic::SinCosSeries(false, ssig2, csig2, C4a, nC4);
+          B41 = SinCosSeries(false, ssig1, csig1, C4a, nC4_),
+          B42 = SinCosSeries(false, ssig2, csig2, C4a, nC4_);
         S12 = A4 * (B42 - B41);
       } else
         // Avoid problems with indeterminate sig1, sig2 on equator
         S12 = 0;
+
       if (!meridian &&
           omg12 < real(0.75) * Math::pi<real>() && // Long difference too big
-          sbet2 - sbet1 < real(1.75)) {           // Lat difference too big
+          sbet2 - sbet1 < real(1.75)) {            // Lat difference too big
         // Use tan(Gamma/2) = tan(omg12/2)
         // * (tan(bet1/2)+tan(bet2/2))/(1+tan(bet1/2)*tan(bet2/2))
         // with tan(x/2) = sin(x)/(1+cos(x))
@@ -371,7 +390,7 @@ namespace GeographicLib {
         // being attached to 0 correctly.  The following ensures the correct
         // behavior.
         if (salp12 == 0 && calp12 < 0) {
-          salp12 = Geodesic::eps2 * calp1;
+          salp12 = tiny_ * calp1;
           calp12 = -1;
         }
         alp12 = atan2(salp12, calp12);
@@ -416,13 +435,13 @@ namespace GeographicLib {
     C2f(eps, C2a);
     real
       A1m1 = A1m1f(eps),
-      AB1 = (1 + A1m1) * (SinCosSeries(true, ssig2, csig2, C1a, nC1) -
-                          SinCosSeries(true, ssig1, csig1, C1a, nC1)),
+      AB1 = (1 + A1m1) * (SinCosSeries(true, ssig2, csig2, C1a, nC1_) -
+                          SinCosSeries(true, ssig1, csig1, C1a, nC1_)),
       A2m1 = A2m1f(eps),
-      AB2 = (1 + A2m1) * (SinCosSeries(true, ssig2, csig2, C2a, nC2) -
-                          SinCosSeries(true, ssig1, csig1, C2a, nC2)),
-      cbet1sq = sq(cbet1),
-      cbet2sq = sq(cbet2),
+      AB2 = (1 + A2m1) * (SinCosSeries(true, ssig2, csig2, C2a, nC2_) -
+                          SinCosSeries(true, ssig1, csig1, C2a, nC2_)),
+      cbet1sq = Math::sq(cbet1),
+      cbet2sq = Math::sq(cbet2),
       w1 = sqrt(1 - _e2 * cbet1sq),
       w2 = sqrt(1 - _e2 * cbet2sq),
       // Make sure it's OK to have repeated dummy arguments
@@ -435,7 +454,7 @@ namespace GeographicLib {
     m12a = (w2 * (csig1 * ssig2) - w1 * (ssig1 * csig2))
       - _f1 * csig1 * csig2 * J12;
     // Missing a factor of _b
-    s12b =  (1 + A1m1) * sig12 + AB1;
+    s12b = (1 + A1m1) * sig12 + AB1;
     if (scalep) {
       real csig12 = csig1 * csig2 + ssig1 * ssig2;
       J12 *= _f1;
@@ -451,19 +470,19 @@ namespace GeographicLib {
     // This solution is adapted from Geocentric::Reverse.
     real k;
     real
-      p = sq(x),
-      q = sq(y),
+      p = Math::sq(x),
+      q = Math::sq(y),
       r = (p + q - 1) / 6;
     if ( !(q == 0 && r <= 0) ) {
       real
         // Avoid possible division by zero when r = 0 by multiplying equations
         // for s and t by r^3 and r, resp.
         S = p * q / 4,            // S = r^3 * s
-        r2 = sq(r),
+        r2 = Math::sq(r),
         r3 = r * r2,
         // The discrimant of the quadratic equation for T3.  This is zero on
         // the evolute curve p^(1/3)+q^(1/3) = 1
-        disc =  S * (S + 2 * r3);
+        disc = S * (S + 2 * r3);
       real u = r;
       if (disc >= 0) {
         real T3 = S + r3;
@@ -483,13 +502,13 @@ namespace GeographicLib {
         u += 2 * r * cos(ang / 3);
       }
       real
-        v = sqrt(sq(u) + q),    // guaranteed positive
+        v = sqrt(Math::sq(u) + q),    // guaranteed positive
         // Avoid loss of accuracy when u < 0.
         uv = u < 0 ? q / (v - u) : u + v, // u+v, guaranteed positive
         w = (uv - q) / (2 * v);           // positive?
       // Rearrange expression for k to avoid loss of accuracy due to
       // subtraction.  Division by 0 not possible because uv > 0, w >= 0.
-      k = uv / (sqrt(uv + sq(w)) + w);   // guaranteed positive
+      k = uv / (sqrt(uv + Math::sq(w)) + w);   // guaranteed positive
     } else {               // q == 0 && r <= 0
       // y = 0 with |x| <= 1.  Handle this case directly.
       // for y small, positive root is k = abs(y)/sqrt(1-x^2)
@@ -513,19 +532,35 @@ namespace GeographicLib {
       sig12 = -1,               // Return value
       // bet12 = bet2 - bet1 in [0, pi); bet12a = bet2 + bet1 in (-pi, 0]
       sbet12 = sbet2 * cbet1 - cbet2 * sbet1,
-      cbet12 = cbet2 * cbet1 + sbet2 * sbet1,
-      sbet12a = sbet2 * cbet1 + cbet2 * sbet1;
-
+      cbet12 = cbet2 * cbet1 + sbet2 * sbet1;
+#if defined(__GNUC__) && __GNUC__ == 4 && \
+  (__GNUC_MINOR__ < 6 || defined(__MINGW32__))
+    // Volatile declaration needed to fix inverse cases
+    // 88.202499451857 0 -88.202499451857 179.981022032992859592
+    // 89.262080389218 0 -89.262080389218 179.992207982775375662
+    // 89.333123580033 0 -89.333123580032997687 179.99295812360148422
+    // which otherwise fail with g++ 4.4.4 x86 -O3 (Linux)
+    // and g++ 4.4.0 (mingw) and g++ 4.6.1 (tdm mingw).
+    real sbet12a;
+    {
+      volatile real xx1 = sbet2 * cbet1;
+      volatile real xx2 = cbet2 * sbet1;
+      sbet12a = xx1 + xx2;
+    }
+#else
+    real sbet12a = sbet2 * cbet1 + cbet2 * sbet1;
+#endif
     bool shortline = cbet12 >= 0 && sbet12 < real(0.5) &&
       lam12 <= Math::pi<real>() / 6;
     real
-      omg12 = shortline ? lam12 / sqrt(1 - _e2 * sq(cbet1)) : lam12,
+      omg12 = (!shortline ? lam12 :
+               lam12 / sqrt(1 - _e2 * Math::sq((cbet1 + cbet2) / 2))),
       somg12 = sin(omg12), comg12 = cos(omg12);
 
     salp1 = cbet2 * somg12;
     calp1 = comg12 >= 0 ?
-      sbet12 + cbet2 * sbet1 * sq(somg12) / (1 + comg12) :
-      sbet12a - cbet2 * sbet1 * sq(somg12) / (1 - comg12);
+      sbet12 + cbet2 * sbet1 * Math::sq(somg12) / (1 + comg12) :
+      sbet12a - cbet2 * sbet1 * Math::sq(somg12) / (1 - comg12);
 
     real
       ssig12 = Math::hypot(salp1, calp1),
@@ -534,23 +569,26 @@ namespace GeographicLib {
     if (shortline && ssig12 < _etol2) {
       // really short lines
       salp2 = cbet1 * somg12;
-      calp2 = sbet12 - cbet1 * sbet2 * sq(somg12) / (1 + comg12);
+      calp2 = sbet12 - cbet1 * sbet2 * Math::sq(somg12) / (1 + comg12);
       SinCosNorm(salp2, calp2);
       // Set return value
       sig12 = atan2(ssig12, csig12);
     } else if (csig12 >= 0 ||
-               ssig12 >= 3 * abs(_f) * Math::pi<real>() * sq(cbet1)) {
+               ssig12 >= 3 * abs(_f) * Math::pi<real>() * Math::sq(cbet1)) {
       // Nothing to do, zeroth order spherical approximation is OK
-
     } else {
       // Scale lam12 and bet2 to x, y coordinate system where antipodal point
       // is at origin and singular point is at y = 0, x = -1.
-      real x, y, lamscale, betscale;
+      real y, lamscale, betscale;
+      // Volatile declaration needed to fix inverse case
+      // 56.320923501171 0 -56.320923501171 179.664747671772880215
+      // which otherwise fails with g++ 4.4.4 x86 -O3
+      volatile real x;
       if (_f >= 0) {            // In fact f == 0 does not get here
         // x = dlong, y = dlat
         {
           real
-            k2 = sq(sbet1) * _ep2,
+            k2 = Math::sq(sbet1) * _ep2,
             eps = k2 / (2 * (1 + sqrt(1 + k2)) + k2);
           lamscale = _f * cbet1 * A3f(eps) * Math::pi<real>();
         }
@@ -563,36 +601,69 @@ namespace GeographicLib {
         real
           cbet12a = cbet2 * cbet1 - sbet2 * sbet1,
           bet12a = atan2(sbet12a, cbet12a);
-        real m0, dummy;
+        real m12a, m0, dummy;
         // In the case of lon12 = 180, this repeats a calculation made in
         // Inverse.
         Lengths(_n, Math::pi<real>() + bet12a, sbet1, -cbet1, sbet2, cbet2,
-                cbet1, cbet2, dummy, x, m0, false, dummy, dummy, C1a, C2a);
-        x = -1 + x/(_f1 * cbet1 * cbet2 * m0 * Math::pi<real>());
+                cbet1, cbet2, dummy, m12a, m0, false,
+                dummy, dummy, C1a, C2a);
+        x = -1 + m12a/(_f1 * cbet1 * cbet2 * m0 * Math::pi<real>());
         betscale = x < -real(0.01) ? sbet12a / x :
-          -_f * sq(cbet1) * Math::pi<real>();
+          -_f * Math::sq(cbet1) * Math::pi<real>();
         lamscale = betscale / cbet1;
         y = (lam12 - Math::pi<real>()) / lamscale;
       }
 
-      if (y > -tol1 && x >  -1 - xthresh) {
+      if (y > -tol1_ && x > -1 - xthresh_) {
         // strip near cut
         if (_f >= 0) {
-          salp1 = min(real( 1), -x); calp1 = - sqrt(1 - sq(salp1));
+          salp1 = min(real(1), -real(x)); calp1 = - sqrt(1 - Math::sq(salp1));
         } else {
-          calp1 = max(real(x > -tol1 ? 0 : -1),  x);
-          salp1 = sqrt(1 - sq(calp1));
+          calp1 = max(real(x > -tol1_ ? 0 : -1), real(x));
+          salp1 = sqrt(1 - Math::sq(calp1));
         }
       } else {
-        // Estimate omega12, by solving the astroid problem.
+        // Estimate alp1, by solving the astroid problem.
+        //
+        // Could estimate alpha1 = theta + pi/2, directly, i.e.,
+        //   calp1 = y/k; salp1 = -x/(1+k);  for _f >= 0
+        //   calp1 = x/(1+k); salp1 = -y/k;  for _f < 0 (need to check)
+        //
+        // However, it's better to estimate omg12 from astroid and use
+        // spherical formula to compute alp1.  This reduces the mean number of
+        // Newton iterations for astroid cases from 2.24 (min 0, max 6) to 2.12
+        // (min 0 max 5).  The changes in the number of iterations are as
+        // follows:
+        //
+        // change percent
+        //    1       5
+        //    0      78
+        //   -1      16
+        //   -2       0.6
+        //   -3       0.04
+        //   -4       0.002
+        //
+        // The histogram of iterations is (m = number of iterations estimating
+        // alp1 directly, n = number of iterations estimating via omg12, total
+        // number of trials = 148605):
+        //
+        //  iter    m      n
+        //    0   148    186
+        //    1 13046  13845
+        //    2 93315 102225
+        //    3 36189  32341
+        //    4  5396      7
+        //    5   455      1
+        //    6    56      0
+        //
+        // Because omg12 is near pi, estimate work with omg12a = pi - omg12
         real k = Astroid(x, y);
-        // estimate omg12a = pi - omg12
         real
           omg12a = lamscale * ( _f >= 0 ? -x * k/(1 + k) : -y * (1 + k)/k ),
           somg12 = sin(omg12a), comg12 = -cos(omg12a);
         // Update spherical estimate of alp1 using omg12 instead of lam12
         salp1 = cbet2 * somg12;
-        calp1 = sbet12a - cbet2 * sbet1 * sq(somg12) / (1 - comg12);
+        calp1 = sbet12a - cbet2 * sbet1 * Math::sq(somg12) / (1 - comg12);
       }
     }
     SinCosNorm(salp1, calp1);
@@ -614,10 +685,10 @@ namespace GeographicLib {
     if (sbet1 == 0 && calp1 == 0)
       // Break degeneracy of equatorial line.  This case has already been
       // handled.
-      calp1 = -eps2;
+      calp1 = -tiny_;
 
     real
-      // sin(alp1) * cos(bet1) = sin(alp0),
+      // sin(alp1) * cos(bet1) = sin(alp0)
       salp0 = salp1 * cbet1,
       calp0 = Math::hypot(calp1, salp1 * sbet1); // calp0 > 0
 
@@ -627,28 +698,29 @@ namespace GeographicLib {
     ssig1 = sbet1; somg1 = salp0 * sbet1;
     csig1 = comg1 = calp1 * cbet1;
     SinCosNorm(ssig1, csig1);
-    SinCosNorm(somg1, comg1);
+    // SinCosNorm(somg1, comg1); -- don't need to normalize!
 
     // Enforce symmetries in the case abs(bet2) = -bet1.  Need to be careful
     // about this case, since this can yield singularities in the Newton
     // iteration.
-    // sin(alp2) * cos(bet2) = sin(alp0),
+    // sin(alp2) * cos(bet2) = sin(alp0)
     salp2 = cbet2 != cbet1 ? salp0 / cbet2 : salp1;
     // calp2 = sqrt(1 - sq(salp2))
     //       = sqrt(sq(calp0) - sq(sbet2)) / cbet2
     // and subst for calp0 and rearrange to give (choose positive sqrt
     // to give alp2 in [0, pi/2]).
     calp2 = cbet2 != cbet1 || abs(sbet2) != -sbet1 ?
-      sqrt(sq(calp1 * cbet1) + (cbet1 < -sbet1 ?
-                                (cbet2 - cbet1) * (cbet1 + cbet2) :
-                                (sbet1 - sbet2) * (sbet1 + sbet2))) / cbet2 :
+      sqrt(Math::sq(calp1 * cbet1) +
+           (cbet1 < -sbet1 ?
+            (cbet2 - cbet1) * (cbet1 + cbet2) :
+            (sbet1 - sbet2) * (sbet1 + sbet2))) / cbet2 :
       abs(calp1);
     // tan(bet2) = tan(sig2) * cos(alp2)
     // tan(omg2) = sin(alp0) * tan(sig2).
     ssig2 = sbet2; somg2 = salp0 * sbet2;
     csig2 = comg2 = calp2 * cbet2;
     SinCosNorm(ssig2, csig2);
-    SinCosNorm(somg2, comg2);
+    // SinCosNorm(somg2, comg2); -- don't need to normalize!
 
     // sig12 = sig2 - sig1, limit to [0, pi]
     sig12 = atan2(max(csig1 * ssig2 - ssig1 * csig2, real(0)),
@@ -658,23 +730,23 @@ namespace GeographicLib {
     omg12 = atan2(max(comg1 * somg2 - somg1 * comg2, real(0)),
                   comg1 * comg2 + somg1 * somg2);
     real B312, h0;
-    real k2 = sq(calp0) * _ep2;
+    real k2 = Math::sq(calp0) * _ep2;
     eps = k2 / (2 * (1 + sqrt(1 + k2)) + k2);
     C3f(eps, C3a);
-    B312 = (SinCosSeries(true, ssig2, csig2, C3a, nC3-1) -
-            SinCosSeries(true, ssig1, csig1, C3a, nC3-1));
+    B312 = (SinCosSeries(true, ssig2, csig2, C3a, nC3_-1) -
+            SinCosSeries(true, ssig1, csig1, C3a, nC3_-1));
     h0 = -_f * A3f(eps);
     domg12 = salp0 * h0 * (sig12 + B312);
     lam12 = omg12 + domg12;
 
     if (diffp) {
       if (calp2 == 0)
-        dlam12 = - 2 * sqrt(1 - _e2 * sq(cbet1)) / sbet1;
+        dlam12 = - 2 * sqrt(1 - _e2 * Math::sq(cbet1)) / sbet1;
       else {
         real dummy;
         Lengths(eps, sig12, ssig1, csig1, ssig2, csig2,
                 cbet1, cbet2, dummy, dlam12, dummy,
-                false, dummy, dummy,  C1a, C2a);
+                false, dummy, dummy, C1a, C2a);
         dlam12 /= calp2 * cbet2;
       }
     }
@@ -683,25 +755,25 @@ namespace GeographicLib {
   }
 
   Math::real Geodesic::A3f(real eps) const throw() {
-    // Evaluation sum(_A3c[k] * eps^k, k, 0, nA3x-1) by Horner's method
+    // Evaluation sum(_A3c[k] * eps^k, k, 0, nA3x_-1) by Horner's method
     real v = 0;
-    for (int i = nA3x; i; )
+    for (int i = nA3x_; i; )
       v = eps * v + _A3x[--i];
     return v;
   }
 
   void Geodesic::C3f(real eps, real c[]) const throw() {
     // Evaluation C3 coeffs by Horner's method
-    // Elements c[1] thru c[nC3 - 1] are set
-    for (int j = nC3x, k = nC3 - 1; k; ) {
+    // Elements c[1] thru c[nC3_ - 1] are set
+    for (int j = nC3x_, k = nC3_ - 1; k; ) {
       real t = 0;
-      for (int i = nC3 - k; i; --i)
+      for (int i = nC3_ - k; i; --i)
         t = eps * t + _C3x[--j];
       c[k--] = t;
     }
 
     real mult = 1;
-    for (int k = 1; k < nC3; ) {
+    for (int k = 1; k < nC3_; ) {
       mult *= eps;
       c[k++] *= mult;
     }
@@ -709,16 +781,16 @@ namespace GeographicLib {
 
   void Geodesic::C4f(real k2, real c[]) const throw() {
     // Evaluation C4 coeffs by Horner's method
-    // Elements c[0] thru c[nC4 - 1] are set
-    for (int j = nC4x, k = nC4; k; ) {
+    // Elements c[0] thru c[nC4_ - 1] are set
+    for (int j = nC4x_, k = nC4_; k; ) {
       real t = 0;
-      for (int i = nC4 - k + 1; i; --i)
+      for (int i = nC4_ - k + 1; i; --i)
         t = k2 * t + _C4x[--j];
       c[--k] = t;
     }
 
     real mult = 1;
-    for (int k = 1; k < nC4; ) {
+    for (int k = 1; k < nC4_; ) {
       mult *= k2;
       c[k++] *= mult;
     }
@@ -729,9 +801,9 @@ namespace GeographicLib {
   // The scale factor A1-1 = mean value of I1-1
   Math::real Geodesic::A1m1f(real eps) throw() {
     real
-      eps2 = sq(eps),
+      eps2 = Math::sq(eps),
       t;
-    switch (nA1/2) {
+    switch (nA1_/2) {
     case 0:
       t = 0;
       break;
@@ -748,7 +820,7 @@ namespace GeographicLib {
       t = eps2*(eps2*(eps2*(25*eps2+64)+256)+4096)/16384;
       break;
     default:
-      STATIC_ASSERT(nA1 >= 0 && nA1 <= 8, "Bad value of nA1");
+      STATIC_ASSERT(nA1_ >= 0 && nA1_ <= 8, "Bad value of nA1_");
       t = 0;
     }
     return (t + eps) / (1 - eps);
@@ -757,9 +829,9 @@ namespace GeographicLib {
   // The coefficients C1[l] in the Fourier expansion of B1
   void Geodesic::C1f(real eps, real c[]) throw() {
     real
-      eps2 = sq(eps),
+      eps2 = Math::sq(eps),
       d = eps;
-    switch (nC1) {
+    switch (nC1_) {
     case 0:
       break;
     case 1:
@@ -843,16 +915,16 @@ namespace GeographicLib {
       c[8] = -429*d/262144;
       break;
     default:
-      STATIC_ASSERT(nC1 >= 0 && nC1 <= 8, "Bad value of nC1");
+      STATIC_ASSERT(nC1_ >= 0 && nC1_ <= 8, "Bad value of nC1_");
     }
   }
 
   // The coefficients C1p[l] in the Fourier expansion of B1p
   void Geodesic::C1pf(real eps, real c[]) throw() {
     real
-      eps2 = sq(eps),
+      eps2 = Math::sq(eps),
       d = eps;
-    switch (nC1p) {
+    switch (nC1p_) {
     case 0:
       break;
     case 1:
@@ -936,16 +1008,16 @@ namespace GeographicLib {
       c[8] = 109167851*d/82575360;
       break;
     default:
-      STATIC_ASSERT(nC1p >= 0 && nC1p <= 8, "Bad value of nC1p");
+      STATIC_ASSERT(nC1p_ >= 0 && nC1p_ <= 8, "Bad value of nC1p_");
     }
   }
 
   // The scale factor A2-1 = mean value of I2-1
   Math::real Geodesic::A2m1f(real eps) throw() {
     real
-      eps2 = sq(eps),
+      eps2 = Math::sq(eps),
       t;
-    switch (nA2/2) {
+    switch (nA2_/2) {
     case 0:
       t = 0;
       break;
@@ -962,7 +1034,7 @@ namespace GeographicLib {
       t = eps2*(eps2*(eps2*(1225*eps2+1600)+2304)+4096)/16384;
       break;
     default:
-      STATIC_ASSERT(nA2 >= 0 && nA2 <= 8, "Bad value of nA2");
+      STATIC_ASSERT(nA2_ >= 0 && nA2_ <= 8, "Bad value of nA2_");
       t = 0;
     }
     return t * (1 - eps) - eps;
@@ -971,9 +1043,9 @@ namespace GeographicLib {
   // The coefficients C2[l] in the Fourier expansion of B2
   void Geodesic::C2f(real eps, real c[]) throw() {
     real
-      eps2 = sq(eps),
+      eps2 = Math::sq(eps),
       d = eps;
-    switch (nC2) {
+    switch (nC2_) {
     case 0:
       break;
     case 1:
@@ -1057,13 +1129,13 @@ namespace GeographicLib {
       c[8] = 6435*d/262144;
       break;
     default:
-      STATIC_ASSERT(nC2 >= 0 && nC2 <= 8, "Bad value of nC2");
+      STATIC_ASSERT(nC2_ >= 0 && nC2_ <= 8, "Bad value of nC2_");
     }
   }
 
   // The scale factor A3 = mean value of I3
   void Geodesic::A3coeff() throw() {
-    switch (nA3) {
+    switch (nA3_) {
     case 0:
       break;
     case 1:
@@ -1119,13 +1191,13 @@ namespace GeographicLib {
       _A3x[7] = -25/real(2048);
       break;
     default:
-      STATIC_ASSERT(nA3 >= 0 && nA3 <= 8, "Bad value of nA3");
+      STATIC_ASSERT(nA3_ >= 0 && nA3_ <= 8, "Bad value of nA3_");
     }
   }
 
   // The coefficients C3[l] in the Fourier expansion of B3
   void Geodesic::C3coeff() throw() {
-    switch (nC3) {
+    switch (nC3_) {
     case 0:
       break;
     case 1:
@@ -1229,13 +1301,13 @@ namespace GeographicLib {
       _C3x[27] = 429/real(114688);
       break;
     default:
-      STATIC_ASSERT(nC3 >= 0 && nC3 <= 8, "Bad value of nC3");
+      STATIC_ASSERT(nC3_ >= 0 && nC3_ <= 8, "Bad value of nC3_");
     }
   }
 
   // The coefficients C4[l] in the Fourier expansion of I4
   void Geodesic::C4coeff() throw() {
-    switch (nC4) {
+    switch (nC4_) {
     case 0:
       break;
     case 1:
@@ -1385,7 +1457,8 @@ namespace GeographicLib {
       _C4x[35] = 1/real(26254800);
       break;
     default:
-      STATIC_ASSERT(nC3 >= 0 && nC4 <= 8, "Bad value of nC4");
+      STATIC_ASSERT(nC3_ >= 0 && nC4_ <= 8, "Bad value of nC4_");
     }
   }
+
 } // namespace GeographicLib
