@@ -2,34 +2,38 @@
  * \file Geoid.cpp
  * \brief Implementation for GeographicLib::Geoid class
  *
- * Copyright (c) Charles Karney (2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2009-2012) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/Geoid.hpp"
+#include <GeographicLib/Geoid.hpp>
 #include <sstream>
 #include <cstdlib>
 #include <algorithm>
+#include <GeographicLib/Utility.hpp>
 
-#define GEOGRAPHICLIB_GEOID_CPP "$Id: Geoid.cpp 6967 2011-02-19 15:53:41Z karney $"
+#define GEOGRAPHICLIB_GEOID_CPP \
+  "$Id: 5c3c23dd877485af9c9e298ddb28c5aac12b5e6a $"
 
 RCSID_DECL(GEOGRAPHICLIB_GEOID_CPP)
 RCSID_DECL(GEOGRAPHICLIB_GEOID_HPP)
 
-#if !defined(GEOID_DEFAULT_PATH)
-#if defined(_MSC_VER)
-#define GEOID_DEFAULT_PATH "C:/cygwin/usr/local/share/GeographicLib/geoids"
-#else
-#define GEOID_DEFAULT_PATH "/usr/local/share/GeographicLib/geoids"
-#endif
+#if !defined(GEOGRAPHICLIB_DATA)
+#  if defined(_MSC_VER)
+#    define GEOGRAPHICLIB_DATA \
+  "C:/Documents and Settings/All Users/Application Data/GeographicLib"
+#  else
+#    define GEOGRAPHICLIB_DATA "/usr/local/share/GeographicLib"
+#  endif
 #endif
+
 #if !defined(GEOID_DEFAULT_NAME)
-#define GEOID_DEFAULT_NAME "egm96-5"
+#  define GEOID_DEFAULT_NAME "egm96-5"
 #endif
 
 #if defined(_MSC_VER)
-// Squelch warnings about unsafe use of getenv and copy
+// Squelch warnings about unsafe use of getenv
 #pragma warning (disable: 4996)
 #endif
 
@@ -101,8 +105,8 @@ namespace GeographicLib {
   // genmatrix(yc,1,length(warr)).abs(c3).genmatrix(yd,length(pows),1)),2)$
   // c3:c0*c3$
 
-  const Math::real Geoid::c0 = 240; // Common denominator
-  const Math::real Geoid::c3[stencilsize * nterms] = {
+  const Math::real Geoid::c0_ = 240; // Common denominator
+  const Math::real Geoid::c3_[stencilsize_ * nterms_] = {
       9, -18, -88,    0,  96,   90,   0,   0, -60, -20,
      -9,  18,   8,    0, -96,   30,   0,   0,  60, -20,
       9, -88, -18,   90,  96,    0, -20, -60,   0,   0,
@@ -151,8 +155,8 @@ namespace GeographicLib {
   //     genmatrix(yc,1,length(warr)).abs(c3n).genmatrix(yd,length(pows),1)),2)$
   // c3n:c0n*c3n$
 
-  const Math::real Geoid::c0n = 372; // Common denominator
-  const Math::real Geoid::c3n[stencilsize * nterms] = {
+  const Math::real Geoid::c0n_ = 372; // Common denominator
+  const Math::real Geoid::c3n_[stencilsize_ * nterms_] = {
       0, 0, -131, 0,  138,  144, 0,   0, -102, -31,
       0, 0,    7, 0, -138,   42, 0,   0,  102, -31,
      62, 0,  -31, 0,    0,  -62, 0,   0,    0,  31,
@@ -185,8 +189,8 @@ namespace GeographicLib {
   //     genmatrix(yc,1,length(warr)).abs(c3s).genmatrix(yd,length(pows),1)),2)$
   // c3s:c0s*c3s$
 
-  const Math::real Geoid::c0s = 372; // Common denominator
-  const Math::real Geoid::c3s[stencilsize * nterms] = {
+  const Math::real Geoid::c0s_ = 372; // Common denominator
+  const Math::real Geoid::c3s_[stencilsize_ * nterms_] = {
      18,  -36, -122,   0,  120,  135, 0,   0,  -84, -31,
     -18,   36,   -2,   0, -120,   51, 0,   0,   84, -31,
      36, -165,  -27,  93,  147,   -9, 0, -93,   18,   0,
@@ -207,14 +211,15 @@ namespace GeographicLib {
     , _dir(path)
     , _cubic(cubic)
     , _a( Constants::WGS84_a<real>() )
-    , _e2( (2 - 1/Constants::WGS84_r<real>())/Constants::WGS84_r<real>() )
+    , _e2( (2 - Constants::WGS84_f<real>()) * Constants::WGS84_f<real>() )
     , _degree( Math::degree<real>() )
     , _eps( sqrt(numeric_limits<real>::epsilon()) )
     , _threadsafe(false)        // Set after cache is read
   {
+    STATIC_ASSERT(sizeof(pixel_t) == pixel_size_, "pixel_t has the wrong size");
     if (_dir.empty())
       _dir = DefaultGeoidPath();
-    _filename = _dir + "/" + _name + ".pgm";
+    _filename = _dir + "/" + _name + (pixel_size_ != 4 ? ".pgm" : ".pgm4");
     _file.open(_filename.c_str(), ios::binary);
     if (!(_file.good()))
       throw GeographicErr("File not readable " + _filename);
@@ -263,8 +268,8 @@ namespace GeographicLib {
       unsigned maxval;
       if (!(_file >> maxval))
         throw GeographicErr("Error reading maxval " + _filename);
-      if (maxval != 0xffffu)
-        throw GeographicErr("Maxval not equal to 2^16-1 " + _filename);
+      if (maxval != pixel_max_)
+        throw GeographicErr("Incorrect value of maxval " + _filename);
       // Add 1 for whitespace after maxval
       _datastart = (unsigned long long)(_file.tellg()) + 1ULL;
       _swidth = (unsigned long long)(_width);
@@ -286,7 +291,7 @@ namespace GeographicLib {
       throw GeographicErr("Raster height is even " + _filename);
     _file.seekg(0, ios::end);
     if (!_file.good() ||
-        _datastart + 2ULL * _swidth * (unsigned long long)(_height) !=
+        _datastart + pixel_size_ * _swidth * (unsigned long long)(_height) !=
         (unsigned long long)(_file.tellg()))
       // Possibly this test should be "<" because the file contains, e.g., a
       // second image.  However, for now we are more strict.
@@ -308,8 +313,8 @@ namespace GeographicLib {
   Math::real Geoid::height(real lat, real lon, bool gradp,
                            real& gradn, real& grade) const {
     if (Math::isnan(lat) || Math::isnan(lon)) {
-      if (gradp) gradn = grade = Math::NaN();
-      return Math::NaN();
+      if (gradp) gradn = grade = Math::NaN<real>();
+      return Math::NaN<real>();
     }
     real
       fx =  lon * _rlonres,
@@ -320,9 +325,9 @@ namespace GeographicLib {
     fx -= ix;
     fy -= iy;
     iy += (_height - 1)/2;
-    ix += ix < 0 ? _width : ix >= _width ? -_width : 0;
+    ix += ix < 0 ? _width : (ix >= _width ? -_width : 0);
     real v00 = 0, v01 = 0, v10 = 0, v11 = 0;
-    real t[nterms];
+    real t[nterms_];
 
     if (_threadsafe || !(ix == _ix && iy == _iy)) {
       if (!_cubic) {
@@ -331,7 +336,7 @@ namespace GeographicLib {
         v10 = rawval(ix    , iy + 1);
         v11 = rawval(ix + 1, iy + 1);
       } else {
-        real v[stencilsize];
+        real v[stencilsize_];
         int k = 0;
         v[k++] = rawval(ix    , iy - 1);
         v[k++] = rawval(ix + 1, iy - 1);
@@ -346,12 +351,12 @@ namespace GeographicLib {
         v[k++] = rawval(ix    , iy + 2);
         v[k++] = rawval(ix + 1, iy + 2);
 
-        const real* c3x = iy == 0 ? c3n : iy == _height - 2 ? c3s : c3;
-        real c0x = iy == 0 ? c0n : iy == _height - 2 ? c0s : c0;
-        for (unsigned i = 0; i < nterms; ++i) {
+        const real* c3x = iy == 0 ? c3n_ : (iy == _height - 2 ? c3s_ : c3_);
+        real c0x = iy == 0 ? c0n_ : (iy == _height - 2 ? c0s_ : c0_);
+        for (unsigned i = 0; i < nterms_; ++i) {
           t[i] = 0;
-          for (unsigned j = 0; j < stencilsize; ++j)
-            t[i] += v[j] * c3x[nterms * j + i];
+          for (unsigned j = 0; j < stencilsize_; ++j)
+            t[i] += v[j] * c3x[nterms_ * j + i];
           t[i] /= c0x;
         }
       }
@@ -362,7 +367,7 @@ namespace GeographicLib {
         v10 = _v10;
         v11 = _v11;
       } else
-        copy(_t, _t + nterms, t);
+        copy(_t, _t + nterms_, t);
     }
     if (!_cubic) {
       real
@@ -405,7 +410,7 @@ namespace GeographicLib {
         lat = min(lat,  90 - 1/(100 * _rlatres));
         lat = max(lat, -90 + 1/(100 * _rlatres));
         fy = (90 - lat) * _rlatres;
-        fy -=  int(fy);
+        fy -= int(fy);
         real
           phi = lat * _degree,
           cosphi = cos(phi),
@@ -421,7 +426,7 @@ namespace GeographicLib {
       if (!_threadsafe) {
         _ix = ix;
         _iy = iy;
-        copy(t, t + nterms, _t);
+        copy(t, t + nterms_, _t);
       }
       return h;
     }
@@ -433,7 +438,7 @@ namespace GeographicLib {
       try {
         _data.clear();
         // Use swap to release memory back to system
-        vector< vector<unsigned short> >().swap(_data);
+        vector< vector<pixel_t> >().swap(_data);
       }
       catch (const exception&) {
       }
@@ -473,8 +478,8 @@ namespace GeographicLib {
       iw = 0;
       ie = _width - 1;
     } else {
-      ie += iw < 0 ? _width : iw >= _width ? -_width : 0;
-      iw += iw < 0 ? _width : iw >= _width ? -_width : 0;
+      ie += iw < 0 ? _width : (iw >= _width ? -_width : 0);
+      iw += iw < 0 ? _width : (iw >= _width ? -_width : 0);
     }
     int oysize = int(_data.size());
     _xsize = ie - iw + 1;
@@ -483,7 +488,7 @@ namespace GeographicLib {
     _yoffset = in;
 
     try {
-      _data.resize(_ysize, vector<unsigned short>(_xsize));
+      _data.resize(_ysize, vector<pixel_t>(_xsize));
       for (int iy = min(oysize, _ysize); iy--;)
         _data[iy].resize(_xsize);
     }
@@ -493,10 +498,10 @@ namespace GeographicLib {
     }
 
     try {
-      vector<char> buf(2 * _xsize);
       for (int iy = in; iy <= is; ++iy) {
         int iy1 = iy, iw1 = iw;
         if (iy < 0 || iy >= _height) {
+          // Allow points "beyond" the poles to support interpolation
           iy1 = iy1 < 0 ? -iy1 : 2 * (_height - 1) - iy1;
           iw1 += _width/2;
           if (iw1 >= _width)
@@ -504,16 +509,14 @@ namespace GeographicLib {
         }
         int xs1 = min(_width - iw1, _xsize);
         filepos(iw1, iy1);
-        _file.read(&(buf[0]), 2 * xs1);
+        Utility::readarray<pixel_t, pixel_t, true>
+          (_file, &(_data[iy - in][0]), xs1);
         if (xs1 < _xsize) {
           // Wrap around longitude = 0
           filepos(0, iy1);
-          _file.read(&(buf[2 * xs1]), 2 * (_xsize - xs1));
+          Utility::readarray<pixel_t, pixel_t, true>
+            (_file, &(_data[iy - in][xs1]), _xsize - xs1);
         }
-        for (int ix = 0; ix < _xsize; ++ix)
-          _data[iy - in][ix] =
-            (unsigned short)((unsigned char)buf[2 * ix] * 256u +
-                             (unsigned char)buf[2 * ix + 1]);
       }
       _cache = true;
     }
@@ -523,24 +526,17 @@ namespace GeographicLib {
     }
   }
 
-  std::string Geoid::DefaultPath() {
-    return string(GEOID_DEFAULT_PATH);
-  }
-
-  std::string Geoid::GeoidPath() {
-    string path;
-    char* geoidpath = getenv("GEOID_PATH");
-    if (geoidpath)
-      path = string(geoidpath);
-    return path;
-  }
-
   std::string Geoid::DefaultGeoidPath() {
     string path;
     char* geoidpath = getenv("GEOID_PATH");
     if (geoidpath)
       path = string(geoidpath);
-    return path.length() ? path : string(GEOID_DEFAULT_PATH);
+    if (path.length())
+      return path;
+    char* datapath = getenv("GEOGRAPHICLIB_DATA");
+    if (datapath)
+      path = string(datapath);
+    return (path.length() ? path : string(GEOGRAPHICLIB_DATA)) + "/geoids";
   }
 
   std::string Geoid::DefaultGeoidName() {
@@ -550,4 +546,5 @@ namespace GeographicLib {
       name = string(geoidname);
     return name.length() ? name : string(GEOID_DEFAULT_NAME);
   }
-}
+
+} // namespace GeographicLib
