@@ -2,14 +2,15 @@
  * \file LocalCartesian.cpp
  * \brief Implementation for GeographicLib::LocalCartesian class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010, 2011) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2011) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/LocalCartesian.hpp"
+#include <GeographicLib/LocalCartesian.hpp>
 
-#define GEOGRAPHICLIB_LOCALCARTESIAN_CPP "$Id: LocalCartesian.cpp 6952 2011-02-14 20:26:44Z karney $"
+#define GEOGRAPHICLIB_LOCALCARTESIAN_CPP \
+  "$Id: 4d15764c089e07855bf6db300271e18f4fa89624 $"
 
 RCSID_DECL(GEOGRAPHICLIB_LOCALCARTESIAN_CPP)
 RCSID_DECL(GEOGRAPHICLIB_LOCALCARTESIAN_HPP)
@@ -20,7 +21,7 @@ namespace GeographicLib {
 
   void LocalCartesian::Reset(real lat0, real lon0, real h0) throw() {
     _lat0 = lat0;
-    _lon0 = lon0 >= 180 ? lon0 - 360 : lon0 < -180 ? lon0 + 360 : lon0;
+    _lon0 = lon0 >= 180 ? lon0 - 360 : (lon0 < -180 ? lon0 + 360 : lon0);
     _h0 = h0;
     _earth.Forward(_lat0, _lon0, _h0, _x0, _y0, _z0);
     real
@@ -33,18 +34,18 @@ namespace GeographicLib {
     _earth.Rotation(sphi, cphi, slam, clam, _r);
   }
 
-  void LocalCartesian::MatrixMultiply(real M[dim2]) const throw() {
-    real t[dim2];
-    copy(M, M + dim2, t);
-    for (size_t i = 0; i < dim2; ++i) {
-      size_t row = i / dim, col = i % dim;
+  void LocalCartesian::MatrixMultiply(real M[dim2_]) const throw() {
+    real t[dim2_];
+    copy(M, M + dim2_, t);
+    for (size_t i = 0; i < dim2_; ++i) {
+      size_t row = i / dim_, col = i % dim_;
       M[i] = _r[row] * t[col] + _r[row+3] * t[col+3] + _r[row+6] * t[col+6];
     }
   }
 
   void LocalCartesian::IntForward(real lat, real lon, real h,
                                   real& x, real& y, real& z,
-                                  real M[dim2]) const throw() {
+                                  real M[dim2_]) const throw() {
     real xc, yc, zc;
     _earth.IntForward(lat, lon, h, xc, yc, zc, M);
     xc -= _x0; yc -= _y0; zc -= _z0;
@@ -57,7 +58,7 @@ namespace GeographicLib {
 
   void LocalCartesian::IntReverse(real x, real y, real z,
                                   real& lat, real& lon, real& h,
-                                  real M[dim2]) const throw() {
+                                  real M[dim2_]) const throw() {
     real
       xc = _x0 + _r[0] * x + _r[1] * y + _r[2] * z,
       yc = _y0 + _r[3] * x + _r[4] * y + _r[5] * z,
