@@ -2,18 +2,20 @@
  * \file UTMUPS.cpp
  * \brief Implementation for GeographicLib::UTMUPS class
  *
- * Copyright (c) Charles Karney (2008, 2009, 2010) <charles@karney.com>
- * and licensed under the LGPL.  For more information, see
+ * Copyright (c) Charles Karney (2008-2012) <charles@karney.com> and licensed
+ * under the MIT/X11 License.  For more information, see
  * http://geographiclib.sourceforge.net/
  **********************************************************************/
 
-#include "GeographicLib/UTMUPS.hpp"
-#include "GeographicLib/MGRS.hpp"
-#include "GeographicLib/PolarStereographic.hpp"
-#include "GeographicLib/TransverseMercator.hpp"
+#include <GeographicLib/UTMUPS.hpp>
 #include <iomanip>
+#include <GeographicLib/MGRS.hpp>
+#include <GeographicLib/PolarStereographic.hpp>
+#include <GeographicLib/TransverseMercator.hpp>
+#include <GeographicLib/Utility.hpp>
 
-#define GEOGRAPHICLIB_UTMUPS_CPP "$Id: UTMUPS.cpp 6918 2010-12-21 12:56:07Z karney $"
+#define GEOGRAPHICLIB_UTMUPS_CPP \
+  "$Id: 5672b003ee47cd660377c111e3fca2b81da86323 $"
 
 RCSID_DECL(GEOGRAPHICLIB_UTMUPS_CPP)
 RCSID_DECL(GEOGRAPHICLIB_UTMUPS_HPP)
@@ -22,30 +24,31 @@ namespace GeographicLib {
 
   using namespace std;
 
-  const Math::real UTMUPS::falseeasting[4] =
-    { MGRS::upseasting * MGRS::tile, MGRS::upseasting * MGRS::tile,
-      MGRS::utmeasting * MGRS::tile, MGRS::utmeasting* MGRS::tile };
-  const Math::real UTMUPS::falsenorthing[4] =
-    { MGRS::upseasting * MGRS::tile, MGRS::upseasting * MGRS::tile,
-      MGRS::maxutmSrow * MGRS::tile, MGRS::minutmNrow * MGRS::tile };
-  const Math::real UTMUPS::mineasting[4] =
-    { MGRS::minupsSind * MGRS::tile,  MGRS::minupsNind * MGRS::tile,
-      MGRS::minutmcol * MGRS::tile,  MGRS::minutmcol * MGRS::tile };
-  const Math::real UTMUPS::maxeasting[4] =
-    { MGRS::maxupsSind * MGRS::tile,  MGRS::maxupsNind * MGRS::tile,
-      MGRS::maxutmcol * MGRS::tile,  MGRS::maxutmcol * MGRS::tile };
-  const Math::real UTMUPS::minnorthing[4] =
-    { MGRS::minupsSind * MGRS::tile,  MGRS::minupsNind * MGRS::tile,
-      MGRS::minutmSrow * MGRS::tile,
-      (MGRS::minutmNrow + MGRS::minutmSrow - MGRS::maxutmSrow) * MGRS::tile };
-  const Math::real UTMUPS::maxnorthing[4] =
-    { MGRS::maxupsSind * MGRS::tile,  MGRS::maxupsNind * MGRS::tile,
-      (MGRS::maxutmSrow + MGRS::maxutmNrow - MGRS::minutmNrow) * MGRS::tile,
-      MGRS::maxutmNrow * MGRS::tile };
+  const Math::real UTMUPS::falseeasting_[4] =
+    { MGRS::upseasting_ * MGRS::tile_, MGRS::upseasting_ * MGRS::tile_,
+      MGRS::utmeasting_ * MGRS::tile_, MGRS::utmeasting_ * MGRS::tile_ };
+  const Math::real UTMUPS::falsenorthing_[4] =
+    { MGRS::upseasting_ * MGRS::tile_, MGRS::upseasting_ * MGRS::tile_,
+      MGRS::maxutmSrow_ * MGRS::tile_, MGRS::minutmNrow_ * MGRS::tile_ };
+  const Math::real UTMUPS::mineasting_[4] =
+    { MGRS::minupsSind_ * MGRS::tile_, MGRS::minupsNind_ * MGRS::tile_,
+      MGRS::minutmcol_ * MGRS::tile_, MGRS::minutmcol_ * MGRS::tile_ };
+  const Math::real UTMUPS::maxeasting_[4] =
+    { MGRS::maxupsSind_ * MGRS::tile_, MGRS::maxupsNind_ * MGRS::tile_,
+      MGRS::maxutmcol_ * MGRS::tile_, MGRS::maxutmcol_ * MGRS::tile_ };
+  const Math::real UTMUPS::minnorthing_[4] =
+    { MGRS::minupsSind_ * MGRS::tile_, MGRS::minupsNind_ * MGRS::tile_,
+      MGRS::minutmSrow_ * MGRS::tile_,
+      (MGRS::minutmNrow_ + MGRS::minutmSrow_ - MGRS::maxutmSrow_)
+      * MGRS::tile_ };
+  const Math::real UTMUPS::maxnorthing_[4] =
+    { MGRS::maxupsSind_ * MGRS::tile_, MGRS::maxupsNind_ * MGRS::tile_,
+      (MGRS::maxutmSrow_ + MGRS::maxutmNrow_ - MGRS::minutmNrow_) * MGRS::tile_,
+      MGRS::maxutmNrow_ * MGRS::tile_ };
 
   int UTMUPS::StandardZone(real lat, real lon, int setzone) {
-    if (setzone < MINPSEUDOZONE || setzone > MAXZONE)
-      throw GeographicErr("Illegal zone requested " + str(setzone));
+    if (!(setzone >= MINPSEUDOZONE && setzone <= MAXZONE))
+      throw GeographicErr("Illegal zone requested " + Utility::str(setzone));
     if (setzone >= MINZONE || setzone == INVALID)
       return setzone;
     if (Math::isnan(lat) || Math::isnan(lon)) // Check if lat or lon is a NaN
@@ -77,7 +80,7 @@ namespace GeographicLib {
     if (zone1 == INVALID) {
       zone = zone1;
       northp = northp1;
-      x = y = gamma = k = Math::NaN();
+      x = y = gamma = k = Math::NaN<real>();
       return;
     }
     real x1, y1, gamma1, k1;
@@ -90,24 +93,26 @@ namespace GeographicLib {
       if (dlon > 60)
         // Check isn't really necessary because CheckCoords catches this case.
         // But this allows a more meaningful error message to be given.
-        throw GeographicErr("Longitude " + str(lon)
+        throw GeographicErr("Longitude " + Utility::str(lon)
                             + "d more than 60d from center of UTM zone "
-                            + str(zone1));
+                            + Utility::str(zone1));
       TransverseMercator::UTM.Forward(lon0, lat, lon, x1, y1, gamma1, k1);
     } else {
       if (abs(lat) < 70)
         // Check isn't really necessary ... (see above).
-        throw GeographicErr("Latitude " + str(lat) + "d more than 20d from "
+        throw GeographicErr("Latitude " + Utility::str(lat)
+                            + "d more than 20d from "
                             + (northp1 ? "N" : "S") + " pole");
       PolarStereographic::UPS.Forward(northp1, lat, lon, x1, y1, gamma1, k1);
     }
     int ind = (utmp ? 2 : 0) + (northp1 ? 1 : 0);
-    x1 += falseeasting[ind];
-    y1 += falsenorthing[ind];
+    x1 += falseeasting_[ind];
+    y1 += falsenorthing_[ind];
     if (! CheckCoords(zone1 != UPS, northp1, x1, y1, mgrslimits, false) )
-      throw GeographicErr("Latitude " + str(lat) + ", longitude " + str(lon)
+      throw GeographicErr("Latitude " + Utility::str(lat)
+                          + ", longitude " + Utility::str(lon)
                           + " out of legal range for "
-                          + (utmp ? "UTM zone " + str(zone1) : "UPS"));
+                          + (utmp ? "UTM zone " + Utility::str(zone1) : "UPS"));
     zone = zone1;
     northp = northp1;
     x = x1;
@@ -120,16 +125,17 @@ namespace GeographicLib {
                        real& lat, real& lon, real& gamma, real& k,
                        bool mgrslimits) {
     if (zone == INVALID || Math::isnan(x) || Math::isnan(y)) {
-      lat = lon = gamma = k = Math::NaN();
+      lat = lon = gamma = k = Math::NaN<real>();
       return;
     }
-    if (! (zone >= MINZONE && zone <= MAXZONE))
-      throw GeographicErr("Zone " + str(zone) + " not in range [0, 60]");
+    if (!(zone >= MINZONE && zone <= MAXZONE))
+      throw GeographicErr("Zone " + Utility::str(zone)
+                          + " not in range [0, 60]");
     bool utmp = zone != UPS;
     CheckCoords(utmp, northp, x, y, mgrslimits);
     int ind = (utmp ? 2 : 0) + (northp ? 1 : 0);
-    x -= falseeasting[ind];
-    y -= falsenorthing[ind];
+    x -= falseeasting_[ind];
+    y -= falsenorthing_[ind];
     if (utmp)
       TransverseMercator::UTM.Reverse(CentralMeridian(zone),
                                       x, y, lat, lon, gamma, k);
@@ -139,34 +145,40 @@ namespace GeographicLib {
 
   void UTMUPS::CheckLatLon(real lat, real lon) {
     if (lat < -90 || lat > 90)
-      throw GeographicErr("Latitude " + str(lat) + "d not in [-90d, 90d]");
+      throw GeographicErr("Latitude " + Utility::str(lat)
+                          + "d not in [-90d, 90d]");
     if (lon < -180 || lon > 360)
-      throw GeographicErr("Latitude " + str(lon) + "d not in [-180d, 360d]");
+      throw GeographicErr("Latitude " + Utility::str(lon)
+                          + "d not in [-180d, 360d]");
     }
 
   bool UTMUPS::CheckCoords(bool utmp, bool northp, real x, real y,
                            bool mgrslimits, bool throwp) {
     // Limits are all multiples of 100km and are all closed on the both ends.
     // Failure tests are such that NaNs succeed.
-    real slop = mgrslimits ? 0 : MGRS::tile;
+    real slop = mgrslimits ? 0 : MGRS::tile_;
     int ind = (utmp ? 2 : 0) + (northp ? 1 : 0);
-    if (x < mineasting[ind] - slop || x > maxeasting[ind] + slop) {
+    if (x < mineasting_[ind] - slop || x > maxeasting_[ind] + slop) {
       if (!throwp) return false;
-      throw GeographicErr("Easting " + str(x/1000) + "km not in "
+      throw GeographicErr("Easting " + Utility::str(x/1000) + "km not in "
                           + (mgrslimits ? "MGRS/" : "")
                           + (utmp ? "UTM" : "UPS") + " range for "
                           + (northp ? "N" : "S" ) + " hemisphere ["
-                          + str((mineasting[ind] - slop)/1000) + "km, "
-                          + str((maxeasting[ind] + slop)/1000) + "km]");
+                          + Utility::str((mineasting_[ind] - slop)/1000)
+                          + "km, "
+                          + Utility::str((maxeasting_[ind] + slop)/1000)
+                          + "km]");
     }
-    if (y < minnorthing[ind] - slop || y > maxnorthing[ind] + slop) {
+    if (y < minnorthing_[ind] - slop || y > maxnorthing_[ind] + slop) {
       if (!throwp) return false;
-      throw GeographicErr("Northing " + str(y/1000) + "km not in "
+      throw GeographicErr("Northing " + Utility::str(y/1000) + "km not in "
                           + (mgrslimits ? "MGRS/" : "")
                           + (utmp ? "UTM" : "UPS") + " range for "
                           + (northp ? "N" : "S" ) + " hemisphere ["
-                          + str((minnorthing[ind] - slop)/1000) + "km, "
-                          + str((maxnorthing[ind] + slop)/1000) + "km]");
+                          + Utility::str((minnorthing_[ind] - slop)/1000)
+                          + "km, "
+                          + Utility::str((maxnorthing_[ind] + slop)/1000)
+                          + "km]");
     }
     return true;
   }
@@ -208,7 +220,8 @@ namespace GeographicLib {
         throw GeographicErr("Illegal zone 0 in " + zonestr +
                             ", use just " + hemi + " for UPS");
       if (!(zone1 >= MINUTMZONE && zone1 <= MAXUTMZONE))
-        throw GeographicErr("Zone " + str(zone1) + " not in range [1, 60]");
+        throw GeographicErr("Zone " + Utility::str(zone1)
+                            + " not in range [1, 60]");
       zone = zone1;
     }
     northp = northp1;
@@ -217,8 +230,9 @@ namespace GeographicLib {
   std::string UTMUPS::EncodeZone(int zone, bool northp) {
     if (zone == INVALID)
       return string("INV");
-    if (! (zone >= MINZONE && zone <= MAXZONE))
-        throw GeographicErr("Zone " + str(zone) + " not in range [0, 60]");
+    if (!(zone >= MINZONE && zone <= MAXZONE))
+        throw GeographicErr("Zone " + Utility::str(zone)
+                            + " not in range [0, 60]");
     ostringstream os;
     if (zone != UPS)
       os << setfill('0') << setw(2) << zone;
@@ -226,6 +240,6 @@ namespace GeographicLib {
     return os.str();
   }
 
-  Math::real UTMUPS::UTMShift() throw() { return real(MGRS::utmNshift); }
+  Math::real UTMUPS::UTMShift() throw() { return real(MGRS::utmNshift_); }
 
 } // namespace GeographicLib
